-------------------------------------------------------------------------------
-- Title      : MainStage
-- Project    : Chip-Module for first submission on 28.04.2000
-------------------------------------------------------------------------------
-- File       : MainStage.vhd
-- Author     : Rolf Schneider  <rolf.schneider@kip.uni-heidelberg.de>
-- Company    : University Heidelberg - KIP
-- Last update: 2000/04/21
-- Platform   : Synopsys - 1999.10
-------------------------------------------------------------------------------
-- This is a first Chip-Module design test for the submission on 28.02.2000
-- to get experience with the AMS-0.35u technologie
-------------------------------------------------------------------------------
-- Description: main stage of top level design (without de/-serializer)
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author    Description
-- 2000/03/03  1.0      schneide  Created
-- 2000/04/03  1.1      schneide  Reduced to Low Latency Design
-------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use WORK.Global.all;

-------------------------------------------------------------------------------
-- ENTITY
-------------------------------------------------------------------------------
entity MainStage is
  port(data_in_mem:                     in  Event_Data_Word;
       data_in:                         in  Hamming_Code;
       error_count:                     in  Error_Counter_Word;
       mux_ext_int_select:              in  STD_LOGIC_VECTOR(1 downto 0);
       mux_code_sig_select:             in  STD_LOGIC;
       mux_enc_noenc_select:            in  STD_LOGIC;
       mux_end_noend_select:            in  STD_LOGIC;
       hamming_dec_status:              out Hamming_Status;
       data_out:                        out Hamming_Code);
end MainStage;

-------------------------------------------------------------------------------
-- ARCHITECTURE
-------------------------------------------------------------------------------
architecture v of MainStage is

  -- SIGNALS ------------------------------------------------------------------
  signal static_low_ebw:                External_Bus;
  signal mux_end_noend_output:          External_Bus;
  signal hamming_dec_output:            Event_Data_Word;
  signal hamming_enc_output:            External_Bus;
  signal mux_ext_int_output:            Event_Data_Word;
  signal data_in_mem_ext:               External_Bus;
  signal error_count_ext:               Event_Data_Word;
  signal end_signature_code_signal:     External_Bus;
  signal start_signature_word_signal:   Event_Data_Word;
  signal mux_code_sig_output:           External_Bus;
  
  -- COMPONENTS ---------------------------------------------------------------
  component Multiplexer_4_edww
    port (
      in_data1:         in  Event_Data_Word;
      in_data2:         in  Event_Data_Word;
      in_data3:         in  Event_Data_Word;
      in_data4:         in  Event_Data_Word;
      sel:              in  STD_LOGIC_VECTOR(1 downto 0);
      out_data:         out Event_Data_Word);
  end component;
  
  component Multiplexer_ebw
    port (
      in_data1:         in  External_Bus;
      in_data2:         in  External_Bus;
      sel:              in  STD_LOGIC;
      out_data:         out External_Bus);
  end component;
  
  component Hamming_Encoder
    port (
      data:             in  Event_Data_Word;
      code:             out Hamming_Code);
  end component;

  component Hamming_Decoder
    port (
      code:             in  Hamming_Code;
      data:             out Event_Data_Word;
      status:           out Hamming_Status);
  end component;

  -- STRUCTURE ----------------------------------------------------------------
begin

  static_low_ebw        <= (others => '0');
  
  mux_end_noend: Multiplexer_ebw
    port map (
      in_data1          => static_low_ebw,              -- if '0'
      in_data2          => data_in,                     -- if '1'
      sel               => mux_end_noend_select,
      out_data          => mux_end_noend_output);

  hamming_dec: Hamming_Decoder
    port map (
      code              => mux_end_noend_output,
      data              => hamming_dec_output,
      status            => hamming_dec_status);

  start_signature_word_signal <= start_signature_word;
  error_count_ext       <= "00" & error_count;
  
  mux_ext_int: Multiplexer_4_edww
    port map (
      in_data1          => hamming_dec_output,          -- if "00"
      in_data2          => data_in_mem,                 -- if "01"
      in_data3          => start_signature_word_signal, -- if "10"
      in_data4          => error_count_ext,             -- if "11";
      sel               => mux_ext_int_select,
      out_data          => mux_ext_int_output);

  hamming_enc: Hamming_Encoder
    port map (
      data              => mux_ext_int_output,
      code              => hamming_enc_output);

  end_signature_code_signal <= end_signature_code;
  
  mux_code_sig: Multiplexer_ebw
    port map (
      in_data1          => hamming_enc_output,          -- if '0'
      in_data2          => end_signature_code_signal,   -- if '1'
      sel               => mux_code_sig_select,
      out_data          => mux_code_sig_output);

  data_in_mem_ext       <= "0" & data_in_mem(7 downto 4) &
                           "0" & data_in_mem(3 downto 1) &
                           "0" & data_in_mem(0) & "00";

  mux_enc_noenc: Multiplexer_ebw
    port map (
      in_data1          => mux_code_sig_output,         -- if '0'
      in_data2          => data_in_mem_ext,             -- if '1'
      sel               => mux_enc_noenc_select,
      out_data          => data_out);

end v;
