-------------------------------------------------------------------------------
-- Title      : TopControl
-- Project    : Chip-Module for first submission on 28.04.2000
-------------------------------------------------------------------------------
-- File       : TopControl.vhd
-- Author     : Rolf Schneider  <rolf.schneider@kip.uni-heidelberg.de>
-- Company    : University Heidelberg - KIP
-- Last update: 2000/04/08
-- Platform   : Synopsys - 1999.10
-------------------------------------------------------------------------------
-- This is a first Chip-Module design test for the submission on 28.04.2000
-- to get experience with the AMS-0.35u technologie
-------------------------------------------------------------------------------
-- Description: Top Control Unit for Chip Module (for MainStage data path)
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author  Description
-- 2000/04/01  1.0      schneide	Created
-------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use WORK.Global.all;

-------------------------------------------------------------------------------
-- ENTITY
-------------------------------------------------------------------------------
entity TopControl is
  port (
    clk:                        in  STD_LOGIC;
    reset:                      in  STD_LOGIC;
    data_in:                    in  External_Bus;
    hamm_st:                    in  Hamming_Status;
    start_rd:                   in  STD_LOGIC;
    start_mem:                  out STD_LOGIC;
    end_mem:                    in  STD_LOGIC;
    hard_end_signature:         in  STD_LOGIC;
    error_count:                out Error_Counter_Word;
    mux_ext_int_select:         out STD_LOGIC_VECTOR(1 downto 0);
    mux_code_sig_select:        out STD_LOGIC);
end TopControl;

-------------------------------------------------------------------------------
-- ARCHITECTURE
-------------------------------------------------------------------------------
architecture v of TopControl is

  -- SIGNALS ------------------------------------------------------------------
  signal end_mem_la:            STD_LOGIC;
  signal start_mem_la:          STD_LOGIC;
  signal soft_end_signature:    STD_LOGIC;
  signal start_mem_fsm:         STD_LOGIC;
  signal start_mem_asyn:        STD_LOGIC;
  signal error_counter_enable:  STD_LOGIC;
  signal error_counter_clear:   STD_LOGIC;
  signal mux_ext_int_select_fsm:STD_LOGIC_VECTOR(1 downto 0);

  signal tc_state:              STD_LOGIC_VECTOR(2 downto 0);
  signal tc_next_state:         STD_LOGIC_VECTOR(2 downto 0);

  -- CONSTANTS ----------------------------------------------------------------
  constant reset_st:            STD_LOGIC_VECTOR(2 downto 0) := "000";
  constant wait_st:             STD_LOGIC_VECTOR(2 downto 0) := "001";
  constant start_internal_st:   STD_LOGIC_VECTOR(2 downto 0) := "010";
  constant output_data_st:      STD_LOGIC_VECTOR(2 downto 0) := "011";
  constant output_end_sig_st:   STD_LOGIC_VECTOR(2 downto 0) := "100";
  constant start_external_st:   STD_LOGIC_VECTOR(2 downto 0) := "101";
  constant switch_internal_st:  STD_LOGIC_VECTOR(2 downto 0) := "110";
  
  -- COMPONENTS ---------------------------------------------------------------
  component GenEndSignature
    port (
      code:                     in  External_Bus;
      end_signature:            out STD_LOGIC);
  end component;

  component ErrorCounter
    port (
      clk:                      in  STD_LOGIC;
      reset:                    in  STD_LOGIC;
      enable:                   in  STD_LOGIC;
      clear:                    in  STD_LOGIC;
      hamm_st:                  in  Hamming_Status;
      data_out:                 out Error_Counter_Word);
  end component;

  -- STRUCTURE ----------------------------------------------------------------
begin

  end_mem_la                    <= not end_mem;
  start_mem                     <= not start_mem_la;
  
  -- end signature ------------------------------------------------------------
  gen_end_sig: GenEndSignature
    port map (
        code                    => data_in,
        end_signature           => soft_end_signature);

  -- asynchron start_mem_la and mux_ext_int_select -----------------------------
  start_mem_asyn                <= soft_end_signature;
  start_mem_la                  <= start_mem_fsm and start_mem_asyn;

  mux_ext_int_select            <=   (mux_ext_int_select_fsm(1)
                                       or (not soft_end_signature))
                                   & (mux_ext_int_select_fsm(0)
                                       or (not soft_end_signature));

  -- error counter ------------------------------------------------------------
  error_counter_enable          <= not soft_end_signature;
  
  error_cnt: ErrorCounter
    port map (
      clk                       => clk,
      reset                     => reset,
      enable                    => error_counter_enable,
      clear                     => error_counter_clear,
      hamm_st                   => hamm_st,
      data_out                  => error_count);

  error_cnt_ctrl: process(clk, reset, start_rd, data_in)
    begin
      if clk'event and clk='1' then
        if start_rd='0' then
          error_counter_clear <= '0';
        elsif data_in/="0000000000000" then
          error_counter_clear <= '1';
        end if;
      end if;
      if reset='0' then
        error_counter_clear <= '0';
      end if;
    end process;
  
  -- top control state machine ------------------------------------------------
  tc_sm_mem: process(clk, reset, tc_next_state)                    -- memory --
    begin
      if reset='0' then
        tc_state <= reset_st;
      elsif clk'event and clk='1' then
        tc_state <= tc_next_state;
      end if;
    end process;

                                                                   -- states  --
  tc_sm: process(tc_state, start_rd, hard_end_signature, soft_end_signature, end_mem_la)
    begin
      case tc_state is

        -- reset_state --
        when reset_st =>
          tc_next_state <= wait_st;

        -- wait_state --
        when wait_st =>
          if (start_rd='0' and hard_end_signature='0') then
            tc_next_state <= start_internal_st;
          elsif (start_rd='0' and hard_end_signature='1') then
            tc_next_state <= start_external_st;
          else
            tc_next_state <= wait_st;
          end if;

        -- start_internal_state --
        when start_internal_st => 
          tc_next_state <= output_data_st;

        -- output_data_state --
        when output_data_st =>
          if (end_mem_la='0') then
            tc_next_state <= output_end_sig_st;
          else
            tc_next_state <= output_data_st;
          end if;

        -- output_end_sig_state --
        when output_end_sig_st => 
          tc_next_state <= wait_st;

        -- start_external_state --
        when start_external_st =>
          if (soft_end_signature='0') then
            tc_next_state <= switch_internal_st;
          else
            tc_next_state <= start_external_st;
          end if;

        -- switch_internal_state --
        when switch_internal_st => 
          tc_next_state <= output_data_st;

        when others => tc_next_state <= reset_st;
      end case;
    end process;

  tc_sm_output: process(tc_state)                                  -- output  --
    begin
      case tc_state is
        when reset_st =>
          start_mem_fsm                 <= '1';
          mux_ext_int_select_fsm        <= "00";
          mux_code_sig_select           <= '0';
        when wait_st =>
          start_mem_fsm                 <= '1';
          mux_ext_int_select_fsm        <= "00";
          mux_code_sig_select           <= '0';
        when start_internal_st =>
          start_mem_fsm                 <= '0';
          mux_ext_int_select_fsm        <= "10";
          mux_code_sig_select           <= '0';
        when output_data_st =>
          start_mem_fsm                 <= '1';
          mux_ext_int_select_fsm        <= "01";
          mux_code_sig_select           <= '0';
        when output_end_sig_st =>
          start_mem_fsm                 <= '1';
          mux_ext_int_select_fsm        <= "00";
          mux_code_sig_select           <= '1';
        when start_external_st =>
          start_mem_fsm                 <= '1';
          mux_ext_int_select_fsm        <= "00";
          mux_code_sig_select           <= '0';
        when switch_internal_st =>
          start_mem_fsm                 <= '1';
          mux_ext_int_select_fsm        <= "01";
          mux_code_sig_select           <= '0';
        when others => 
          start_mem_fsm                 <= '1';
          mux_ext_int_select_fsm        <= "00";
          mux_code_sig_select           <= '0';
      end case;
    end process;
    
end v;


