-------------------------------------------------------------------------------
-- Title      : TopTree
-- Project    : Chip-Module for first submission on 28.04.2000
-------------------------------------------------------------------------------
-- File       : TopTree.vhd
-- Author     : Rolf Schneider  <rolf.schneider@kip.uni-heidelberg.de>
-- Company    : University Heidelberg - KIP
-- Last update: 2000/04/08
-- Platform   : Synopsys - 1999.10
-------------------------------------------------------------------------------
-- This is a first Chip-Module design test for the submission on 28.04.2000
-- to get experience with the AMS-0.35u technologie
-------------------------------------------------------------------------------
-- Description: 
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author    Description
-- 2000/02/26  1.0      schneide  Created
-- 2000/03/30  1.1      schneide  Reduced to Low Latency Design
-------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use WORK.Global.all;

-------------------------------------------------------------------------------
-- ENTITY
-------------------------------------------------------------------------------
entity TopTree is
  port (
    clk:                        in  STD_LOGIC;     -- readout clock, independent
                                                   -- to system clock
    reset:                      in  STD_LOGIC;
    end_sig:                    in  STD_LOGIC;     -- first chip in the queue (la)
    start_rd:                   in  STD_LOGIC;     -- signals the readout start (la)
    start_mem:                  out STD_LOGIC;     -- start reading internal data
    end_mem:                    in  STD_LOGIC;     -- last internal data was sent
    readout_mode:               in  STD_LOGIC;     -- with/without hamming
    hamm_st:                    out Hamming_Status;-- hamming decoder status word
    data_in_mem:                in  Event_Data_Word; -- internal data
    data_in:                    in  External_Bus;  -- external data
    data_out:                   out External_Bus); -- data output
end TopTree;

-------------------------------------------------------------------------------
-- ARCHITECTURE
-------------------------------------------------------------------------------
architecture v of TopTree is

  -- SIGNALS ------------------------------------------------------------------
  signal mux_ext_int_select:    STD_LOGIC_VECTOR(1 downto 0);
  signal mux_code_sig_select:   STD_LOGIC;

  signal data_in_reg:           External_Bus;
  signal data_out_reg:          External_Bus;
  signal hamm_st_reg:           Hamming_Status;
  signal error_count:           Error_Counter_Word;
    
  -- COMPONENTS ---------------------------------------------------------------
  component TopControl
    port (
      clk:                      in  STD_LOGIC;
      reset:                    in  STD_LOGIC;
      data_in:                  in  External_Bus;
      hamm_st:                  in  Hamming_Status;
      start_rd:                 in  STD_LOGIC;
      start_mem:                out STD_LOGIC;
      end_mem:                  in  STD_LOGIC;
      hard_end_signature:       in  STD_LOGIC;
      error_count:              out Error_Counter_Word;
      mux_ext_int_select:       out STD_LOGIC_VECTOR(1 downto 0);
      mux_code_sig_select:      out STD_LOGIC);
  end component;
  
  component MainStage
    port (
      data_in_mem:              in  Event_Data_Word;
      data_in:                  in  Hamming_Code;
      error_count:              in  Error_Counter_Word;
      mux_ext_int_select:       in  STD_LOGIC_VECTOR(1 downto 0);
      mux_code_sig_select:      in  STD_LOGIC;
      mux_enc_noenc_select:     in  STD_LOGIC;
      mux_end_noend_select:     in  STD_LOGIC;
      hamming_dec_status:       out Hamming_Status;
      data_out:                 out Hamming_Code);
  end component;
  
  component Register_ebw is
    port (
      clk:                      in  STD_LOGIC;
      reset:                    in  STD_LOGIC;
      in_data:                  in  External_Bus;
      out_data:                 out External_Bus);
  end component;

  component Register_hsw is
    port (
      clk:                      in  STD_LOGIC;
      reset:                    in  STD_LOGIC;
      in_data:                  in  Hamming_Status;
      out_data:                 out Hamming_Status);
  end component;

  component Multiplexer_edww
    port (
      in_data1:                 in  Event_Data_Word;
      in_data2:                 in  Event_Data_Word;
      sel:                      in  STD_LOGIC;
      out_data:                 out Event_Data_Word);
  end component;
  
  -- STRUCTURE ----------------------------------------------------------------
begin

  -- top control unit
  top_ctrl: TopControl
    port map (
      clk                       => clk,
      reset                     => reset,
      data_in                   => data_in_reg,
      hamm_st                   => hamm_st_reg,
      start_rd                  => start_rd,
      start_mem                 => start_mem,
      end_mem                   => end_mem,
      hard_end_signature        => end_sig,
      error_count               => error_count,
      mux_ext_int_select        => mux_ext_int_select,
      mux_code_sig_select       => mux_code_sig_select);
  
  -- external bus input register
  input_reg_ext1: Register_ebw
    port map (
      clk                       => clk,
      reset                     => reset,
      in_data                   => data_in,
      out_data                  => data_in_reg);

  -- main stage
  main_stage: MainStage
    port map (
      data_in_mem               => data_in_mem,
      data_in                   => data_in_reg,
      error_count               => error_count,
      mux_ext_int_select        => mux_ext_int_select,
      mux_code_sig_select       => mux_code_sig_select,
      mux_enc_noenc_select      => readout_mode,
      mux_end_noend_select      => end_sig,
      hamming_dec_status        => hamm_st_reg,
      data_out                  => data_out_reg);

  -- external bus output register
  ob_reg: Register_ebw
    port map (
      clk                       => clk,
      reset                     => reset,
      in_data                   => data_out_reg,
      out_data                  => data_out);
  
  -- external bus output register
  hst_reg: Register_hsw
    port map (
      clk                       => clk,
      reset                     => reset,
      in_data                   => hamm_st_reg,
      out_data                  => hamm_st);
  
end v;

