/**************************************************************************
 * Copyright(c) 1998-1999, ALICE Experiment at CERN, All rights reserved. *
 *                                                                        *
 * Author: The ALICE Off-line Project.                                    *
 * Contributors are mentioned in the code where appropriate.              *
 *                                                                        *
 * Permission to use, copy, modify and distribute this software and its   *
 * documentation strictly for non-commercial purposes is hereby granted   *
 * without fee, provided that the above copyright notice appears in all   *
 * copies and that both the copyright notice and this permission notice   *
 * appear in the supporting documentation. The authors make no claims     *
 * about the suitability of this software for any purpose. It is          *
 * provided "as is" without express or implied warranty.                  *
 **************************************************************************/

/*
$Log: AliTRDgtuBlock.cxx,v $
Revision 1.1.1.1  2001/01/16 18:02:33  schneide
Import of TRDgtu

*/

///////////////////////////////////////////////////////////////////////////
//                                                                       //
//  Global Tracking Unit - Block class                                   //
//  This class handles data and output for one block of the detector     //
//  One block consists of six chambers, ordered on the top of each other //
//                                                                       //
//  module: (TRDgtu) Global Tracking Unit        author: Rolf Schneider  //
//                                                                       //
///////////////////////////////////////////////////////////////////////////

#include "AliTRDgtuConst.h"
#include "AliTRDgtuGlobal.h"
#include "AliTRDgtuLink.h"
#include "AliTRDgtuTMU.h"
#include "AliTRDgtuVisual.h"
#include "AliTRDgtuBlock.h"

ClassImp(AliTRDgtuBlock)

//_________________________________________________________________________
AliTRDgtuBlock::AliTRDgtuBlock():TObject()
{
  //
  //  AliTRDgtuBlock default constructor
  //
  //  init a (chamber-)link for every layer within one block
  //  init a Track Matching Unit (TMU) for every padrow. One TMU works on
  //    one padrow and six layers
  //

  for(Int_t lLayer=0; lLayer<kLayers; lLayer++)
    fLink[lLayer] = new AliTRDgtuLink(lLayer, fSector, fChamber);

  for(Int_t lTMUindex=0; lTMUindex<kPadrowsPerChamber; lTMUindex++) 
    fTMU[lTMUindex] = new AliTRDgtuTMU(fSector, fChamber, lTMUindex);
}

//_________________________________________________________________________
AliTRDgtuBlock::AliTRDgtuBlock(Int_t iSector, Int_t iChamber):TObject()
{
  //
  //  AliTRDgtuBlock default constructor with identification
  //

  fSector  = iSector;
  fChamber = iChamber;

  for(Int_t lLayer=0; lLayer<kLayers; lLayer++)
    fLink[lLayer] = new AliTRDgtuLink(lLayer, fSector, fChamber);

  for(Int_t lTMUindex=0; lTMUindex<kPadrowsPerChamber; lTMUindex++) 
    fTMU[lTMUindex] = new AliTRDgtuTMU(fSector, fChamber, lTMUindex);
}

//_________________________________________________________________________
AliTRDgtuBlock::~AliTRDgtuBlock()
{
  //
  //  AliTRDgtuBlock default destructor
  //
  //  free memory
  //

  for(Int_t lLayer=0; lLayer<kLayers; lLayer++)
    delete fLink[lLayer];

  for(Int_t lTMUindex=0; lTMUindex<kPadrowsPerChamber; lTMUindex++)
    delete fTMU[lTMUindex];
}

//_________________________________________________________________________
void AliTRDgtuBlock::ReadASCII(Int_t iLayer, char *iFilename)
{
  //
  //  read a ASCII input file for one chamber within the current block
  //
  //  file format:
  //    The first lines can be comment lines. The comment delimiter is ";".
  //    Every following line represents a candidate. The parameters are
  //    seperated by a tab and ordered: RPos, Angle, ZPos, Amplitude, 
  //    TrQuality, RPosMidlayer.
  //
  //  example:
  //
  //  ;RPos   Angle   ZPos    Ampl.   TrQual. RPosMid
  //  599     24      10      80      1       -1
  //  1500    15      10      20      1       -1
  //

  fLink[iLayer]->ReadASCII(iFilename);
}

//_________________________________________________________________________
void AliTRDgtuBlock::AddCandidate(Int_t iLayer, AliTRDgtuCandidate *iCand)
{
  //
  //  add a candidate to the Link of a specified Layer
  //

  fLink[iLayer]->AddCandidate(iCand);
}
//_________________________________________________________________________
void AliTRDgtuBlock::CalcMidlayer()
{
  //
  //  calculate the middle layer positions for the whole block
  //  the position is calculated by RPos, Angle and Layer
  //

  for(Int_t lLayer=0; lLayer<kLayers; lLayer++) {
    DBG_I ( printf("calculate MidLayer positions for detector (%d, %d, %d, -)\n", lLayer, fSector, fChamber); )
    fLink[lLayer]->CalcMidlayer(lLayer);
  }
}

//_________________________________________________________________________
void AliTRDgtuBlock::InitTMUs()
{
  //
  //  initialize the Track Matching Units (TMUs) for the current Block and
  //  the data input.
  //
  //  Every TMU holds a candidate array for every layer with candidates only
  //  for the padrow, specific for the TMU
  //

  TObjArray *lLinkDataObj = NULL;
  AliTRDgtuCandidate *lCandidate = NULL;
  Int_t lPadrow;
  Int_t lIndex = 0;

  DBG_I ( printf("initialize TMUs for Block (-, %d, %d, -)\n", fSector, fChamber); )

  for(Int_t lLayer=0; lLayer<kLayers; lLayer++) {
    lLinkDataObj = fLink[lLayer]->GetLinkDataObj();
    lIndex = 0;
    if(lLinkDataObj->GetLast()+1) {
      do {
        lCandidate = (AliTRDgtuCandidate *)(lLinkDataObj->At(lIndex));
        lPadrow = lCandidate->GetZPos();
        fTMU[lPadrow]->AddCandidate(lCandidate, lLayer);
        lIndex++;
      } while (lIndex <= lLinkDataObj->GetLast());
    }
  }
}

//_________________________________________________________________________
void AliTRDgtuBlock::Clear()
{
  //
  //  clear the candidates for the current Block
  //

  for(Int_t lLayer=0; lLayer<kLayers; lLayer++) {
    DBG_A ( printf("clear link data of detector (%d, %d, %d, -)\n", lLayer, fSector, fChamber); )
    fLink[lLayer]->Clear();
  }
}

//_________________________________________________________________________
void AliTRDgtuBlock::OutputBlockData()
{
  //
  //  output candidates for the current Block, ordered by chamber links
  //

  HEAD ( printf("Link Candidates of Block (-, %d, %d, -)\n", fSector, fChamber); )
  for(Int_t lLayer=0; lLayer<kLayers; lLayer++) {
    DBG_N ( printf("\n"); )
    DBG_N ( printf("**** Layer %d:\n", lLayer); )
    DBG_N ( printf("\n"); )
      fLink[lLayer]->Output();
  }
  DBG_N ( printf("\n"); )
}

//_________________________________________________________________________
void AliTRDgtuBlock::OutputTMUs()
{
  //
  //  output candidates for the current Block, ordered by TMUs
  //

  HEAD ( printf("Track Matching Candidates of Block (-, %d, %d, -)\n", fSector, fChamber); )
  for(Int_t lTMUindex=0; lTMUindex<kPadrowsPerChamber; lTMUindex++) {
    DBG_N ( printf("**** Padrow %02d: ", lTMUindex); )
    if(fTMU[lTMUindex]->NumberOfCandidates()) {
      printf("\n");
      fTMU[lTMUindex]->OutputCandidates();
    } else
      printf("-- no candidates --\n");
  }
  DBG_N ( printf("\n"); )
}

//_________________________________________________________________________
void AliTRDgtuBlock::FindTracks(Int_t iPadrow)
{
  //
  //  find tracks within one Track Matching Unit (eq. one Padrow)
  //

  DBG_I ( printf("**** Find Track in TMU (-, %d, %d, %d)\n", fSector, fChamber, iPadrow); )
  fTMU[iPadrow]->FindTracksV0();
}

//_________________________________________________________________________
void AliTRDgtuBlock::OutputTracks(Int_t iPadrow)
{
  //
  //  output tracks within one Track Matching Unit (eq. one Padrow)
  //

  fTMU[iPadrow]->OutputTracks();
}

//_________________________________________________________________________
void AliTRDgtuBlock::VisualTMUcandidates2D(AliTRDgtuVisual *iVisual, Int_t iPadrow)
{
  //
  //  give an 2D visualisation of candidates for the specified padrow
  //

  fTMU[iPadrow]->VisualCandidates2D(iVisual);
}

//_________________________________________________________________________
void AliTRDgtuBlock::VisualTMUtracks2D(AliTRDgtuVisual *iVisual, Int_t iPadrow)
{
  //
  //  give an 2D visualisation of candidates for the specified padrow
  //

  fTMU[iPadrow]->VisualTracks2D(iVisual);
}

//_________________________________________________________________________
void AliTRDgtuBlock::VisualBlockCandidates3D(AliTRDgtuVisual *iVisual)
{
  //
  //  give an 3D visualisation of candidates for whole block
  //

  TObjArray *lLinkDataObj;
  Int_t lIndex;

  for(Int_t lLayer=0; lLayer<kLayers; lLayer++) {
    lLinkDataObj = fLink[lLayer]->GetLinkDataObj();
    lIndex = 0;
    if(lLinkDataObj->At(0))
      do {
        iVisual->DrawCandidate3D((AliTRDgtuCandidate *)lLinkDataObj->At(lIndex), lLayer);
      lIndex++;
    } while (lIndex <= lLinkDataObj->GetLast());
  }
}

