/**************************************************************************
 * Copyright(c) 1998-1999, ALICE Experiment at CERN, All rights reserved. *
 *                                                                        *
 * Author: The ALICE Off-line Project.                                    *
 * Contributors are mentioned in the code where appropriate.              *
 *                                                                        *
 * Permission to use, copy, modify and distribute this software and its   *
 * documentation strictly for non-commercial purposes is hereby granted   *
 * without fee, provided that the above copyright notice appears in all   *
 * copies and that both the copyright notice and this permission notice   *
 * appear in the supporting documentation. The authors make no claims     *
 * about the suitability of this software for any purpose. It is          *
 * provided "as is" without express or implied warranty.                  *
 **************************************************************************/

/*
$Log: AliTRDgtuCandidate.cxx,v $
Revision 1.1.1.1  2001/01/16 18:02:33  schneide
Import of TRDgtu

*/

///////////////////////////////////////////////////////////////////////////
//                                                                       //
//  Global Tracking Unit - Candidate class                               //
//  This class represents one candidate which is produced by             //
//  Falk Lessers linear fit engine                                       //
//                                                                       //
//  module: (TRDgtu) Global Tracking Unit        author: Rolf Schneider  //
//                                                                       //
///////////////////////////////////////////////////////////////////////////

#include "AliTRDgtuConst.h"
#include "AliTRDgtuGlobal.h"
#include "AliTRDgtuCandidate.h"

ClassImp(AliTRDgtuCandidate)
 
//_________________________________________________________________________
AliTRDgtuCandidate::AliTRDgtuCandidate():TObject()
{
  //
  //  AliTRDgtuCandidate default constructor
  //
  //  initialize candidate with values "undefined"
  //

  fRPos         = XXX;
  fAngle        = XXX;
  fZPos         = XXX;
  fAmplitude    = XXX;
  fTrQuality    = XXX;
  fRPosMidlayer = XXX;

  for(Int_t lNo = 0; lNo < kParticlesPerCandidate; lNo ++)
    fParticle[lNo] = XXX;

}

//_________________________________________________________________________
AliTRDgtuCandidate::~AliTRDgtuCandidate()
{
  //
  //  AliTRDgtuCandidate default destructor
  //

}

//_________________________________________________________________________
void AliTRDgtuCandidate::AddParticle(Int_t iParticle)
{
  //
  //  add a global particle number to the candidate
  //

  Int_t lNo;
  for(lNo = 0; (lNo < kParticlesPerCandidate) && (fParticle[lNo] != XXX); lNo ++);

  if(lNo == kParticlesPerCandidate)
    DBG_W ( printf("WARNING: max. Number of Particles per Candidate is %d\n", kParticlesPerCandidate); )
  else
    fParticle[lNo] = iParticle;
}

//_________________________________________________________________________
void AliTRDgtuCandidate::AddParticle(Int_t iParticle, Float_t iX1, Float_t iY1, Float_t iX2, Float_t iY2)
{
  //
  //  add a global particle number and position to the candidate
  //

  Int_t lNo;
  for(lNo = 0; (lNo < kParticlesPerCandidate) && (fParticle[lNo] != XXX); lNo ++);

  if(lNo == kParticlesPerCandidate)
    DBG_W ( printf("WARNING: max. Number of Particles per Candidate is %d\n", kParticlesPerCandidate); )
  else {
    fParticle[lNo] = iParticle;
    SetParticlePos(lNo, iX1, iY1, iX2, iY2);
  }
}

//_________________________________________________________________________
void AliTRDgtuCandidate::CalcMidlayer(Int_t iLayer)
{
  //
  //  calculate middle layer position out of position (RPos) and layer
  //

  Int_t lAngle = fAngle - ( kAngleMax >> 1 );
  fRPosMidlayer = ( fRPos << 2 ) + lAngle;

  fRPosMidlayer = RangeTest("RPosMidlayer", fRPosMidlayer, 0, kRPosMidlayerMax);
  DBG_A ( printf("set RPosMidlayer\t= %d\n", fRPosMidlayer); )
}

//_________________________________________________________________________
void AliTRDgtuCandidate::GetParticlePos(Int_t iParticleNo, Float_t &iX1, Float_t &iY1, Float_t &iX2, Float_t &iY2)
{
  //
  //  get the start and end position of specified particle of this candidate
  //

  if(iParticleNo >= kParticlesPerCandidate) {
    DBG_W ( printf("WARNING: cannot access %d. particle of current candidate\n", iParticleNo); )
    return;
  }

  Int_t lOffset = iParticleNo * 4;
  iX1 = fParticlePos[lOffset  ];
  iY1 = fParticlePos[lOffset+1];
  iX2 = fParticlePos[lOffset+2];
  iY2 = fParticlePos[lOffset+3];
}  

//_________________________________________________________________________
Bool_t AliTRDgtuCandidate::IsValid()
{
  //
  //  checks if no value of candidate is undefined
  //

  if((fRPos == XXX) || (fAngle == XXX) || (fZPos == XXX)
      || (fAmplitude == XXX) || (fTrQuality == XXX) || (fRPosMidlayer == XXX))
    return(FALSE);
  else
    return(TRUE);
}

//_________________________________________________________________________
Int_t AliTRDgtuCandidate::NumberOfParticles()
{
  //
  //  return the number of involved particles for this candidate
  //

  Int_t lNo;
  for(lNo = 0; (lNo < kParticlesPerCandidate) && (fParticle[lNo] != XXX); lNo ++);
  return(lNo);
}

//_________________________________________________________________________
void AliTRDgtuCandidate::Output()
{
  //
  //  output candidate incl. parameter header
  //

  DBG_N ( printf("\tRPos\tAngle\tZPos\tAmpl.\tTrQ\tRPosMid\n"); )
  DBG_N ( printf("\t%d\t%d\t%d\t%d\t%d\t%d\n", fRPos, fAngle, fZPos,
      fAmplitude, fTrQuality, fRPosMidlayer); )
}

//_________________________________________________________________________
void AliTRDgtuCandidate::SetAmplitude(Int_t iAmplitude)
{
  //
  //  set the ave. amplitude of candidate
  //
  //  kAmplitudeBits defines the number of bits used for Amplitude ("AliTRDgtuConst.h")
  //  kAmplitudeMax defines the maximum value Amplitude ("AliTRDgtuConst.h")
  //

  fAmplitude = RangeTest("Amplitude", iAmplitude, 0, kAmplitudeMax);
  DBG_A ( printf("set Amplitude\t= %d\n", fAmplitude); )
}

//_________________________________________________________________________
void AliTRDgtuCandidate::SetAngle(Int_t iAngle)
{
  //
  //  set the angle of candidate
  //
  //  kAngleBits defines the number of bits used for Angle ("AliTRDgtuConst.h")
  //  kAngleMax defines the maximum value Angle ("AliTRDgtuConst.h")
  //

  fAngle = RangeTest("Angle", iAngle, 0, kAngleMax);
  DBG_A ( printf("set Angle\t\t= %d\n", fAngle); )
}

//_________________________________________________________________________
void AliTRDgtuCandidate::SetParticlePos(Int_t iParticleNo, Float_t iX1, Float_t iY1, Float_t iX2, Float_t iY2)
{
  //
  //  set the start and endposition of specified particle (track)
  //

  if(iParticleNo >= kParticlesPerCandidate) {
    DBG_W ( printf("WARNING: cannot access %d. particle of current candidate\n", iParticleNo); )
    return;
  }

  Int_t lOffset = iParticleNo * 4;
  fParticlePos[lOffset  ] = iX1;
  fParticlePos[lOffset+1] = iY1;
  fParticlePos[lOffset+2] = iX2;
  fParticlePos[lOffset+3] = iY2;

}

//_________________________________________________________________________
void AliTRDgtuCandidate::SetRPos(Int_t iRPos)
{
  //
  //  set the position of candidate in rphi direction 
  //
  //  kRPosBits defines the number of bits used for RPos ("AliTRDgtuConst.h")
  //  kRPosMax defines the maximum value of RPos ("AliTRDgtuConst.h")
  //

  fRPos = RangeTest("RPos", iRPos, 0, kRPosMax);
  DBG_A ( printf("set RPos\t\t= %d\n", fRPos); )
}

//_________________________________________________________________________
void AliTRDgtuCandidate::SetRPosMidlayer(Int_t iRPosMidlayer)
{
  //
  //  set the position of candidate in rphi direction of the middle layer
  //
  //  kRPosMidlayerBits defines the number of bits used for RPosMidlayer ("AliTRDgtuConst.h")
  //  kRPosMidlayerMax defines the maximum value RPosMidlayer ("AliTRDgtuConst.h")
  //

  fRPosMidlayer = RangeTest("RPosMidlayer", iRPosMidlayer, 0, kRPosMidlayerMax);
  DBG_A ( printf("set RPosMidlayer\t= %d\n", fRPosMidlayer); )
}

//_________________________________________________________________________
void AliTRDgtuCandidate::SetTrQuality(Int_t iTrQuality)
{
  //
  //  set the number for transition radiation quality of candidate
  //
  //  kTrQualityBits defines the number of bits used for TrQuality ("AliTRDgtuConst.h")
  //  kTrQualityMax defines the maximum value TrQuality ("AliTRDgtuConst.h")
  //

  fTrQuality = RangeTest("TrQuality", iTrQuality, 0, kTrQualityMax);
  DBG_A ( printf("set TrQuality\t= %d\n", fTrQuality); )
}

//_________________________________________________________________________
void AliTRDgtuCandidate::SetZPos(Int_t iZPos)
{
  //
  //  set the position of candidate in z direction within one chamber
  //  this is equivalent to the padrow
  //
  //  kZPosBits defines the number of bits used for Zpos ("AliTRDgtuConst.h")
  //  kZposMax defines the maximum value Zpos ("AliTRDgtuConst.h")
  //

  fZPos = RangeTest("ZPos", iZPos, 0, kZPosMax);
  DBG_A ( printf("set ZPos\t\t= %d\n", fZPos); )
}

//_________________________________________________________________________
Int_t AliTRDgtuCandidate::RangeTest(char *iVarName, Int_t iValue, Int_t iMin, Int_t iMax)
{
  //
  //  checks if a value is within a given range
  //

  Int_t lValue;

  if(iValue < iMin) {
    lValue = iMin;
    DBG_W ( printf("WARNING: %s cannot be <%d\n", iVarName, iMin); )
  } else if(iValue > iMax) {
    lValue = iMax;
    DBG_W ( printf("WARNING: %s cannot be >%d\n", iVarName, iMax); )
  }
  else
    lValue = iValue;

  return(lValue);
}

