/**************************************************************************
 * Copyright(c) 1998-1999, ALICE Experiment at CERN, All rights reserved. *
 *                                                                        *
 * Author: The ALICE Off-line Project.                                    *
 * Contributors are mentioned in the code where appropriate.              *
 *                                                                        *
 * Permission to use, copy, modify and distribute this software and its   *
 * documentation strictly for non-commercial purposes is hereby granted   *
 * without fee, provided that the above copyright notice appears in all   *
 * copies and that both the copyright notice and this permission notice   *
 * appear in the supporting documentation. The authors make no claims     *
 * about the suitability of this software for any purpose. It is          *
 * provided "as is" without express or implied warranty.                  *
 **************************************************************************/

/*
$Log: AliTRDgtuLink.cxx,v $
Revision 1.1.1.1  2001/01/16 18:02:33  schneide
Import of TRDgtu

*/

///////////////////////////////////////////////////////////////////////////
//                                                                       //
//  Global Tracking Unit - Link class                                    //
//  A Link is an array of candidates belonging to one chamber.           //
//  The candidates are ordered specific.                                 //
//                                                                       //
//  module: (TRDgtu) Global Tracking Unit        author: Rolf Schneider  //
//                                                                       //
///////////////////////////////////////////////////////////////////////////

#include "AliTRDgtuConst.h"
#include "AliTRDgtuGlobal.h"
#include "AliTRDgtuLink.h"

ClassImp(AliTRDgtuLink)
 
//_________________________________________________________________________
AliTRDgtuLink::AliTRDgtuLink():TObject()
{
  //
  //  AliTRDgtuLink default constructor
  //
  //  init an array of candidates for one chamber link
  //

  fLinkData = new TObjArray(kCandsPerChamberMax, 0);
}

//_________________________________________________________________________
AliTRDgtuLink::AliTRDgtuLink(Int_t iPlane, Int_t iSector, Int_t iChamber):TObject()
{
  //
  //  AliTRDgtuLink default constructor with identification
  //

  fPlane   = iPlane;
  fSector  = iSector;
  fChamber = iChamber;

  fLinkData = new TObjArray(kCandsPerChamberMax, 0);
}

//_________________________________________________________________________
AliTRDgtuLink::~AliTRDgtuLink()
{
  //
  //  AliTRDgtuLink default destructor
  //
  //  free memory
  //

  fLinkData->Delete();
  delete fLinkData;
}

//_________________________________________________________________________
void AliTRDgtuLink::AddCandidate(AliTRDgtuCandidate *iCandidate)
{
  //
  //  add a candidate to the current chamber link. Mind the order you add
  //  the candidates. The last added candidate will be read out at last.
  //

  if(fLinkData->GetEntries() >= kCandsPerChamberMax) {
    DBG_W ( printf("WARNING: max. number of candidates per detector is %d\n",
        kCandsPerChamberMax); )
    return;
  }

  if(!iCandidate->IsValid())
    DBG_W ( printf("WARNING: added candidate in (%d, %d, %d, -) is not complete\n", fPlane, fSector, fChamber); )

  fLinkData->AddLast(iCandidate);
}

//_________________________________________________________________________
void AliTRDgtuLink::CalcMidlayer(Int_t iLayer)
{
  //
  //  calculate the middle layer positions for the whole chamber
  //  the position is calculated by RPos, Angle and Layer
  //

  Int_t Index = 0;
  AliTRDgtuCandidate *lCandidate = NULL;

  if(fLinkData->At(0))
    do {
      lCandidate = (AliTRDgtuCandidate *)fLinkData->At(Index);
      lCandidate->CalcMidlayer(iLayer);
      Index++;
    } while (Index <= fLinkData->GetLast());
}

//_________________________________________________________________________
void AliTRDgtuLink::Clear()
{
  //
  //  clear the candidates for the current chamber
  //

  fLinkData->Clear();
}

//_________________________________________________________________________
void AliTRDgtuLink::ReadASCII(char *iFilename)
{
  //
  //  read a ASCII input file for the current chamber
  //
  //  file format:
  //    The first lines can be comment lines. The comment delimiter is ";".
  //    Every following line represents a candidate. The parameters are
  //    seperated by a tab and ordered: RPos, Angle, ZPos, Amplitude, 
  //    TrQuality, RPosMidlayer.
  //
  //  example:
  //
  //  ;RPos   Angle   ZPos    Ampl.   TrQual. RPosMid
  //  599     24      10      80      1       -1
  //  1500    15      10      20      1       -1
  //

  FILE *lFP;
  char lLine[120];
  int  lLinenBr = 0;
  int  lRPos, lAngle, lZPos, lAmplitude, lTrQuality, lRPosMidlayer;

  DBG_I ( printf("**** read ASCII input file <%s> for Link (%d, %d, %d, -)\n", iFilename, fPlane, fSector, fChamber); )

  if(!(fLinkData->IsEmpty()))
    DBG_W ( printf("WARNING: current link is not empty\n"); )

  if(!(lFP=fopen(iFilename, "r"))) {
    DBG_N ( printf("ERROR: cannot open input file <%s>\n", iFilename); )
    return;
  }

  while(fgets(lLine, 120, lFP)) {
    lLinenBr++;
    if(lLine[0] != ';') {
      if(6 != (sscanf(lLine, "%d\t%d\t%d\t%d\t%d\t%d\n", &lRPos, &lAngle, &lZPos,
          &lAmplitude, &lTrQuality, &lRPosMidlayer))) {
        DBG_N ( printf("ERROR: strange input file <%s> at line number %d\n", iFilename, lLinenBr); )
        return;
      }
      AliTRDgtuCandidate *lCand = new AliTRDgtuCandidate();
      lCand->SetRPos(lRPos);
      lCand->SetAngle(lAngle);
      lCand->SetZPos(lZPos);
      lCand->SetAmplitude(lAmplitude);
      lCand->SetTrQuality(lTrQuality);
      lCand->SetRPosMidlayer(lRPosMidlayer);
      AddCandidate(lCand);
    }
  }

  fclose(lFP);
}

//_________________________________________________________________________
TObjArray *AliTRDgtuLink::GetLinkDataObj()
{
  //
  //  pointer to the chamber link data array of type (TObjArray *)
  //

  return(fLinkData);
}

//_________________________________________________________________________
void AliTRDgtuLink::Output()
{
  //
  //  output the current chamber link data
  //

  Int_t Index = 0;
  AliTRDgtuCandidate *lCandidate = NULL;

  DBG_N ( printf("\tNo.\tRPos\tAngle\tZPos\tAmpl.\tTrQ\tRPosMid\n"); )
  if(fLinkData->At(0))
    do {
      lCandidate = (AliTRDgtuCandidate *)fLinkData->At(Index);
      DBG_N ( printf("\t%d\t%d\t%d\t%d\t%d\t%d\t%d\n", Index,
          lCandidate->GetRPos(),
          lCandidate->GetAngle(),
          lCandidate->GetZPos(),
          lCandidate->GetAmplitude(),
          lCandidate->GetTrQuality(),
          lCandidate->GetRPosMidlayer()); )
      Index++;
    } while (Index <= fLinkData->GetLast());
  else
    DBG_N ( printf("\t-- no candidates --\n"); )
}

