/**************************************************************************
 * Copyright(c) 1998-1999, ALICE Experiment at CERN, All rights reserved. *
 *                                                                        *
 * Author: The ALICE Off-line Project.                                    *
 * Contributors are mentioned in the code where appropriate.              *
 *                                                                        *
 * Permission to use, copy, modify and distribute this software and its   *
 * documentation strictly for non-commercial purposes is hereby granted   *
 * without fee, provided that the above copyright notice appears in all   *
 * copies and that both the copyright notice and this permission notice   *
 * appear in the supporting documentation. The authors make no claims     *
 * about the suitability of this software for any purpose. It is          *
 * provided "as is" without express or implied warranty.                  *
 **************************************************************************/

/*
$Log: AliTRDgtuReal.cxx,v $
Revision 1.3  2001/02/06 17:38:16  schneide
Replacement of constant array for RealTracks by a dynamic TClonesArray

Revision 1.2  2001/01/30 18:48:27  schneide
Changes for AliRoot 3.04, Visualisation of Tracklets, FindNextTrack added

Revision 1.1.1.1  2001/01/16 18:02:33  schneide
Import of TRDgtu

*/

///////////////////////////////////////////////////////////////////////////
//                                                                       //
//  Global Tracking Unit - Real Track class                              //
//  This class represets a real track and constains all information      //
//  needed by the AliTRD simulation.                                     //
//                                                                       //
//  module: (TRDgtu) Global Tracking Unit        author: Rolf Schneider  //
//                                                                       //
///////////////////////////////////////////////////////////////////////////

#include <AliTRDdigit.h>
#include <AliTRDhit.h>
#include "AliTRDgtuConst.h"
#include "AliTRDgtuGlobal.h"
#include "AliTRDgtuReal.h"

ClassImp(AliTRDgtuReal)

//_________________________________________________________________________
AliTRDgtuReal::AliTRDgtuReal():TObject()
{
  //
  //  AliTRDgtuReal default constructor
  //

  fParticles        = -1;
  fPrimaryParticles = -1;

  fPlaneMin   = fPlaneMax   = -1;
  fSectorMin  = fSectorMax  = -1;
  fChamberMin = fChamberMax = -1;
}

//_________________________________________________________________________
AliTRDgtuReal::AliTRDgtuReal(Int_t iParticles, Int_t iPrimaryParticles):TObject()
{
  //
  //  AliTRDgtuReal default constructor
  //

  fParticles        = iParticles;
  fPrimaryParticles = iPrimaryParticles;

  fRealTrack = new TClonesArray("AliTRDgtuRealTrack", iParticles);
  for(Int_t parno=0; parno<iParticles; parno++)
    new((*fRealTrack)[parno]) AliTRDgtuRealTrack();

  fPlaneMin   = fPlaneMax   = -1;
  fSectorMin  = fSectorMax  = -1;
  fChamberMin = fChamberMax = -1;
}

//_________________________________________________________________________
AliTRDgtuReal::~AliTRDgtuReal()
{
  //
  //  AliTRDgtuReal default destructor
  //
  //  free memory
  //

  fRealTrack->Clear();
  delete fRealTrack;
}

//_________________________________________________________________________
void AliTRDgtuReal::Dump()
{
  //
  //  dumps global Real data to stdout
  //

  printf("--AliTRDgtuReal--------------------------------------\n");
  printf("  particles         : ");
  (fParticles==-1) ? printf("-\n") : printf("%d\n", fParticles);
  printf("  primary particles : ");
  (fPrimaryParticles==-1) ? printf("-\n") : printf("%d\n", fPrimaryParticles);
  printf("  plane range       : %d .. %d\n", fPlaneMin, fPlaneMax);
  printf("  sector range      : %d .. %d\n", fSectorMin, fSectorMax);
  printf("  chamber range     : %d .. %d\n", fChamberMin, fChamberMax);
  printf("-----------------------------------------------------\n");
}

//_________________________________________________________________________
Int_t AliTRDgtuReal::GetTRDparticles()
{
  //
  //  count the number of particles within TRD
  //

  if(!CheckEnv()) return(-1);

  Int_t lTRDpar = 0;

  for(Int_t parno=0; parno<fParticles; parno++)
    if(((AliTRDgtuRealTrack *)fRealTrack->At(parno))->GetTrackID() > -1) lTRDpar++;

  return(lTRDpar);
}

//_________________________________________________________________________
Int_t AliTRDgtuReal::GetTRDprimaryParticles()
{
  //
  //  count the number of primary particles within TRD
  //

  if(!CheckEnv()) return(-1);

  Int_t lTRDprimpar = 0;

  for(Int_t parno=0; parno<fPrimaryParticles; parno++)
    if(((AliTRDgtuRealTrack *)fRealTrack->At(parno))->GetTrackID() > -1) lTRDprimpar++;

  return(lTRDprimpar);
}

//_________________________________________________________________________
Int_t AliTRDgtuReal::Local2Global(Int_t iPlane, Int_t iSector, Int_t iChamber)
{
  //
  //  Conversion between local and global notation
  //

  Int_t lGlobal = (iChamber * kSectors * kLayers) + (iSector * kLayers) + iPlane;

  return(lGlobal);
}

//_________________________________________________________________________
Bool_t AliTRDgtuReal::CheckEnv()
{
  //
  //  checks if the environment is set
  //

  if(fParticles == -1) {
    DBG_W( printf("ERROR: AliTRDgtuReal not initialized (number of particles)\n"); )
    return(0);
  }

  if(fPrimaryParticles == -1)
    DBG_W( printf("WARNING: AliTRDgtuReal not initialized (number of primary particles)\n"); )

  return(1);
}

//_________________________________________________________________________
Bool_t AliTRDgtuReal::CheckRange()
{
  //
  //  checks if the ranges are set
  //

  if((fPlaneMin == -1) || (fPlaneMax == -1) || (fSectorMin == -1) || (fSectorMax == -1) || (fChamberMin == -1) || (fChamberMax == -1)) {
    DBG_W( printf("ERROR: AliTRDgtuReal ranges not set\n"); )
    return(0);
  }

  return(1);
}

//_________________________________________________________________________
void AliTRDgtuReal::FillRealTracks(AliTRDgeometry *TRDgeometry, AliTRDdigitsManager *DigitsManager, TClonesArray *ParticleArray, Bool_t info)
{
  //
  //  generate RealTracks informations
  //

  if((!CheckEnv()) || (!CheckRange())) return;

  for(Int_t chamber=fChamberMin; chamber<=fChamberMax; chamber++) {
    for(Int_t sector=fSectorMin; sector<=fSectorMax; sector++) {
      for(Int_t plane=fPlaneMin; plane<=fPlaneMax; plane++) {
        if(info) printf("working on chamber %d, sector %2d, plane %d ...\n",
            chamber, sector, plane);
        // get TRD geometry
        Int_t lDet     = TRDgeometry->GetDetector(plane, chamber, sector);
        Int_t lTimeMax = TRDgeometry->GetTimeMax();
        Int_t lColMax  = TRDgeometry->GetColMax(plane);
        Int_t lRowMax  = TRDgeometry->GetRowMax(plane, chamber, sector);
        for(Int_t timebin=0; timebin<lTimeMax; timebin++) {
          for(Int_t column=0; column<lColMax; column++) {
            for(Int_t padrow=0; padrow<lRowMax; padrow++) {
              for(Int_t tno=0; tno<3; tno++) {
                Int_t trackno = DigitsManager->GetTrack(tno, padrow, column, timebin, lDet);
                if(trackno>-1) {
                  AliTRDgtuRealTrack *lRealTrack = (AliTRDgtuRealTrack *)fRealTrack->At(trackno);
                  //TParticle *Particle = (TParticle *)ParticleArray->At(trackno);
                  AliTRDdigit *Digit = DigitsManager->GetDigit(padrow, column, timebin, lDet);
                  // fill track
                  lRealTrack->SetTrackID(trackno);
                  lRealTrack->SetSector(sector);
                  lRealTrack->SetChamber(chamber);
                  Int_t lAmp = Digit->GetAmp();
                  //if(lAmp > 255) lAmp = 255;
                  lRealTrack->IncAmplitudeSum(plane, lAmp);
                  lRealTrack->IncDigitsSum(plane, 1);
                  if(lRealTrack->GetStartPadrow(plane) == -1) lRealTrack->SetStartPadrow(plane, padrow);
                  lRealTrack->SetEndPadrow(plane, padrow);
                  if(lRealTrack->GetStartColumn(plane) == -1) lRealTrack->SetStartColumn(plane, column);
                  lRealTrack->SetEndColumn(plane, column);
                  delete Digit;
                }
              } // tno (track 0..2)
            } // padrow
          } // column
        } // timebin
      } // plane
    } // sector
  } // chamber

}

//_________________________________________________________________________
void AliTRDgtuReal::GetExactPositions(AliTRDgeometry *TRDgeometry, AliRun *gAlice, AliTRDv1 *TRD, TTree *hitTree)
{
  //
  //  loop over all hits and store exact positions of stiff tracks
  //

  if((!CheckEnv()) || (!CheckRange())) return;

  Float_t RminArr[fParticles][kLayers];
  Float_t RmaxArr[fParticles][kLayers];

  Float_t lXmin[fParticles][kLayers];
  Float_t lXmax[fParticles][kLayers];
  Float_t lYmin[fParticles][kLayers];
  Float_t lYmax[fParticles][kLayers];
  Float_t lZmin[fParticles][kLayers];
  Float_t lZmax[fParticles][kLayers];

  // init min. and max. radius array
  for(Int_t parno=0; parno<fParticles; parno++) {
    for(Int_t plane=0; plane<kLayers; plane++) {
      RminArr[parno][plane] = 1000;
      RmaxArr[parno][plane] = -1.0;
    }
  }

  // loop over primary particles
  for(Int_t parno=0; parno<fPrimaryParticles; parno++) {
   gAlice->ResetHits();
   hitTree->GetEvent(parno);
   Int_t nHit = TRD->Hits()->GetEntriesFast();
   // loop over all hits depending (indirect) to particle
    for(Int_t hit = 0; hit < nHit; hit++) {
      AliTRDhit *TRDhit = (AliTRDhit *)TRD->Hits()->UncheckedAt(hit);
      Int_t lTrackID = TRDhit->GetTrack();
      if(((AliTRDgtuRealTrack *)fRealTrack->At(lTrackID))->GetTrackID() == -1) continue;
      Int_t lDet = TRDhit->GetDetector();
      Int_t lPlane = TRDgeometry->GetPlane(lDet);
      Float_t lX = TRDhit->X();
      Float_t lY = TRDhit->Y();
      Float_t lZ = TRDhit->Z();
      Float_t lR = TMath::Sqrt(lX*lX + lY*lY);
      if(lR < 250.0) {
	printf("AliTRDgtuReal::GetExactPositions : WARNING - hit outside the TRD found at position (%5.2f, %5.2f, %5.2f).\n", lX, lY, lZ);
	continue;
      }
      if(lR < RminArr[lTrackID][lPlane]) {
        lXmin[lTrackID][lPlane] = lX;
        lYmin[lTrackID][lPlane] = lY;
        lZmin[lTrackID][lPlane] = lZ;
        RminArr[lTrackID][lPlane] = lR;
      }
      if(lR > RmaxArr[lTrackID][lPlane]) {
        lXmax[lTrackID][lPlane] = lX;
        lYmax[lTrackID][lPlane] = lY;
        lZmax[lTrackID][lPlane] = lZ; 
        RmaxArr[lTrackID][lPlane] = lR;
      }
    } // hit
  } // particle

  Float_t pos[3], rot[3];
  Float_t timeBinSize = TRDgeometry->GetTimeBinSize();

  for(Int_t parno=0; parno<fParticles; parno++) {
    for(Int_t lPla=fPlaneMin; lPla<=fPlaneMax; lPla++) {
      AliTRDgtuRealTrack *lRT = (AliTRDgtuRealTrack *)fRealTrack->At(parno);
      if(!lRT->GetDigitsSum(lPla)) continue;
      Int_t lSec = lRT->GetSector();
      Int_t lCha = lRT->GetChamber();
      Int_t lDet =  TRDgeometry->GetDetector(lPla, lCha, lSec);
      Float_t rowPadSize  = TRDgeometry->GetRowPadSize(lPla, lCha, lSec);
      Float_t colPadSize  = TRDgeometry->GetColPadSize(lPla);
      Float_t row0  = TRDgeometry->GetRow0(lPla, lCha, lSec);
      Float_t col0  = TRDgeometry->GetCol0(lPla);
      Float_t time0 = TRDgeometry->GetTime0(lPla);
      // transformation for innermost Hit
      pos[0] = lXmin[parno][lPla]; pos[1] = lYmin[parno][lPla]; pos[2] = lZmin[parno][lPla];
      TRDgeometry->Rotate(lDet, pos, rot);
      lRT->SetExStartPadrow(lPla, (rot[2] - row0) / rowPadSize);
      lRT->SetExStartColumn(lPla, (rot[1] - col0) / colPadSize);
      lRT->SetExStartTimebin(lPla, (rot[0] - time0) / timeBinSize);
      // transformation for outermost Hit
      pos[0] = lXmax[parno][lPla]; pos[1] = lYmax[parno][lPla]; pos[2] = lZmax[parno][lPla];
      TRDgeometry->Rotate(lDet, pos, rot);
      lRT->SetExEndPadrow(lPla, (rot[2] - row0) / rowPadSize);
      lRT->SetExEndColumn(lPla, (rot[1] - col0) / colPadSize);
      lRT->SetExEndTimebin(lPla, (rot[0] - time0) / timeBinSize);
      //printf("startrow %5.2f, endrow %5.2f, startcol %5.2f, endcol %5.2f, starttime %5.2f, endtime %5.2f\n", fRealTrack[parno].GetExStartPadrow(lPla), fRealTrack[parno].GetExEndPadrow(lPla), fRealTrack[parno].GetExStartColumn(lPla), fRealTrack[parno].GetExEndColumn(lPla), fRealTrack[parno].GetExStartTimebin(lPla), fRealTrack[parno].GetExEndTimebin(lPla));
    }
  }
}

