/**************************************************************************
 * Copyright(c) 1998-1999, ALICE Experiment at CERN, All rights reserved. *
 *                                                                        *
 * Author: The ALICE Off-line Project.                                    *
 * Contributors are mentioned in the code where appropriate.              *
 *                                                                        *
 * Permission to use, copy, modify and distribute this software and its   *
 * documentation strictly for non-commercial purposes is hereby granted   *
 * without fee, provided that the above copyright notice appears in all   *
 * copies and that both the copyright notice and this permission notice   *
 * appear in the supporting documentation. The authors make no claims     *
 * about the suitability of this software for any purpose. It is          *
 * provided "as is" without express or implied warranty.                  *
 **************************************************************************/

/*
$Log: AliTRDgtuRealTrack.cxx,v $
Revision 1.2  2001/02/06 17:38:17  schneide
Replacement of constant array for RealTracks by a dynamic TClonesArray

Revision 1.1.1.1  2001/01/16 18:02:33  schneide
Import of TRDgtu

*/

///////////////////////////////////////////////////////////////////////////
//                                                                       //
//  Global Tracking Unit - Real Track class                              //
//  This class represets a real track and constains all information      //
//  needed by the AliTRD simulation.                                     //
//                                                                       //
//  module: (TRDgtu) Global Tracking Unit        author: Rolf Schneider  //
//                                                                       //
///////////////////////////////////////////////////////////////////////////

#include <TFile.h>
#include "AliTRDgtuConst.h"
#include "AliTRDgtuGlobal.h"
#include "AliTRDgtuRealTrack.h"

ClassImp(AliTRDgtuRealTrack)

//_________________________________________________________________________
AliTRDgtuRealTrack::AliTRDgtuRealTrack():TObject()
{
  //
  //  AliTRDgtuRealTrack default constructor
  //

  fTrackID = -1;
  fSector  = -1;
  fChamber = -1;

  for(Int_t plane=0; plane<kLayers; plane++) {
    fAmplitudeSum[plane]  = 0;
    fDigitsSum[plane]     = 0;
    fStartPadrow[plane]   = -1;
    fEndPadrow[plane]     = -1;
    fStartColumn[plane]   = -1;
    fEndColumn[plane]     = -1;
    fExStartPadrow[plane] = -1.0;
    fExEndPadrow[plane]   = -1.0;
    fExStartColumn[plane] = -1.0;
    fExEndColumn[plane]   = -1.0;
    fExStartTimebin[plane]= -1.0;
    fExEndTimebin[plane]  = -1.0;
  }
}

//_________________________________________________________________________
AliTRDgtuRealTrack::AliTRDgtuRealTrack(Int_t iTrackID, Int_t iSector, Int_t iChamber):TObject()
{
  //
  //  AliTRDgtuRealTrack default constructor
  //

  fTrackID = iTrackID;
  fSector  = -1;
  fChamber = -1;

  for(Int_t plane=0; plane<kLayers; plane++) {
    fAmplitudeSum[plane]  = 0;
    fDigitsSum[plane]     = 0;
    fStartPadrow[plane]   = -1;
    fEndPadrow[plane]     = -1;
    fStartColumn[plane]   = -1;
    fEndColumn[plane]     = -1;
    fExStartPadrow[plane] = -1.0;
    fExEndPadrow[plane]   = -1.0;
    fExStartColumn[plane] = -1.0;
    fExEndColumn[plane]   = -1.0;
    fExStartTimebin[plane]= -1.0;
    fExEndTimebin[plane]  = -1.0;
  }
}

//_________________________________________________________________________
AliTRDgtuRealTrack::~AliTRDgtuRealTrack()
{
  //
  //  AliTRDgtuRealTrack default destructor
  //
  //  free memory
  //

}

//_________________________________________________________________________
void AliTRDgtuRealTrack::Dump()
{
  //
  //  dumps Track data to stdout
  //

  printf("--AliTRDgtuRealTrack---------------------------------\n");
  printf("  trackID        : "); (fTrackID==-1) ? printf("-\n") : printf("%d\n", fTrackID);
  printf("  chamber        : "); (fChamber==-1) ? printf("-\n") : printf("%d\n", fChamber);
  printf("  sector         : "); (fSector==-1)  ? printf("-\n") : printf("%d\n", fSector);
  printf("  amplitude sum  : %d\n", GetAmplitudeSum());
  printf("  digits sum     : %d\n", GetDigitsSum());
  for(Int_t plane=0; plane<kLayers; plane++) {
    printf("  plane %d (mean) : (col %6.2f, row %5.2f, time %5.2f) - (col %6.2f, row %5.2f, time %5.2f)\n", plane, (Float_t)fStartColumn[plane], (Float_t)fStartPadrow[plane], 0.0, (Float_t)fEndColumn[plane], (Float_t)fEndPadrow[plane], 30.0);
    printf("  plane %d (real) : (col %6.2f, row %5.2f, time %5.2f) - (col %6.2f, row %5.2f, time %5.2f)\n", plane, fExStartColumn[plane], fExStartPadrow[plane], fExStartTimebin[plane], fExEndColumn[plane], fExEndPadrow[plane], fExEndTimebin[plane]);
  }
  printf("-----------------------------------------------------\n");
}

//_________________________________________________________________________
Int_t AliTRDgtuRealTrack::GetAmplitudeSum()
{
  //
  //  return the sum of amplitudes of all planes
  //

  Int_t lAmp = 0;

  for(Int_t plane=0; plane<kLayers; plane++)
    lAmp += fAmplitudeSum[plane];

  return(lAmp);
}

//_________________________________________________________________________
Int_t AliTRDgtuRealTrack::GetDigitsSum()
{
  //
  //  return the sum of digits of all planes
  //

  Int_t lDig = 0;

  for(Int_t plane=0; plane<kLayers; plane++)
    lDig += fDigitsSum[plane];

  return(lDig);
}

