/**************************************************************************
 * Copyright(c) 1998-1999, ALICE Experiment at CERN, All rights reserved. *
 *                                                                        *
 * Author: The ALICE Off-line Project.                                    *
 * Contributors are mentioned in the code where appropriate.              *
 *                                                                        *
 * Permission to use, copy, modify and distribute this software and its   *
 * documentation strictly for non-commercial purposes is hereby granted   *
 * without fee, provided that the above copyright notice appears in all   *
 * copies and that both the copyright notice and this permission notice   *
 * appear in the supporting documentation. The authors make no claims     *
 * about the suitability of this software for any purpose. It is          *
 * provided "as is" without express or implied warranty.                  *
 **************************************************************************/

/*
$Log: AliTRDgtuStatistic.cxx,v $
Revision 1.2  2001/01/30 18:48:27  schneide
Changes for AliRoot 3.04, Visualisation of Tracklets, FindNextTrack added

Revision 1.1.1.1  2001/01/16 18:02:33  schneide
Import of TRDgtu

*/

///////////////////////////////////////////////////////////////////////////
//                                                                       //
//  Global Tracking Unit - Statistic class                               //
//  This class allows a simple 2D and 3D visualisation of candidates     //
//  and  tracks.                                                         //
//                                                                       //
//  module: (TRDgtu) Global Tracking Unit        author: Rolf Schneider  //
//                                                                       //
///////////////////////////////////////////////////////////////////////////

#include <TParticle.h>
#include <AliTRDhit.h>
#include "AliTRDgtuConst.h"
#include "AliTRDgtuGlobal.h"
#include "AliTRDgtuStatistic.h"
#include "AliTRDgtuRealTrack.h"

ClassImp(AliTRDgtuStatistic)

//_________________________________________________________________________
AliTRDgtuStatistic::AliTRDgtuStatistic(AliRun *gAlice):TObject()
{
  //
  //  AliTRDgtuStatistic default constructor
  //

  HEAD( printf("AliTRDgtuStatistic\n"); )

  if(!(fgAlice = gAlice)) {
    DBG_N ( printf("ERROR: no gAlice environment found\n"); )
    return;
  } else DBG_I ( printf("--- found gAlice object\n"); )

  if(!(fTRD = (AliTRDv1*)gAlice->GetDetector("TRD"))) {
    DBG_N ( printf("ERROR: cannot find TRD in gAlice\n"); )
    return;
  } else DBG_I ( printf("--- found TRD object\n"); )

  if(!(fTRDgeometry = fTRD->GetGeometry())) {
    DBG_N ( printf("ERROR: cannot find TRD Geometry in TRD\n"); )
    return;
  } else DBG_I ( printf("--- found TRD geometry\n"); )

  if((fParticles = gAlice->GetEvent(0)) <= 0) {
    DBG_N ( printf("ERROR: cannot get event 0\n"); )
    return;
  } else DBG_I ( printf("--- got event 0\n"); )

  DBG_N ( printf("--- read particles array\n"); )
//  fParticleArray = fgAlice->Particles();

  fDigitsManager = new AliTRDdigitsManager();
  if(!(fDigitsManager->ReadDigits()))
    DBG_W ( printf("WARNING: cannot read Digits file\n"); )
  else DBG_I ( printf("--- read Digits\n"); )

  DBG_N ( printf("--- read hit tree\n"); )
  fHitTree = fgAlice->TreeH();
  fPrimaryParticles = (Int_t)fHitTree->GetEntries();

  SetPlaneRange(0, kLayers);
  SetSectorRange(0, kSectors);
  SetChamberRange(0, kChambers);

  h1 = NULL;
}

//_________________________________________________________________________
AliTRDgtuStatistic::~AliTRDgtuStatistic()
{
  //
  //  AliTRDgtuStatistic default destructor
  //

}

//_________________________________________________________________________
void AliTRDgtuStatistic::SetReal(AliTRDgtuReal *iReal)
{
  //
  //  set AliTRDgtuReal enviroment
  //

  fReal = iReal;
  if((fReal->GetParticles() == fParticles) && (fReal->GetPrimaryParticles() == fPrimaryParticles)
)
    DBG_N ( printf("--- AliTRDgtuReal root file seems to match\n"); )
  else
    DBG_N ( printf("*** WARNING: number of particles of root files do not match with AliTRDgtuReal root file\n"); )

}

//_________________________________________________________________________
TH1F *AliTRDgtuStatistic::GenH1(Float_t iPtMin, Bool_t iPrimary)
{
  //
  //  histogram: real energy loss vs pt
  //
  //  iPtMin:    for all particles with pt >= iPtMin
  //  iPrimary:  if iPrimary == TRUE, only for primary particles
  //

  Char_t title[120];
  sprintf(title, "real: energy loss vs. pt (pt>%4.2f, %s part.)", iPtMin, (iPrimary) ? "prim." : "all");
  h1 = new TH1F("h1", title, 50.0, 0.0, 5.0);
  h1->SetXTitle("pt"); h1->SetYTitle("amplitude");

  Int_t parcut = (iPrimary) ? fPrimaryParticles : fParticles;

  for(Int_t parno=0; parno<parcut; parno++) {
    TParticle *Particle = (TParticle *)fParticleArray->At(parno);
    Float_t lPt = Particle->Pt();
    if(lPt >= iPtMin) {
      AliTRDgtuRealTrack *RealTrack = fReal->GetRealTrack(parno);
      if(RealTrack->GetTrackID() != -1) {
        h1->Fill(lPt, RealTrack->GetAmplitudeSum());
      }
    } // pt min
  } // parno

  return(h1);
}

//_________________________________________________________________________
TH1F *AliTRDgtuStatistic::GenH2(Float_t iPtMin, Bool_t iPrimary)
{
  //
  //  histogram: digits vs pt
  //
  //  iPtMin:    for all particles with pt >= iPtMin
  //  iPrimary:  if iPrimary == TRUE, only for primary particles
  //

  Char_t title[120];
  sprintf(title, "real: digits vs. pt (pt>%4.2f, %s part.)", iPtMin, (iPrimary) ? "prim." : "all");
  h2 = new TH1F("h2", title, 50.0, 0.0, 5.0);
  h2->SetXTitle("pt"); h2->SetYTitle("digits");

  Int_t parcut = (iPrimary) ? fPrimaryParticles : fParticles;

  for(Int_t parno=0; parno<parcut; parno++) {
    TParticle *Particle = (TParticle *)fParticleArray->At(parno);
    Float_t lPt = Particle->Pt();
    if(lPt >= iPtMin) {
      AliTRDgtuRealTrack *RealTrack = fReal->GetRealTrack(parno);
      if(RealTrack->GetTrackID() != -1) {
        h2->Fill(lPt, RealTrack->GetDigitsSum());
      }
    } // pt min
  } // parno

  return(h2);
}

//_________________________________________________________________________
TH1F *AliTRDgtuStatistic::GenH3(Float_t iPtMin, Bool_t iPrimary)
{
  //
  //  histogram: pt distribution
  //
  //  iPtMin:    for all particles with pt >= iPtMin
  //  iPrimary:  if iPrimary == TRUE, only for primary particles
  //

  Char_t title[120];
  sprintf(title, "real: pt distribution (pt>%4.2f, %s part.)", iPtMin, (iPrimary) ? "prim." : "all");
  h3 = new TH1F("h3", title, 50.0, 0.0, 5.0);
  h3->SetXTitle("pt");

  Int_t parcut = (iPrimary) ? fPrimaryParticles : fParticles;

  for(Int_t parno=0; parno<parcut; parno++) {
    TParticle *Particle = (TParticle *)fParticleArray->At(parno);
    Float_t lPt = Particle->Pt();
    if(lPt >= iPtMin) {
      AliTRDgtuRealTrack *RealTrack = fReal->GetRealTrack(parno);
      if(RealTrack->GetTrackID() != -1) {
        h3->Fill(lPt, 1);
      }
    } // pt min
  } // parno

  return(h3);
}

//_________________________________________________________________________
TH2F *AliTRDgtuStatistic::GenH4(Float_t iPtMin, Bool_t iPrimary)
{
  //
  //  histogram: mean deflection vs. pt
  //
  //  iPtMin:    for all particles with pt >= iPtMin
  //  iPrimary:  if iPrimary == TRUE, only for primary particles
  //

  Char_t title[120];
  sprintf(title, "mean: deflection vs. pt (pt>%4.2f, %s part.)", iPtMin, (iPrimary) ? "prim." : "all");
  h4 = new TH2F("h4", title, 50.0, 0.0, 5.0, 20, -10, 10);
  h4->SetXTitle("pt"); h4->SetYTitle("deflection");

  Int_t parcut = (iPrimary) ? fPrimaryParticles : fParticles;

  for(Int_t parno=0; parno<parcut; parno++) {
    TParticle *Particle = (TParticle *)fParticleArray->At(parno);
    Float_t lPt = Particle->Pt();
    if(lPt >= iPtMin) {
      AliTRDgtuRealTrack *RealTrack = fReal->GetRealTrack(parno);
      if(RealTrack->GetTrackID() != -1) {
	for(Int_t plane=0; plane<6; plane++) {
	  if(RealTrack->GetExStartColumn(plane) != -1.0) {
	    Float_t meandefl = Pt2Defl(lPt, Particle->GetPDG()->Charge(),
				       plane, (Int_t)RealTrack->GetExStartColumn(plane));
	    h4->Fill(lPt, meandefl);
	  }
	} // plane
      }
    } // pt min
  } // parno

  return(h4);
}

//_________________________________________________________________________
TH2F *AliTRDgtuStatistic::GenH5(Float_t iPtMin, Bool_t iPrimary)
{
  //
  //  histogram: real deflection vs. pt
  //
  //  iPtMin:    for all particles with pt >= iPtMin
  //  iPrimary:  if iPrimary == TRUE, only for primary particles
  //

  Char_t title[120];
  sprintf(title, "real: deflection vs. pt (pt>%4.2f, %s part.)", iPtMin, (iPrimary) ? "prim." : "all");
  h5 = new TH2F("h5", title, 50.0, 0.0, 5.0, 20, -10, 10);
  h5->SetXTitle("pt"); h5->SetYTitle("deflection");

  Int_t parcut = (iPrimary) ? fPrimaryParticles : fParticles;

  for(Int_t parno=0; parno<parcut; parno++) {
    TParticle *Particle = (TParticle *)fParticleArray->At(parno);
    Float_t lPt = Particle->Pt();
    if(lPt >= iPtMin) {
      AliTRDgtuRealTrack *RealTrack = fReal->GetRealTrack(parno);
      if(RealTrack->GetTrackID() != -1) {
	for(Int_t plane=0; plane<6; plane++) {
	  if(RealTrack->GetExStartColumn(plane) != -1.0) {
	    Float_t realdefl = Pos2Defl(RealTrack->GetExStartColumn(plane),
					RealTrack->GetExEndColumn(plane),
					RealTrack->GetExStartTimebin(plane),
					RealTrack->GetExEndTimebin(plane),
					plane,
					(Int_t)RealTrack->GetExStartColumn(plane));
	    h5->Fill(lPt, realdefl);
	  }
	} // plane
      }
    } // pt min
  } // parno

  return(h5);
}

//_________________________________________________________________________
void AliTRDgtuStatistic::SetPlaneRange(Int_t iMin, Int_t iMax)
{
  //
  //  Set the current range of Planes
  //

  fPlaneMin = iMin;
  fPlaneMax = iMax;
}

//_________________________________________________________________________
void AliTRDgtuStatistic::SetSectorRange(Int_t iMin, Int_t iMax)
{
  //
  //  Set the current range of Planes
  //

  fSectorMin = iMin;
  fSectorMax = iMax;
}

//_________________________________________________________________________
void AliTRDgtuStatistic::SetChamberRange(Int_t iMin, Int_t iMax)
{
  //
  //  Set the current range of Planes
  //

  fChamberMin = iMin;
  fChamberMax = iMax;
}

//_________________________________________________________________________
Float_t AliTRDgtuStatistic::Pos2Defl(Float_t xinner, Float_t xouter, Float_t tinner, Float_t touter, Int_t plane, Int_t column)
{
  //
  //  coonvert position to deflection in mm at outer side of detector
  //

  // lenght unit: mm
  Float_t d     = (xouter - xinner) * 10;
          d    *= 30.0/(touter-tinner);
  Float_t kR    = fTRDgeometry->GetTime0(plane) * 10;
  Float_t b     = fTRDgeometry->GetCol0(plane) * 10 + (column * fTRDgeometry->GetColPadSize(plane) * 10);
  Float_t g     = b*30/kR;

  Float_t defl  = d - g;
  return(defl);
}

//_________________________________________________________________________
Float_t AliTRDgtuStatistic::Pt2Defl(Float_t pt, Float_t charge, Int_t plane, Int_t column)
{
  //
  //  convert transverse momentum to deflection in mm at outer side of detector
  //

  // lenght unit: mm
  Float_t r     = (pt * 1000.0) / (0.3 * 0.2);
  Float_t kR    = fTRDgeometry->GetTime0(plane) * 10;
  Float_t b     = fTRDgeometry->GetCol0(plane)*10 + (column * fTRDgeometry->GetColPadSize(plane) * 10);
  Float_t a     = TMath::Sqrt(kR*kR + b*b);
  Float_t alpha = charge * TMath::ASin(a/(2.0*r));
  Float_t beta  = TMath::ATan(b/kR);
  Float_t gamma = (alpha >=0 ) ? (alpha - beta) : (-alpha + beta); //
  Float_t g     = b*30/kR;
  Float_t d     = TMath::Tan(gamma) * 30;

  Float_t defl  = (charge >= 0.0) ? -(d+g) : -(-d+g);

  return(defl);
}

//_________________________________________________________________________
void AliTRDgtuStatistic::GetParticlePos(Int_t iParticle, Int_t iDetector, Float_t *iGPos1, Float_t *iGPos2, Float_t *iLPos1, Float_t *iLPos2)
{
  //
  //  calculate the exact global and local position for given Particle within given TRD detector
  //

  Float_t lCPx1 = 0.0, lCPy1 = 0.0, lCPz1 = 0.0, lCPx2 = 0.0, lCPy2 = 0.0, lCPz2 = 0.0;
  Float_t lCPr1 = 1000.0, lCPr2 = 0.0;
  Float_t lX = 0.0, lY = 0.0, lZ = 0.0, lR = 0.0;

  TTree *hitTree = fgAlice->TreeH();
  Int_t nTrack = (Int_t)hitTree->GetEntries();

  for(Int_t lNo = 0; lNo < nTrack; lNo++) {
    fgAlice->ResetHits();
    hitTree->GetEvent(lNo);
    Int_t nHit = fTRD->Hits()->GetEntriesFast();
    for(Int_t hit = 0; hit < nHit; hit++) {
      AliTRDhit *TRDhit = (AliTRDhit *)fTRD->Hits()->UncheckedAt(hit);
      if((TRDhit->GetTrack() == iParticle) && (iDetector == TRDhit->GetDetector())) {
        lX = TRDhit->X();
        lY = TRDhit->Y();
        lZ = TRDhit->Z();
        lR = sqrt(lX*lX + lY*lY);
        if(lR < lCPr1) {lCPr1 = lR, lCPx1 = lX, lCPy1 = lY; lCPz1 = lZ;}
        if(lR > lCPr2) {lCPr2 = lR, lCPx2 = lX, lCPy2 = lY; lCPz2 = lZ;}
      }
    }
  }

  // global coordinates
  iGPos1[0] = lCPx1; iGPos1[1] = lCPy1; iGPos1[2] = lCPz1; 
  iGPos2[0] = lCPx2; iGPos2[1] = lCPy2; iGPos2[2] = lCPz2; 

  // local coordinates
  Float_t pos[3], rot[3];
  Int_t lPla = fTRDgeometry->GetPlane(iDetector);
  Int_t lCha = fTRDgeometry->GetChamber(iDetector);
  Int_t lSec = fTRDgeometry->GetSector(iDetector);
  Float_t row0  = fTRDgeometry->GetRow0(lPla, lCha, lSec);
  Float_t col0  = fTRDgeometry->GetCol0(lPla);
  Float_t time0 = fTRDgeometry->GetTime0(lPla);
  Float_t rowPadSize  = fTRDgeometry->GetRowPadSize(lPla, lCha, lSec);
  Float_t colPadSize  = fTRDgeometry->GetColPadSize(lPla);
  Float_t timeBinSize = fTRDgeometry->GetTimeBinSize();
  // transformation for innermost Hit
  pos[0] = lCPx1; pos[1] = lCPy1; pos[2] = lCPz1;
  fTRDgeometry->Rotate(iDetector, pos, rot);
  iLPos1[0] = (rot[2] - row0) / rowPadSize;
  iLPos1[1] = (rot[1] - col0) / colPadSize;
  iLPos1[2] = (rot[0] - time0) / timeBinSize;
  // transformation for outermost Hit
  pos[0] = lCPx2; pos[1] = lCPy2; pos[2] = lCPz2;
  fTRDgeometry->Rotate(iDetector, pos, rot);
  iLPos2[0] = (rot[2] - row0) / rowPadSize;
  iLPos2[1] = (rot[1] - col0) / colPadSize;
  iLPos2[2] = (rot[0] - time0) / timeBinSize;

}
