/**************************************************************************
 * Copyright(c) 1998-1999, ALICE Experiment at CERN, All rights reserved. *
 *                                                                        *
 * Author: The ALICE Off-line Project.                                    *
 * Contributors are mentioned in the code where appropriate.              *
 *                                                                        *
 * Permission to use, copy, modify and distribute this software and its   *
 * documentation strictly for non-commercial purposes is hereby granted   *
 * without fee, provided that the above copyright notice appears in all   *
 * copies and that both the copyright notice and this permission notice   *
 * appear in the supporting documentation. The authors make no claims     *
 * about the suitability of this software for any purpose. It is          *
 * provided "as is" without express or implied warranty.                  *
 **************************************************************************/

/*
$Log: AliTRDgtuTMU.cxx,v $
Revision 1.1.1.1  2001/01/16 18:02:33  schneide
Import of TRDgtu

*/

///////////////////////////////////////////////////////////////////////////
//                                                                       //
//  Global Tracking Unit - TMU class                                     //
//  This class represets a Track Matching Unit. One TMU is resposible    //
//  for one padrow within all layers.                                    //
//                                                                       //
//  module: (TRDgtu) Global Tracking Unit        author: Rolf Schneider  //
//                                                                       //
///////////////////////////////////////////////////////////////////////////

#include "AliTRDgtuConst.h"
#include "AliTRDgtuGlobal.h"
#include "AliTRDgtuCandidate.h"
#include "AliTRDgtuTrack.h"
#include "AliTRDgtuVisual.h"
#include "AliTRDgtuTMU.h"

ClassImp(AliTRDgtuTMU)

//_________________________________________________________________________
AliTRDgtuTMU::AliTRDgtuTMU():TObject()
{
  //
  //  AliTRDgtuTMU default constructor
  //
  //  init an array of candidates for each layer within the current TMU
  //  and reset the additional flag, used by the TMU
  //  init an array for the found tracks
  //

  for(Int_t lLayer=0; lLayer<kLayers; lLayer++) {
    fCandidate[lLayer] = new TObjArray(kCandsPerPadrowMax, 0);
    for(Int_t lIndex=0; lIndex<kCandsPerPadrowMax; lIndex++)
      fUsedForFit[lLayer][lIndex] = FALSE;
  }

  fTrack = new TObjArray(kTracksPerPadrowMax, 0);
}

//_________________________________________________________________________
AliTRDgtuTMU::AliTRDgtuTMU(Int_t iSector, Int_t iChamber, Int_t iPadrow):TObject()
{
  //
  //  AliTRDgtuTMU default constructor with identification
  //

  fSector  = iSector;
  fChamber = iChamber;
  fPadrow  = iPadrow;

  for(Int_t lLayer=0; lLayer<kLayers; lLayer++) {
    fCandidate[lLayer] = new TObjArray(kCandsPerPadrowMax, 0);
    for(Int_t lIndex=0; lIndex<kCandsPerPadrowMax; lIndex++)
      fUsedForFit[lLayer][lIndex] = FALSE;
  }

  fTrack = new TObjArray(kTracksPerPadrowMax, 0);
}

//_________________________________________________________________________
AliTRDgtuTMU::~AliTRDgtuTMU()
{
  //
  //  AliTRDgtuTMU default destructor
  //
  //  free memory
  //

  for(Int_t lLayer=0; lLayer<kLayers; lLayer++) {
    fCandidate[lLayer]->Clear();
    delete fCandidate[lLayer];
  }

  fTrack->Delete();
  delete fTrack;
}

//_________________________________________________________________________
void AliTRDgtuTMU::AddCandidate(AliTRDgtuCandidate *iCandidate, Int_t iLayer)
{
  //
  //  add a candidate to the current Track Matching Unit. There is a
  //  candidate array for each layer.
  //

  if(fCandidate[iLayer]->GetEntries() >= kCandsPerPadrowMax) {
    DBG_W ( printf("WARNING: max. number of candidates per padrow is %d\n", kCandsPerPadrowMax); )
    return;
  }

  if((iCandidate->GetZPos() < 0) || (iCandidate->GetZPos() >= kPadrowsPerChamber)) {
    DBG_N ( printf("WARNING: ZPos is not in valid range (%d..%d\n", 0, kPadrowsPerChamber); )
    return;
  }

  fCandidate[iLayer]->AddLast(iCandidate);
}

//_________________________________________________________________________
void AliTRDgtuTMU::AddTrack(AliTRDgtuTrack *iTrack)
{
  //
  //  add a track to the current Track Matching Unit
  //

  if(fTrack->GetEntries() >= kTracksPerPadrowMax) {
    DBG_W ( printf("WARNING: max. number of tracks per padrow is %d\n", kTracksPerPadrowMax); )
    return;
  }

  fTrack->AddLast(iTrack);
}

//_________________________________________________________________________
Int_t AliTRDgtuTMU::NumberOfCandidates()
{
  //
  //  return the total number of candidates for the current TMU
  //

  Int_t lNumber = 0;

  for(Int_t lLayer=0; lLayer<kLayers; lLayer++) {
    if(fCandidate[lLayer]->At(0))
      lNumber += fCandidate[lLayer]->GetLast() + 1;
  }

  return(lNumber);
}

//_________________________________________________________________________
void AliTRDgtuTMU::FindTracksV0()
{
  //
  //  Track Matching Unit
  //

  AliTRDgtuCandidate *lCmpCandidate; // current compare candidate
  AliTRDgtuCandidate *lCandidate;    // current candidate
  AliTRDgtuTrack     *lTrack;        // probably found track

  DBG_A ( printf("**** start track matching in TMU (-, %d, %d, %d)\n", fSector, fChamber, fPadrow); )
  for(Int_t lCmpLayer=0; lCmpLayer<kCompareLayers; lCmpLayer++) {
    for(Int_t lCmpCandI=0; lCmpCandI<=fCandidate[lCmpLayer]->GetLast(); lCmpCandI++) {
      lCmpCandidate = (AliTRDgtuCandidate *)fCandidate[lCmpLayer]->At(lCmpCandI);
      fUsedForFit[lCmpLayer][lCmpCandI] = TRUE; // this candidate is used for a fit
      lTrack = new AliTRDgtuTrack();
      lTrack->AddCandidate(lCmpCandidate);      // the compare candidate is a fit by itself
      for(Int_t lLayer=0; lLayer<kLayers; lLayer++) {
        if(lLayer != lCmpLayer) {               // NO comparison within one layer
          for(Int_t lCandI=0; lCandI<=fCandidate[lLayer]->GetLast(); lCandI++) {
            lCandidate = (AliTRDgtuCandidate *)fCandidate[lLayer]->At(lCandI);
            if((!fUsedForFit[lLayer][lCandI]) && HitConditionCheck(lCmpCandidate, lCandidate)) {
	      //printf("%d\t%d\t%d\t%d\n", lCmpLayer, lCmpCandI, lLayer, lCandI);
              lTrack->AddCandidate(lCandidate);
              fUsedForFit[lLayer][lCandI] = TRUE;
	    }
	  }
	}
      }
      if(lTrack->NumberOfCandidates() >= kHitsToFitMin) {
        DBG_A ( printf("found %d fits -> \t*** TRACK ***\n", lTrack->NumberOfCandidates()); )
        lTrack->CalcTrack();
        AddTrack(lTrack);
      } else {
        DBG_A ( printf("found only %d fits -> sorry, no track\n", lTrack->NumberOfCandidates()); )
        delete lTrack;
      }
    }
  }
}

//_________________________________________________________________________
void AliTRDgtuTMU::OutputCandidates()
{
  //
  //  output the candidates for the current TMU
  //

  Int_t lIndex = 0;
  AliTRDgtuCandidate *lCandidate = NULL;

  DBG_N ( printf("\tLayer\tNo.\tRPos\tAngle\tZPos\tAmpl.\tTrQ\tRPosMid\n"); )
  for(Int_t lLayer=0; lLayer<kLayers; lLayer++) {
    DBG_N ( printf("\t%d", lLayer); )
    lIndex = 0;
    if(fCandidate[lLayer]->At(0))
      do {
        lCandidate = (AliTRDgtuCandidate *)fCandidate[lLayer]->At(lIndex);
        if(lIndex) DBG_N ( printf("\t%d", lLayer); )
        printf("\t%d\t(%d)\t%d\t(%d)\t%d\t%d\t%d\n", lIndex,
            lCandidate->GetRPos(),
            lCandidate->GetAngle(),
            lCandidate->GetZPos(),
            lCandidate->GetAmplitude(),
            lCandidate->GetTrQuality(),
            lCandidate->GetRPosMidlayer());
        lIndex++;
      } while (lIndex <= fCandidate[lLayer]->GetLast());
    else
      printf("\t-- no candidates --\n");
  }
}

//_________________________________________________________________________
void AliTRDgtuTMU::OutputTracks()
{
  //
  //  output the tracks for the current TMU
  //

  HEAD ( printf("Tracks of TMU (-, %d, %d, %d)\n", fSector, fChamber, fPadrow); )
  if(fTrack->GetLast() + 1) {
    for(Int_t lTrackI=0; lTrackI<=fTrack->GetLast(); lTrackI++) {
      ((AliTRDgtuTrack *)fTrack->At(lTrackI))->Output();
    }
  } else {
    DBG_N ( printf("-- no tracks --\n"); )
    DBG_N ( printf("\n"); )
  }
  DBG_N ( printf("\n"); )
}

//_________________________________________________________________________
void AliTRDgtuTMU::VisualCandidates2D(AliTRDgtuVisual *iVisual)
{
  //
  //  give an 2D visualisation of all candidates of the current TMU
  //

  Int_t lIndex = 0;
  AliTRDgtuCandidate *lCandidate = NULL;

  for(Int_t lLayer=0; lLayer<kLayers; lLayer++) {
    lIndex = 0;
    if(fCandidate[lLayer]->At(0))
      do {
        lCandidate = (AliTRDgtuCandidate *)fCandidate[lLayer]->At(lIndex);
        iVisual->DrawCandidate2D(lCandidate, lLayer);
        lIndex++;
      } while (lIndex <= fCandidate[lLayer]->GetLast());
  }
}

//_________________________________________________________________________
void AliTRDgtuTMU::VisualTracks2D(AliTRDgtuVisual *iVisual)
{
  //
  //  visualize 2D the tracks for the current TMU
  //

  if(fTrack->GetLast() + 1) {
    for(Int_t lTrackI=0; lTrackI<=fTrack->GetLast(); lTrackI++) {
      iVisual->DrawTrack2D((AliTRDgtuTrack *)fTrack->At(lTrackI));
      iVisual->DrawTrackCandProj2D((AliTRDgtuTrack *)fTrack->At(lTrackI));
    }
  }
}

//_________________________________________________________________________
Bool_t AliTRDgtuTMU::RangeCheck(Int_t iValue, Int_t iRef, Int_t iRange)
{
  //
  //  checks if a value is within a given range
  //

  if((iValue < (iRef - iRange)) || (iValue > (iRef + iRange)))
    return(FALSE);
  else
    return(TRUE);
}

//_________________________________________________________________________
Bool_t AliTRDgtuTMU::HitConditionCheck(AliTRDgtuCandidate *iCmpCandidate, AliTRDgtuCandidate *iCandidate)
{
  //
  //  check the hit condition of two candidates
  //

  if(RangeCheck(iCmpCandidate->GetAngle(), iCandidate->GetAngle(), kAngleDiff) &&
     //RangeCheck(iCmpCandidate->GetAmplitude(), iCandidate->GetAmplitude(), kAmplitudeDiff) &&
     RangeCheck(iCmpCandidate->GetRPosMidlayer(), iCandidate->GetRPosMidlayer(), kRPosMidlayerDiff))
    return(TRUE);
  else
    return(FALSE);
}
