/**************************************************************************
 * Copyright(c) 1998-1999, ALICE Experiment at CERN, All rights reserved. *
 *                                                                        *
 * Author: The ALICE Off-line Project.                                    *
 * Contributors are mentioned in the code where appropriate.              *
 *                                                                        *
 * Permission to use, copy, modify and distribute this software and its   *
 * documentation strictly for non-commercial purposes is hereby granted   *
 * without fee, provided that the above copyright notice appears in all   *
 * copies and that both the copyright notice and this permission notice   *
 * appear in the supporting documentation. The authors make no claims     *
 * about the suitability of this software for any purpose. It is          *
 * provided "as is" without express or implied warranty.                  *
 **************************************************************************/

/*
$Log: AliTRDgtuTrack.cxx,v $
Revision 1.1.1.1  2001/01/16 18:02:33  schneide
Import of TRDgtu

*/

///////////////////////////////////////////////////////////////////////////
//                                                                       //
//  Global Tracking Unit - Track class                                   //
//  This class represets a Track. All candidated which are involved in   //
//  this track are stored within the track                               //
//                                                                       //
//  module: (TRDgtu) Global Tracking Unit        author: Rolf Schneider  //
//                                                                       //
///////////////////////////////////////////////////////////////////////////

#include "AliTRDgtuConst.h"
#include "AliTRDgtuGlobal.h"
#include "AliTRDgtuCandidate.h"
#include "AliTRDgtuTrack.h"

ClassImp(AliTRDgtuTrack)
 
//_________________________________________________________________________
AliTRDgtuTrack::AliTRDgtuTrack():AliTRDgtuCandidate()
{
  //
  //  AliTRDgtuTrack default constructor
  //
  //  init an array of candidates, which are involved in the track
  //

  fCandidate = new TObjArray(kCandsPerTrackMax, 0);
}

//_________________________________________________________________________
AliTRDgtuTrack::~AliTRDgtuTrack()
{
  //
  //  AliTRDgtuTrack default destructor
  //
  //  free memory
  //

  fCandidate->Clear();
  delete fCandidate;
}

//_________________________________________________________________________
void AliTRDgtuTrack::AddCandidate(AliTRDgtuCandidate *iCandidate)
{
  //
  //  add a candidate to the array which include the candidates involved
  //  in the track
  //

  if(fCandidate->GetEntries() >= kCandsPerTrackMax) {
    DBG_W ( printf("WARNING: max. number of candidates per track is %d\n", kCandsPerTrackMax); )
    return;
  }

  fCandidate->AddLast(iCandidate);
}

//_________________________________________________________________________
void AliTRDgtuTrack::CalcTrack()
{
  //
  //  calculate the track parameters out of the involved candidates
  //

  if(NumberOfCandidates() < kHitsToFitMin) {
    DBG_N ( printf("ERROR: you tried to calculate a track with less than %d candidates\n", kHitsToFitMin); )
    return;
  }

  Int_t   lAngle        = 0;    // ave. angle of track
  Int_t   lAmplitude    = 0;    // ave. amplitude of track
  Int_t   lTrQuality    = 0;    // number for quality of transition radiation
  Int_t   lRPosMidlayer = 0;    // position in rphi direction of middle layer

  AliTRDgtuCandidate *lCandidate;

  for(Int_t lCandI=0; lCandI<=fCandidate->GetLast(); lCandI++) {
    lCandidate = (AliTRDgtuCandidate *)fCandidate->At(lCandI);
    lAngle        += lCandidate->GetAngle();
    lAmplitude    += lCandidate->GetAmplitude();
    lTrQuality    += lCandidate->GetTrQuality();
    lRPosMidlayer += lCandidate->GetRPosMidlayer();
  }

  lAngle        = (Int_t)(((Float_t)lAngle + 0.5) / NumberOfCandidates());
  lAmplitude    = (Int_t)(((Float_t)lAmplitude + 0.5) / NumberOfCandidates());
  lTrQuality    = (Int_t)(((Float_t)lTrQuality + 0.5) / NumberOfCandidates());
  lRPosMidlayer = (Int_t)(((Float_t)lRPosMidlayer + 0.5) / NumberOfCandidates());

  SetAngle(lAngle);
  SetAmplitude(lAmplitude);
  SetTrQuality(lTrQuality);
  SetRPosMidlayer(lRPosMidlayer);
}

//_________________________________________________________________________
Int_t AliTRDgtuTrack::NumberOfCandidates()
{
  //
  //  return the number of candidates involved in the current track
  //

  return(fCandidate->GetLast() + 1);
}

//_________________________________________________________________________
void AliTRDgtuTrack::Output()
{
  //
  //  output current track with involved candidates
  //

  AliTRDgtuCandidate *lCandidate;

  DBG_N ( printf("**** Track: \tAngle\tAmpl.\tTrQ.\tRPosMid\n"); )
  DBG_N ( printf("            \t%d\t%d\t%d\t%d\n", GetAngle(), GetAmplitude(),
      GetTrQuality(), GetRPosMidlayer()); )
  DBG_N ( printf("**** Cands: \tRpos\tAngle\tZPos\tAmpl.\tTrQ.\tRPosMid\n"); )

  for(Int_t lCandI=0; lCandI<=fCandidate->GetLast(); lCandI++) {
    lCandidate = (AliTRDgtuCandidate *)fCandidate->At(lCandI);
    DBG_N ( printf("            \t%d\t%d\t%d\t%d\t%d\t%d\n", lCandidate->GetRPos(), 
                                                             lCandidate->GetAngle(),
                                                             lCandidate->GetZPos(),
                                                             lCandidate->GetAmplitude(),
                                                             lCandidate->GetTrQuality(),
                                                             lCandidate->GetRPosMidlayer()); )
  }
  DBG_N ( printf("\n"); )    
}

