/**************************************************************************
 * Copyright(c) 1998-1999, ALICE Experiment at CERN, All rights reserved. *
 *                                                                        *
 * Author: The ALICE Off-line Project.                                    *
 * Contributors are mentioned in the code where appropriate.              *
 *                                                                        *
 * Permission to use, copy, modify and distribute this software and its   *
 * documentation strictly for non-commercial purposes is hereby granted   *
 * without fee, provided that the above copyright notice appears in all   *
 * copies and that both the copyright notice and this permission notice   *
 * appear in the supporting documentation. The authors make no claims     *
 * about the suitability of this software for any purpose. It is          *
 * provided "as is" without express or implied warranty.                  *
 **************************************************************************/

/*
$Log: AliTRDgtuVisual.cxx,v $
Revision 1.2  2001/01/30 18:48:27  schneide
Changes for AliRoot 3.04, Visualization of Tracklets, FindNextTrack added

Revision 1.1.1.1  2001/01/16 18:02:33  schneide
Import of TRDgtu

*/

///////////////////////////////////////////////////////////////////////////
//                                                                       //
//  Global Tracking Unit - Visual class                                  //
//  This class allows a simple 2D and 3D visualization of candidates     //
//  and  tracks.                                                         //
//                                                                       //
//  module: (TRDgtu) Global Tracking Unit        author: Rolf Schneider  //
//                                                                       //
///////////////////////////////////////////////////////////////////////////

#include <stdlib.h>

#include <TCanvas.h>
#include <TBox.h>
#include <TLine.h>
#include <TArrow.h>
#include <TBRIK.h>
#include <TPaveText.h>
#include <TNode.h>

#include <TObjArray.h>

#include "AliTRDgtuConst.h"
#include "AliTRDgtuGlobal.h"
#include "AliTRDgtuCandidate.h"
#include "AliTRDgtuTrack.h"
#include "AliTRDgtuVisual.h"

ClassImp(AliTRDgtuVisual)

const Int_t kPadXMin = 0;
const Int_t kPadXMax = 1500;

#define  MIDLAYER_R    (((fLayer_r[kLayers-1] - fLayer_r[0])/2) + fLayer_r[0] + fLayer_h[0])

//_________________________________________________________________________
AliTRDgtuVisual::AliTRDgtuVisual():TObject()
{
  //
  //  AliTRDgtuVisual default constructor
  //
  //  calculate the detector dimensions
  //

  for(Int_t lLayer=0; lLayer<kLayers; lLayer++) { 
    fLayer_r[lLayer] = 2940.0F + (lLayer * 123.33F); // radius
    fLayer_w[lLayer] = 996.0F + (lLayer * 44.4F);    // width
    fLayer_h[lLayer] = 35.0F;                        // height
    fLayer_z[lLayer] = 1200.0F;                      // length
  }

  fCanvas2D = NULL;
  fPad2D    = NULL;
  fCanvas3D = NULL;
  fPad3D    = NULL;
}

//_________________________________________________________________________
AliTRDgtuVisual::~AliTRDgtuVisual()
{
  //
  //  AliTRDgtuVisual default destructor
  //

}

//_________________________________________________________________________
TPad *AliTRDgtuVisual::Init2D()
{
  //
  //  Init Canvas and Pad for 2D visualization
  //

  DBG_I ( printf("**** initialize 2D visualization\n"); )

  fCanvas2D = new TCanvas("Canvas2D", "GTU -- 2D-Visualization", 100, 10, 500, 500);
  fCanvas2D->Draw();

  fPad2D = new TPad("Pad2D", "Pad2D", 0.02F, 0.02F, 0.98F, 0.98F, 18, 2, 1);
  fPad2D->Range(kPadXMin, fLayer_r[0]-150, kPadXMax, fLayer_r[kLayers-1]+250);
  fPad2D->Draw();
  fPad2D->cd();
  return(fPad2D);
}

//_________________________________________________________________________
TPad *AliTRDgtuVisual::Init3D()
{
  //
  //  Init Canvas and Pad 3D visualization
  //

  DBG_I ( printf("**** initialize 3D visualization\n"); )

  fCanvas3D = new TCanvas("Canvas3D", "GTU -- 3D-Visualization", 100, 10, 500, 500);
  fCanvas3D->Draw();

  fPad3D = new TPad("Pad3D", "Pad3D", 0.02F, 0.02F, 0.98F, 0.98F, 18, 2, 1);
  fPad3D->Draw();
  fPad3D->cd();
  return(fPad3D);
}

//_________________________________________________________________________
void AliTRDgtuVisual::DrawBlock2D()
{
  //
  //  draw a front view of a detector block (2D)
  //

  DBG_A ( printf("draw block in 2D view\n"); )

  if(!fPad2D) {
    DBG_N ( printf("ERROR: you tried to draw 2D block visualization without initialization\n"); )
    return;
  }

  Float_t x1, y1, x2, y2;
  Float_t pad_xmid = (kPadXMax - kPadXMin)/2;
  TBox *LayerBox = new TBox();

  for(Int_t lLayer=0; lLayer<kLayers; lLayer++) {
    x1 = pad_xmid - (fLayer_w[lLayer]/2);
    x2 = pad_xmid + (fLayer_w[lLayer]/2);
    y1 = fLayer_r[lLayer];
    y2 = fLayer_r[lLayer] + fLayer_h[lLayer];

    LayerBox->SetFillStyle(1001);
    LayerBox->SetFillColor(20);
    LayerBox->DrawBox(x1, y1, x2, y2);
    LayerBox->SetFillStyle(0);
    LayerBox->SetLineColor(15);
    LayerBox->SetLineWidth(1);
    LayerBox->DrawBox(x1, y1, x2, y2);
  }

  x1 = pad_xmid - (fLayer_w[kLayers-1]/2) - 50;
  x2 = pad_xmid + (fLayer_w[kLayers-1]/2) + 50;
  y1 = y2 = ((fLayer_r[kLayers-1] - fLayer_r[0])/2) + fLayer_r[0] + fLayer_h[0];

  TLine *Midlayer = new TLine(x1, y1, x2, y2);
  Midlayer->Draw();
}

//_________________________________________________________________________
void AliTRDgtuVisual::DrawBlock3D()
{
  //
  //  draw a view of a detector block (3D)
  //

  DBG_A ( printf("draw block in 3D view\n"); )

  if(!fPad3D) {
    DBG_N ( printf("ERROR: you tried to draw 3D block visualization without initialization\n"); )
    return;
  }

  Int_t lLayer;
  Char_t *lLayerStr[kLayers+1]; // inclusive virtual middle layer

  //
  // Create Layer Shapes
  //
  TBRIK *lLayerBlock[kLayers+1];
  for(lLayer=0; lLayer<kLayers; lLayer++) {
    lLayerStr[lLayer] = NumberString("LAYER",lLayer);
    lLayerBlock[lLayer] = new TBRIK(lLayerStr[lLayer],
                                    NumberString("Layer",lLayer),
                                    "void",
                                    fLayer_w[lLayer]/2,
                                    fLayer_h[lLayer]/2,
                                    fLayer_z[lLayer]/2);
    lLayerBlock[lLayer]->SetLineColor(9);
  }
  lLayerStr[kLayers] = NumberString("LAYER",lLayer);
  lLayerBlock[kLayers] = new TBRIK(lLayerStr[kLayers],
                                   NumberString("Layer",kLayers),
                                   "void",
                                   (fLayer_w[5]/2)+100,
                                   1.0,
                                   (fLayer_z[5]/2)+100);
  lLayerBlock[kLayers]->SetLineColor(3);

  //
  // Create Nodes
  //
  TNode *lNodeBlock[kLayers+1];
  float r_origin = ((fLayer_r[kLayers-1]-fLayer_r[0])/2)+fLayer_r[0];
  lNodeBlock[kLayers] = new TNode(NumberString("NODE",kLayers),NumberString("NODE",kLayers),lLayerStr[kLayers]);
  lNodeBlock[kLayers]->cd();
  float r_offset = fLayer_h[0]/2;
  for(lLayer=0; lLayer<kLayers; lLayer++) {
    lNodeBlock[lLayer] = new TNode(NumberString("NODE",lLayer),
                                   NumberString("NODE",lLayer),
                                   lLayerStr[lLayer],
                                   0, fLayer_r[lLayer]-r_origin-r_offset, 0);
  }
  lNodeBlock[kLayers]->cd();
  lNodeBlock[kLayers]->Draw();
}

//_________________________________________________________________________
void AliTRDgtuVisual::DrawCandidate2D(AliTRDgtuCandidate *iCandidate, Int_t iLayer)
{
  //
  //  draw a candidate into the current 2D view
  //

  DBG_A ( printf("draw candidate in 2D view\n"); )

  if(!fPad2D) {
    DBG_N ( printf("ERROR: you tried to draw 2D candidate visualization without initialization\n"); )
    return;
  }

  const Float_t lCandLength = 20;

  Int_t lRPos;
  Float_t x1, y1, x2, y2, angle;

  Int_t pad_xmid = kPadXMin + ((kPadXMax - kPadXMin)/2);

  TLine *CandLine = new TLine();
  CandLine->SetLineWidth(2);
  CandLine->SetLineColor(2);

  lRPos = iCandidate->GetRPos();
  x1 = x2 = pad_xmid + ((lRPos-(kRPosMax>>1)) * fLayer_w[iLayer] / kRPosMax);
  y1 = fLayer_r[iLayer] + fLayer_h[iLayer] - lCandLength;
  y2 = fLayer_r[iLayer] + fLayer_h[iLayer] + lCandLength;
  angle = iCandidate->GetAngle() - (float)(kAngleMax>>1);
  x1 -= (float)(tan(kAngleMaxRes) * (-lCandLength)) * angle;
  x2 -= (float)(tan(kAngleMaxRes) * (+lCandLength)) * angle;
  CandLine->DrawLine(x1, y1, x2, y2);
}

//_________________________________________________________________________
void AliTRDgtuVisual::DrawTracklet2D(AliTRDfitTracklet *iTracklet, Int_t iLayer)
{
  //
  //  draw a tracklet into the current 2D view
  //

  DBG_A ( printf("draw tracklet in 2D view\n"); )

  if(!fPad2D) {
    DBG_N ( printf("ERROR: you tried to draw 2D tracklet visualization without initialization\n"); )
    return;
  }

}

//_________________________________________________________________________
void AliTRDgtuVisual::DrawCandidate3D(AliTRDgtuCandidate *iCandidate, Int_t iLayer)
{
  //
  //  draw a candidate into the current 3D view
  //

  DBG_A ( printf("draw candidate in 3D view\n"); )

  if(!fPad3D) {
    DBG_N ( printf("ERROR: you tried to draw 3D candidate visualization without initialization\n"); )
    return;
  }

   Float_t lPoint[]={0,-500,0,0,500,0};
   TPolyLine3D *lCandLine;
   lCandLine = new TPolyLine3D(2, lPoint, (Option_t *)"");
   lCandLine->SetLineColor(2);
   lCandLine->SetLineWidth(3);
   Float_t layer_off = -(fLayer_r[kLayers-1]-fLayer_r[0])/2;
   Float_t layer_dist = fLayer_r[1]-fLayer_r[0];

   Float_t calc_r_pos = (iCandidate->GetRPos() * (fLayer_w[iLayer]/2047)) - (fLayer_w[iLayer]/2);
   lPoint[0] = calc_r_pos;    // r-pos begin
   lPoint[3] = calc_r_pos;    // r-pos end
   lPoint[1] = layer_off+(iLayer*layer_dist)-60;   // layer begin
   lPoint[4] = layer_off+(iLayer*layer_dist)+30;    // layer end
   Float_t calc_z_pos = (iCandidate->GetZPos() * (fLayer_z[iLayer]/23)) - (fLayer_z[iLayer]/2);
   lPoint[2] = calc_z_pos;    // z-pos begin
   lPoint[5] = calc_z_pos;    // z-pos end
   lCandLine->DrawPolyLine(2, lPoint, (Option_t *)"");
}

//_________________________________________________________________________
void AliTRDgtuVisual::DrawTracklet3D(AliTRDgeometry *TRDgeometry, AliTRDfitTracklet *iTracklet, Int_t iPlane, Int_t iChamber, Int_t iSector, Int_t iPadrow)
{
  //
  //  draw a candidate into the current 3D view
  //

  DBG_A ( printf("draw candidate in 3D view\n"); )

  if(!fPad3D) {
    DBG_N ( printf("ERROR: you tried to draw 3D candidate visualization without initialization\n"); )
    return;
  }

  Float_t lPoint[]={0,-500,0,0,500,0};
  TPolyLine3D *lfitline;
  lfitline = new TPolyLine3D(2, lPoint, (Option_t *)"");
  lfitline->SetLineColor(3);
  lfitline->SetLineWidth(3);
  Float_t layer_off = -(fLayer_r[kLayers-1]-fLayer_r[0])/2;
  Float_t layer_dist = fLayer_r[1]-fLayer_r[0];

  Float_t calc_r_pos = (iTracklet->GetfRPos() * (fLayer_w[iPlane]/(TRDgeometry->GetColMax(iPlane)*127.0))) - (fLayer_w[iPlane]/2);
  lPoint[0] = calc_r_pos;    // r-pos begin
  lPoint[3] = calc_r_pos - ((iTracklet->GetfAngle()-128.0)/128.0) * (fLayer_w[iPlane]/TRDgeometry->GetColMax(iPlane));    // r-pos end
  lPoint[1] = layer_off+(iPlane*layer_dist)-30;   // layer begin (-60)
  lPoint[4] = layer_off+(iPlane*layer_dist);    // layer end  (+30)
  Float_t calc_z_pos = (iPadrow * (fLayer_z[iPlane]/TRDgeometry->GetRowMax(iPlane, iChamber, iSector))) - (fLayer_z[iPlane]/2);
  lPoint[2] = calc_z_pos;    // z-pos begin
  lPoint[5] = calc_z_pos;    // z-pos end
  lfitline->DrawPolyLine(2, lPoint, (Option_t *)"");
}

//_________________________________________________________________________
void AliTRDgtuVisual::DrawTrack2D(AliTRDgtuTrack *iTrack)
{
  //
  //  draw a Track into the current 2D view
  //

  DBG_A ( printf("draw track in 2D view\n"); )

  if(!fPad2D) {
    DBG_N ( printf("ERROR: you tried to draw 2D track visualization without initialization\n"); )
    return;
  }

  Float_t x1, y1, x2, y2, pad_xmid, angle;

  pad_xmid = kPadXMin + ((kPadXMax - kPadXMin)/2);

  x1 = x2 = pad_xmid + ((iTrack->GetRPosMidlayer()-(kRPosMidlayerMax>>1))
            * (fLayer_w[0] + ((fLayer_w[kLayers-1] - fLayer_w[0]) / 2))
            / kRPosMidlayerMax);
  y1 = fLayer_r[0] - 50;
  y2 = fLayer_r[kLayers-1] + 150;
  angle = iTrack->GetAngle() - (Float_t)(kAngleMax>>1);
  x1 -= (Float_t)(tan(kAngleMaxRes) * (y1 - MIDLAYER_R)) * angle;
  x2 -= (float)(tan(kAngleMaxRes) * (y2 - MIDLAYER_R)) * angle;
  TArrow *midlayer = new TArrow(x1, y1, x2, y2, 0.02F);
  midlayer->SetLineColor(3);
  midlayer->SetLineWidth(2);
  midlayer->Draw();
}

//_________________________________________________________________________
void AliTRDgtuVisual::DrawTrackCandProj2D(AliTRDgtuTrack *iTrack)
{
  //
  //  draw a projection of Candidates of Track to middle layer
  //

  DBG_A ( printf("draw track candidates projection in 2D view\n"); )

  if(!fPad2D) {
    DBG_N ( printf("ERROR: you tried to draw 2D track candidates projection visualization without initialization\n"); )
    return;
  }

  const Float_t lCandLength = 20;
  Int_t pad_xmid = kPadXMin + ((kPadXMax - kPadXMin)/2);

  TObjArray *lCandidatesArray = iTrack->GetCandidates();
  AliTRDgtuCandidate *lCandidate;
  TLine *ProjCandLine = new TLine();
  ProjCandLine->SetLineWidth(2);
  ProjCandLine->SetLineColor(4);

  Float_t x1, x2, y1, y2, angle;

  for(Int_t lCandI=0; lCandI<=lCandidatesArray->GetLast(); lCandI++) {
    lCandidate = (AliTRDgtuCandidate *)lCandidatesArray->At(lCandI);
    y1 = y2 = ((fLayer_r[kLayers-1] - fLayer_r[0])/2) + fLayer_r[0] + fLayer_h[0];
    y1 -= lCandLength;
    y2 += lCandLength;
    x1 = x2 = pad_xmid + ((lCandidate->GetRPosMidlayer()-(kRPosMidlayerMax>>1))
            * (fLayer_w[0] + ((fLayer_w[kLayers-1] - fLayer_w[0]) / 2))
            / kRPosMidlayerMax);
    angle = lCandidate->GetAngle() - (float)(kAngleMax>>1);
    x1 -= (float)(tan(kAngleMaxRes) * (-lCandLength)) * angle;
    x2 -= (float)(tan(kAngleMaxRes) * (+lCandLength)) * angle;
    ProjCandLine->DrawLine(x1, y1, x2, y2);
  }
}

//_________________________________________________________________________
void AliTRDgtuVisual::DrawStats2D(Int_t iPlane, Int_t iSector, Int_t iChamber, Int_t iPadrow)
{
  //
  //  draw stats into the current 2D view
  //

  DBG_A ( printf("draw stats in 2D view\n"); )

  if(!fPad2D) {
    DBG_N ( printf("ERROR: you tried to draw stats without visualization initialization\n"); )
    return;
  }

  Char_t hstr[80];

  TPaveText *lStats = new TPaveText(kPadXMax-350, fLayer_r[kLayers-1]+100, kPadXMax-50, fLayer_r[kLayers-1]+230);

  if(iPlane >= 0) {
  sprintf(hstr, "Plane: %d", iPlane);
  lStats->InsertText(hstr);
  }
  if(iSector >= 0) {
  sprintf(hstr, "Sector: %d", iSector);
  lStats->InsertText(hstr);
  }
  if(iChamber >= 0) {
  sprintf(hstr, "Chamber: %d", iChamber);
  lStats->InsertText(hstr);
  }
  if(iPadrow >= 0) {
  sprintf(hstr, "Padrow: %d", iPadrow);
  lStats->InsertText(hstr);
  }
  lStats->SetTextAlign(12);
  lStats->Draw();
}

//_________________________________________________________________________
Char_t *AliTRDgtuVisual::NumberString(Char_t *iStr, Int_t iNumber)
{
  //
  //  create and alloc memory for a string with a number appended
  //

  Char_t *lStr, lHStr[80];

  sprintf(lHStr, "%s%d", iStr, iNumber);
  lStr = (Char_t *)malloc(strlen(lHStr)+2);
  strcpy(lStr, lHStr);

  return(lStr);
}

