///////////////////////////////////////////////////////////////////
////// CONVERT POSITION TO DEFLECTION (REAL)
///////////////////////////////////////////////////////////////////

Float_t Pos2Defl(AliTRDgeometry *TRDgeometry, Float_t xinner, Float_t xouter, Float_t tinner, Float_t touter, Int_t plane, Int_t column)
{
  Float_t lColPadSize = TRDgeometry->GetColPadSize(plane);

  Float_t d     = (xouter - xinner) * lColPadSize;
  ((touter-tinner) == 0.0) ? d = 99999.0 : d *= TRDgeometry->GetTimeMax() / (touter-tinner);
  Float_t kR    = TRDgeometry->GetTime0(plane);
  Float_t b     = TRDgeometry->GetCol0(plane) + (column * lColPadSize);
  Float_t g     = (b * 3.0) / kR;

  Float_t defl  = d - g;
  //printf("Col0 %f, padsize %f, col %d, kR %f, b %f, d %f, g %f\n", TRDgeometry->GetCol0(plane), lColPadSize, column, kR, b, d, g);
  return(10.0 * defl); // conversion to [mm]
}

///////////////////////////////////////////////////////////////////
////// CONVERT FIT-ANGLE TO DEFLECTION (FIT)
///////////////////////////////////////////////////////////////////

Float_t Angle2Defl(AliTRDgeometry *TRDgeometry, Int_t iFitAngle, Int_t plane, Int_t column)
{
  Float_t lColPadSize = TRDgeometry->GetColPadSize(plane);

  Float_t d     = (Float_t)(-(iFitAngle - 128.0) * lColPadSize) / 128.0;
  Float_t kR    = TRDgeometry->GetTime0(plane);
  Float_t b     = TRDgeometry->GetCol0(plane) + (column * lColPadSize);
  Float_t g     = (b * 3.0) / kR;

  Float_t defl  = d - g;
  //printf("Col0 %f, kR %f, b %f, d %f, g %f\n", TRDgeometry->GetCol0(plane), kR, b, d, g);
  return(10.0 * defl); // conversion to [mm]
}

///////////////////////////////////////////////////////////////////
////// CONVERT TRANSVERSE MOMENTUM [GeV/c] TO DEFLECTION [mm]
///////////////////////////////////////////////////////////////////

Float_t Pt2Defl(AliTRDgeometry *TRDgeometry, Float_t pt, Float_t charge, Int_t plane, Int_t column)
{
  Float_t lColPadSize = TRDgeometry->GetColPadSize(plane);

  Float_t r     = (pt * 100.0) / (0.3 * 0.4);
  Float_t kR    = TRDgeometry->GetTime0(plane);
  Float_t b     = TRDgeometry->GetCol0(plane) + (column * lColPadSize);
  Float_t a     = TMath::Sqrt(kR*kR + b*b);
  Float_t u     = a / (2.0 * r); if(u>1) u=1; if(u<-1) u=-1;
  Float_t alpha = charge * TMath::ASin(u);
  Float_t beta  = TMath::ATan(b / kR);
  Float_t gamma = (alpha >=0 ) ? (alpha - beta) : (-alpha + beta);
  Float_t g     = (b * 3.0) / kR;
  Float_t d     = TMath::Tan(gamma) * 3.0;

  Float_t defl  = (charge >= 0.0) ? -(d+g) : -(-d+g);
  //printf("pt %6.3f, charge %5.1f, plane %d, column %3d, r %10.3f, kR %8.3f, b %8.3f, a %8.3f, alpha %7.3f, beta %7.3f, gamma %7.3f, g %7.3f, d %7.3f, defl %7.3f\n", pt, charge, plane, column, r, kR, b, a, alpha, beta, gamma, g, d, defl);
  return(10.0 * defl); // conversion to [mm]
}

///////////////////////////////////////////////////////////////////
////// CONVERT DEFLECTION [mm] TO TRANSVERSE MOMENTUM [GeV/c]
///////////////////////////////////////////////////////////////////

Float_t Defl2Pt(AliTRDgeometry *TRDgeometry, Float_t defl, Int_t plane, Int_t column)
{
  Float_t lColPadSize = TRDgeometry->GetColPadSize(plane);

  if(defl == 0.0) return(10000.0);

  Float_t kR    = TRDgeometry->GetTime0(plane);
  Float_t b     = TRDgeometry->GetCol0(plane) + (column * lColPadSize);
  Float_t a     = TMath::Sqrt(kR*kR + b*b);
  Float_t g     = (b * 3.0) / kR;
  Float_t d     = (defl/10.0) + g;
  Float_t gamma = TMath::ATan(d / 3.0);
  Float_t beta  = TMath::ATan(b / kR);
  Float_t alpha = beta - gamma;
  Float_t r     = a / (2.0 * TMath::Sin(alpha));

  r *= (alpha<0) ? -1 : 1;

  Float_t pt    = (r * 0.3 * 0.4) / 100.0;
  //printf("pt %6.3f, charge %5.1f, plane %d, column %3d, r %10.3f, kR %8.3f, b %8.3f, a %8.3f, alpha %7.3f, beta %7.3f, gamma %7.3f, g %7.3f, d %7.3f, defl %7.3f\n", pt, 0.0, plane, column, r, kR, b, a, alpha, beta, gamma, g, d, defl);

  return(pt);
}

///////////////////////////////////////////////////////////////////
////// CALC ERROR IN [MM] (FIT)
///////////////////////////////////////////////////////////////////

Float_t amps2error(AliTRDgeometry *TRDgeometry, Int_t ampL, Int_t ampC, Int_t ampR, Int_t plane)
{
  Float_t dLCR = 0.5;
  Float_t lColPadSize = TRDgeometry->GetColPadSize(plane);

  Float_t diff  = ampR - ampL;
  Float_t diff2 = diff*diff;
  Float_t sum   = ampL + ampC + ampR;
  Float_t sum2  = sum*sum;

  Float_t error = (dLCR/sum2) * TMath::Sqrt((2*sum2) + (3*diff2));

  return(10.0 * lColPadSize * error); // conversion to [mm]
}

///////////////////////////////////////////////////////////////////
////// CONVERT OFFSET TO MM (FIT)
///////////////////////////////////////////////////////////////////

Float_t offset2mm(AliTRDgeometry *TRDgeometry, Int_t offset, Int_t plane)
{
  Float_t lColPadSize = TRDgeometry->GetColPadSize(plane);

  return((Float_t)(10.0 * lColPadSize * (offset - 63.0)) / 127.0);
}

///////////////////////////////////////////////////////////////////
////// CONVERT COLUMN TO MM (FIT)
///////////////////////////////////////////////////////////////////

Float_t column2mm(AliTRDgeometry *TRDgeometry, Int_t column, Int_t offset, Int_t plane)
{
  Float_t lColPadSize = TRDgeometry->GetColPadSize(plane);

  return(10.0 * lColPadSize * (column + 0.5) + offset2mm(TRDgeometry, offset, plane));
}

///////////////////////////////////////////////////////////////////
////// GET ANGLE COL [PAD] (REAL)
///////////////////////////////////////////////////////////////////

Float_t getanglecol(AliTRDgeometry *TRDgeometry, AliTRDgtuRealTrack *RT, Int_t plane)
{
  Float_t timediff = RT->GetExEndTimebin(plane)-RT->GetExStartTimebin(plane);
  if(timediff == 0.0) {
    Float_t realangle = 999999.9;
    printf("WARNING: division by zero in getanglecol of stat_macros.C\n");
  }
  else
    Float_t realangle = (RT->GetExEndColumn(plane)-RT->GetExStartColumn(plane))/timediff;

  return(realangle);
}

///////////////////////////////////////////////////////////////////
////// GET START COLUMN (TIMEBIN 0) [PAD] (REAL)
///////////////////////////////////////////////////////////////////

Float_t getstartcol(AliTRDgeometry *TRDgeometry, AliTRDgtuRealTrack *RT, Int_t plane)
{
  Float_t realangle = getanglecol(TRDgeometry, RT, plane);
  Float_t realcol0 = RT->GetExStartColumn(plane) - (RT->GetExStartTimebin(plane)*realangle);

  return(realcol0);
}

///////////////////////////////////////////////////////////////////
////// GET END COLUMN (LAST TIMEBIN) [PAD] (REAL)
///////////////////////////////////////////////////////////////////

Float_t getendcol(AliTRDgeometry *TRDgeometry, AliTRDgtuRealTrack *RT, Int_t plane)
{
  Int_t lTimeMax      = TRDgeometry->GetTimeMax();
  Float_t realangle   = getanglecol(TRDgeometry, RT, plane);
  Float_t lastrealcol = RT->GetExEndColumn(plane) + ((lTimeMax-RT->GetExEndTimebin(plane))*realangle);

  return(lastrealcol);
}

///////////////////////////////////////////////////////////////////
////// GET ANGLE ROW [PAD] (REAL)
///////////////////////////////////////////////////////////////////

Float_t getanglerow(AliTRDgeometry *TRDgeometry, AliTRDgtuRealTrack *RT, Int_t plane)
{
  Float_t realangle = (RT->GetExEndPadrow(plane)-RT->GetExStartPadrow(plane))/(RT->GetExEndTimebin(plane)-RT->GetExStartTimebin(plane));

  return(realangle);
}

///////////////////////////////////////////////////////////////////
////// GET START PADROW (TIMEBIN 0) [PAD] (REAL)
///////////////////////////////////////////////////////////////////

Float_t getstartrow(AliTRDgeometry *TRDgeometry, AliTRDgtuRealTrack *RT, Int_t plane)
{
  Float_t realangle = getanglerow(TRDgeometry, RT, plane);
  Float_t realrow0 = RT->GetExStartPadrow(plane) - (RT->GetExStartTimebin(plane)*realangle);

  return(realrow0);
}

///////////////////////////////////////////////////////////////////
////// GET END PADROW (LAST TIMEBIN) [PAD] (REAL)
///////////////////////////////////////////////////////////////////

Float_t getendrow(AliTRDgeometry *TRDgeometry, AliTRDgtuRealTrack *RT, Int_t plane)
{
  Int_t lTimeMax      = TRDgeometry->GetTimeMax();
  Float_t realangle   = getanglerow(TRDgeometry, RT, plane);
  Float_t lastrealrow = RT->GetExEndPadrow(plane) + ((lTimeMax-RT->GetExEndTimebin(plane))*realangle);

  return(lastrealrow);
}

///////////////////////////////////////////////////////////////////
////// PADROW2ANGLE (REAL)
///////////////////////////////////////////////////////////////////
Float_t padrow2angle(AliTRDgeometry *TRDgeometry, Int_t iDet, Float_t padrow)
{
  // return angle in degrees !!!
  Int_t plane = TRDgeometry->GetPlane(iDet);
  Int_t chamber = TRDgeometry->GetChamber(iDet);
  Int_t sector = TRDgeometry->GetSector(iDet);
  Float_t zpos = TRDgeometry->GetRow0(plane, chamber, sector) + (padrow*TRDgeometry->GetRowPadSize(plane, chamber, sector));
  Float_t rpos =  TRDgeometry->GetTime0(plane);
  Float_t theta = (180.0*TMath::ATan(zpos/rpos))/TMath::Pi();

  return(90.0 - theta); // conversion to aliroot coordinates
}

///////////////////////////////////////////////////////////////////
////// TRACKLET2REF (FIT)
///////////////////////////////////////////////////////////////////
Float_t tracklet2refphi(AliTRDgeometry *TRDgeometry, AliTRDfitTracklet *tracklet, Int_t iDet, Float_t column)
{
  Float_t lColPadSize = TRDgeometry->GetColPadSize(plane);
  Float_t offset = tracklet->GetfRPos();
  Float_t phipos = column2mm(TRDgeometry, column, offset, plane);
  Float_t rpos = TRDgeometry->GetTime0(plane);
  Float_t refrpos = TRDgeometry->GetTime0(TRDgeometry->Nplan()-1) - 
}

///////////////////////////////////////////////////////////////////
////// FINDNEXTTRACK (REAL) [mm]
///////////////////////////////////////////////////////////////////
Int_t FindNextTrack(AliTRDgeometry *TRDgeometry, AliTRDgtuReal *Real, Int_t trackno, Int_t det, Int_t padrow, Int_t nParticles) {
  Float_t minDistance = 1000000.0;
  Float_t minTrack = -1;

  Int_t plane = TRDgeometry->GetPlane(det);
  Float_t trackpos = getstartcol(TRDgeometry, Real->GetRealTrack(trackno), plane);

  for(Int_t track=0; track<nParticles; track++) {
    if(track == trackno) continue;
    AliTRDgtuRealTrack *RT = Real->GetRealTrack(track);
    Int_t lDet = TRDgeometry->GetDetector(plane, RT->GetChamber(), RT->GetSector());
    if(lDet != det) continue;
    Int_t comp_padrow = RT->GetExStartPadrow(plane);
    if((comp_padrow != padrow) || (comp_padrow == -1)) continue;
    Float_t comp_pos = getstartcol(TRDgeometry, RT, plane);
    Float_t distance = TMath::Abs(trackpos-comp_pos);
    if(distance < minDistance) {
      minDistance = distance;
      minTrack = track;
    }
  }

  Float_t lColPadSize = TRDgeometry->GetColPadSize(plane);

  return(10.0 * lColPadSize * minDistance);
}

