/**************************************************************************
 * Copyright(c) 1998-1999, ALICE Experiment at CERN, All rights reserved. *
 *                                                                        *
 * Author: The ALICE Off-line Project.                                    *
 * Contributors are mentioned in the code where appropriate.              *
 *                                                                        *
 * Permission to use, copy, modify and distribute this software and its   *
 * documentation strictly for non-commercial purposes is hereby granted   *
 * without fee, provided that the above copyright notice appears in all   *
 * copies and that both the copyright notice and this permission notice   *
 * appear in the supporting documentation. The authors make no claims     *
 * about the suitability of this software for any purpose. It is          *
 * provided "as is" without express or implied warranty.                  *
 **************************************************************************/

/*
$Log:$
*/

///////////////////////////////////////////////////////////////////////////
//                                                                       //
//  TRAP simulation - Filter class                                       //
//  This class represents one filter channel. The channels are combinded //
//  in the TRAP module (AliTRDtrap)                                      //
//                                                                       //
//  order: NonLinearity                                                  //
//         Pedestal                                                      //
//         Gain                                                          //
//         Tail                                                          //
//         Crosstalk                                                     //
//                                                                       //
//  module: (TRDtrap) TRAP simulation model      author: Rolf Schneider  //
//                                                                       //
///////////////////////////////////////////////////////////////////////////

#include <iostream>
#include "AliTRDgeometry.h"
#include "AliTRDtrap.h"
#include "AliTRDtrapConst.h"
#include "AliTRDtrapFilter.h"

ClassImp(AliTRDtrapFilter)
 
//_________________________________________________________________________
AliTRDtrapFilter::AliTRDtrapFilter():TObject()
{
  //
  //  AliTRDtrapFilter default constructor
  //

  fTRAP1       = TRUE;

  fChannel     = XXX;
  fChamber     = XXX;
  fMCM         = XXX;
  fPadrow      = XXX;
  fPlane       = XXX;
  fSector      = XXX;

  fFilterLeft  = NULL;
  fFilterRight = NULL;
  fADCvalue    = NULL;
  fTAILvalue   = NULL;
  fCTvalue     = NULL;
  fNLvalue     = NULL;
  fPDvalue     = NULL;
  fGAINvalue   = NULL;
  fTimeMax     = 30;

  fNLped       = NULL;
  fPedestal    = 0;
  fPedestalDyn = 0;
  fGainFactor  = 0;
  fGainPedestal = 0;
  fAlpha_l     = 249;
  fLambda_l    = 285;
  fLambda_s    = 211;
  fTailPed     = 0;
  fCtPed       = 0;

  fC0          = +29; // +0.02910888
  fC1          = -44; // -0.04309046
  fC2          = -26; // -0.02579803
  fC3          = +74; // +0.07202855
  fC4          = -17; // -0.01659233

}

//_________________________________________________________________________
AliTRDtrapFilter::AliTRDtrapFilter(Int_t iPlane, Int_t iSector, Int_t iChamber, Int_t iPadrow, Int_t iMCM, Int_t iChannel):TObject()
{
  //
  //  AliTRDtrapFilter constructor with identification numbers
  //

  fTRAP1       = TRUE;

  fChannel     = iChannel;
  fChamber     = iChamber;
  fMCM         = iMCM;
  fPadrow      = iPadrow;
  fPlane       = iPlane;
  fSector      = iSector;

  fFilterLeft  = NULL;
  fFilterRight = NULL;
  fADCvalue    = NULL;
  fTAILvalue   = NULL;
  fCTvalue     = NULL;
  fNLvalue     = NULL;
  fPDvalue     = NULL;
  fGAINvalue   = NULL;
  fTimeMax     = 30;

  fNLped       = NULL;
  fPedestal    = 0;
  fPedestalDyn = 0;
  fGainFactor  = 0;
  fGainPedestal = 0;
  fAlpha_l     = 249;
  fLambda_l    = 285;
  fLambda_s    = 211;
  fTailPed     = 0;
  fCtPed       = 0;

  fC0          = +29; // +0.02910888
  fC1          = -44; // -0.04309046
  fC2          = -26; // -0.02579803
  fC3          = +74; // +0.07202855
  fC4          = -17; // -0.01659233

}

//_________________________________________________________________________
AliTRDtrapFilter::~AliTRDtrapFilter()
{
  //
  //  AliTRDtrapFilter default destructor
  //

  if(fADCvalue)  delete fADCvalue;
  if(fTAILvalue) delete fTAILvalue;
  if(fCTvalue)   delete fCTvalue;
  if(fNLvalue)   delete fNLvalue;
  if(fPDvalue)   delete fPDvalue;
  if(fGAINvalue) delete fGAINvalue;

  if(fNLped)     delete fNLped;

}

//_________________________________________________________________________
void AliTRDtrapFilter::CrosstalkCancellation()
{
  //
  //  Crosstalk cancellation filter
  //  results are stored into fCTvalue array
  //

  DBG_I( printf("Running CrosstalkCancellation ...\n"); )

  AliTRDtrapFilter *lFilterLeft  = (AliTRDtrapFilter *)fFilterLeft;
  AliTRDtrapFilter *lFilterRight = (AliTRDtrapFilter *)fFilterRight;

  Int_t iC0 = (fC0 % 0x007F)<<4; // not absolut correct !!
  Int_t iC1 = (fC1 % 0x007F)<<4;
  Int_t iC2 = (fC2 % 0x007F)<<4;
  Int_t iC3 = (fC3 % 0x007F)<<4;
  Int_t iC4 = (fC4 % 0x007F)<<4;

  fCtPed = (((iC0 * (fTailPed<<3)) >> 14) +
     	    ((iC1 * (fTailPed<<3)) >> 14) +
     	    ((iC2 * (fTailPed<<3)) >> 14) +
     	    ((iC3 * (fTailPed<<3)) >> 14) +
  	    ((iC4 * (fTailPed<<3)) >> 14) +
     	    (fTailPed<<3)) >> 3;

  for(Int_t i=0; i<fTimeMax; i++) {
    Int_t left = 0;
    if(lFilterLeft)
      left = ((iC0 * (lFilterLeft->GetValue(TAIL, i)<<3)) >> 14) +
	     ((iC1 * (lFilterLeft->GetValue(TAIL, i-1)<<3)) >> 14) +
	     ((iC2 * (lFilterLeft->GetValue(TAIL, i-2)<<3)) >> 14) +
	     ((iC3 * (lFilterLeft->GetValue(TAIL, i-3)<<3)) >> 14) +
             ((iC4 * (lFilterLeft->GetValue(TAIL, i-4)<<3)) >> 14);

    Int_t right = 0;
    if(lFilterRight)
      right = ((iC0 * (lFilterRight->GetValue(TAIL, i)<<3)) >> 14) +
	      ((iC1 * (lFilterRight->GetValue(TAIL, i-1)<<3)) >> 14) +
	      ((iC2 * (lFilterRight->GetValue(TAIL, i-2)<<3)) >> 14) +
	      ((iC3 * (lFilterRight->GetValue(TAIL, i-3)<<3)) >> 14) +
              ((iC4 * (lFilterRight->GetValue(TAIL, i-4)<<3)) >> 14);

    Int_t sum;
    if(fTRAP1)
      sum = ((left + (GetValue(TAIL, i-2)<<3) + right) >> 3) & 0x0FFF;
    else {
      sum = ((left + (GetValue(TAIL, i-2)<<3) + right) >> 3);
      if(sum<0) sum = 0x0000;
      if(sum>0x0FFF) sum = 0x0FFF;
    }
    SetValue(CT, i, sum);
  }
}

//_________________________________________________________________________
void AliTRDtrapFilter::GainCorrection()
{
  //
  //  Gain correction filter
  //  results are stored into fGAINvalue array
  //

  DBG_I( printf("Running GainCorrection ...\n"); )

  Int_t iGainFactor = (fGainFactor & 0x007F) | 0x0800;

  for ( Int_t i=0; i<fTimeMax; i++ ) {
    Int_t iValue;
    if(fTRAP1) {
      iValue = ((GetValue(PD, i) * iGainFactor) >> 11) & 0x0FFF;
      iValue = (iValue + (fGainPedestal & 0x01FF)) & 0x0FFF;
    } else {
      iValue = ((GetValue(PD, i) * iGainFactor) >> 11);
      if(iValue>0x0FFF) iValue = 0x0FFF;
      iValue = (iValue + (fGainPedestal & 0x01FF));
      if(iValue>0x0FFF) iValue = 0x0FFF;
    }
    SetValue(GAIN, i, iValue);
  }

}

//_________________________________________________________________________
Int_t AliTRDtrapFilter::GetValue(ARRAY_TYPE iType, Int_t iTimebin)
{
  //
  //  Return one array value
  //

  TArrayI *lArray;
  switch(iType) {
    case ADC:  lArray = fADCvalue;
               break;
    case TAIL: lArray = fTAILvalue;
               break;
    case CT:   lArray = fCTvalue;
               break;
    case NL:   lArray = fNLvalue;
               break;
    case PD:   lArray = fPDvalue;
               break;
    case GAIN: lArray = fGAINvalue;
               break;
    default:   lArray = NULL;
               break;
  }

  if(iTimebin>=0)
    return(lArray->At(iTimebin));
  else
    return fTailPed;

}

//_________________________________________________________________________
void AliTRDtrapFilter::Info()
{
  //
  //  Print some infos about the current state of the filter channel
  //

  cout << "-- Info filter channel -----------------------------" << endl;
  cout << "Plane:   " << fPlane << endl;
  cout << "Sector:  " << fSector << endl;
  cout << "Chamber: " << fChamber << endl;
  cout << "Padrow:  " << fPadrow << endl;
  cout << "MCM:     " << fMCM << endl;
  cout << "Channel: " << fChannel << endl;
  cout << "----------------------------------------------------" << endl;
  cout << "Number of time bins: " << fTimeMax << endl;
  cout << "----------------------------------------------------" << endl;

}

//_________________________________________________________________________
void AliTRDtrapFilter::Init()
{
  //
  //  Initialize the ADC values array
  //

  if(fTimeMax <= -1) {
    DBG_W( printf("Number of time bins is not set.\n"); )
    return;
  } else if (fTimeMax == 0) {
    DBG_W( printf("Number of time bins is zero.\n"); )
    return;
  } else {
    if(!fADCvalue) fADCvalue = new TArrayI(fTimeMax);
    if(!fTAILvalue) fTAILvalue = new TArrayI(fTimeMax);
    if(!fCTvalue) fCTvalue = new TArrayI(fTimeMax);
    if(!fNLvalue) fNLvalue = new TArrayI(fTimeMax);
    if(!fPDvalue) fPDvalue = new TArrayI(fTimeMax);
    if(!fGAINvalue) fGAINvalue = new TArrayI(fTimeMax);

    if(fADCvalue->GetSize()!=fTimeMax) fADCvalue->Set(fTimeMax);
    if(fTAILvalue->GetSize()!=fTimeMax) fTAILvalue->Set(fTimeMax);
    if(fCTvalue->GetSize()!=fTimeMax) fCTvalue->Set(fTimeMax);
    if(fNLvalue->GetSize()!=fTimeMax) fNLvalue->Set(fTimeMax);
    if(fPDvalue->GetSize()!=fTimeMax) fPDvalue->Set(fTimeMax);
    if(fGAINvalue->GetSize()!=fTimeMax) fGAINvalue->Set(fTimeMax);
  }

  if(!fNLped) fNLped = new TArrayI(64);

}

//_________________________________________________________________________
void AliTRDtrapFilter::NonLinearityCorrection()
{
  //
  //  Nonlinearity correction filter
  //  results are stored into fNLvalue array
  //
  //  functionality: bitshift (<<2)
  //                 LUT for pedestal
  //

  DBG_I( printf("Running NonLinearityCorrection ...\n"); )

  for(Int_t i=0; i<fTimeMax; i++) {
    Int_t iValue = GetValue(ADC, i)<<2;
    Int_t iSector = (iValue >> 6) & 0x003F;
    if(fTRAP1)
      iValue = (iValue + GetNLped(iSector)) & 0x0FFF;
    else {
      iValue = (iValue + GetNLped(iSector));
      if(iValue>0x0FFF) iValue = 0x0FFF;
    }
    SetValue(NL, i, iValue);
  }
}

//_________________________________________________________________________
void AliTRDtrapFilter::OutputValues(ARRAY_TYPE iType)
{
  //
  //  Dump the array values to standard output
  //

  Int_t left = 0, center = 0, right = 0;

  printf("---------------------------------------------------\n");
  for(Int_t lTimebin=0; lTimebin<fTimeMax; lTimebin++) {
    if(fFilterLeft)  left  = ((AliTRDtrapFilter *)fFilterLeft)->GetValue(iType, lTimebin);
    if(fFilterRight) right = ((AliTRDtrapFilter *)fFilterRight)->GetValue(iType, lTimebin);
    center = GetValue(iType, lTimebin);
    printf("\t(%4d)\t%4d\t(%4d)\n", left, center, right);
  }
  printf("----------------------------------------------------\n");

}

//_________________________________________________________________________
void AliTRDtrapFilter::PedestalCorrection()
{
  //
  //  Pedestal correction filter
  //  results are stored into fPDvalue array
  //

  DBG_I( printf("Running PedestalCorrection ...\n"); )

  for ( Int_t i=0; i<fTimeMax; i++ ) {
    Int_t iValue;
    if(fTRAP1)
      iValue = (GetValue(NL, i) + fPedestal - fPedestalDyn) & 0x0FFF;
    else {
      iValue = (GetValue(NL, i) + fPedestal - fPedestalDyn);
      if(iValue>0x0FFF) iValue = 0x0FFF;
      if(iValue<0) iValue = 0x0000;
    }
    SetValue(PD, i, iValue);
  }

}

//_________________________________________________________________________
Int_t AliTRDtrapFilter::ReadFilter(Char_t *iFilename)
{
  //
  //  Read the ADC values out of an ASCII input file (Markus Gutfleisch)
  //

  DBG_I( printf("Read filter data from file %s ...\n", iFilename); )

  Int_t i, j, x, y, adcvalue;
  Double_t maximum, value;
  FILE* handle;
  char buffer[80];

  handle = fopen ( iFilename, "r" );	     /* open file in ascii mode */
  if (handle==NULL) {
    DBG_E( printf ( "cannot open file %s\n", iFilename ); )
    return 1;
  }
 
  //ReadLine ( handle, buffer );
  i = 0;
  while ( ( ( buffer[i++]=getc(handle) )!='\n' ) && !feof(handle) );
  buffer[i]='\0';

  if ( strncmp ("KIP00", buffer, 5 )!=0 ) {
    DBG_E( printf ( "no suitable id in %s\n", iFilename ); )
    DBG_E( printf ( "will continue reading file ...\n" ); )
  }
  
  //ReadLine ( handle, buffer );
  i = 0;
  while ( ( ( buffer[i++]=getc(handle) )!='\n' ) && !feof(handle) );
  buffer[i]='\0';

  while ( buffer[0]=='#' ) {
    //ReadLine ( handle, buffer );
    i = 0;
    while ( ( ( buffer[i++]=getc(handle) )!='\n' ) && !feof(handle) );
    buffer[i]='\0';
  }

  if ( strncmp ("size", buffer, 4 )!=0 ) {
    DBG_E( printf ( "no size specification in %s\n", iFilename ); )
    DBG_E( printf ( "found line <%s> instead.\n", buffer ); )
    return 1;
  }

  i=5;
  while (buffer[i] != ' ')
    i++;
  while (buffer[i] == ' ')
    i++;
  
  sscanf( &buffer[5], "%d", &y );
  sscanf( &buffer[i], "%d", &x );
  
  fTimeMax = x;
  Init();
  // create_filter ( x, (y-1)/2, filter );

  //ReadLine ( handle, buffer );
  i = 0;
  while ( ( ( buffer[i++]=getc(handle) )!='\n' ) && !feof(handle) );
  buffer[i]='\0';

  if ( strncmp ("depth", buffer, 5 )!=0 ) {
    DBG_E( printf ( "no depth specification in %s\n", iFilename ); )
    return 1;
  }

  sscanf ( &buffer[6], "%lf", &maximum );

  for ( i=0; i<fTimeMax; i++ ) {
    for ( j=0; j<3; j++ ) {
      fscanf ( handle, "%lf", &value );
      adcvalue = (Int_t)((1024.0/maximum)*value);
      if( j==1 ) SetValue(ADC, i, adcvalue+100);
    }
  }
  
  fclose ( handle );
  return 0;
}

//_________________________________________________________________________
void AliTRDtrapFilter::SetValue(ARRAY_TYPE iType, Int_t iTimebin, Int_t value)
{
  //
  //  Put one array value
  //

  TArrayI *lArray;
  switch(iType) {
    case ADC:  lArray = fADCvalue;
               break;
    case TAIL: lArray = fTAILvalue;
               break;
    case CT:   lArray = fCTvalue;
               break;
    case NL:   lArray = fNLvalue;
               break;
    case PD:   lArray = fPDvalue;
               break;
    case GAIN: lArray = fGAINvalue;
               break;
    default:   lArray = NULL;
               break;
  }

  if(!lArray) Init();

  if(value < 0) value = 0;
  lArray->AddAt(value, iTimebin);
}

//_________________________________________________________________________
void AliTRDtrapFilter::TailCancellation()
{
  //
  //  Tail cancellation filter
  //  results are stored into fTAILvalue array
  //

  DBG_I( printf("Running TailCancellation ...\n"); )

  Int_t rem1, rem2;
  Int_t correction, result;
  Int_t iFactor = 100<<2; //fPedestal

  Int_t iAlpha_l = fAlpha_l & 0x03FF;
  Int_t iLambda_l = fLambda_l & 0x01FF; iLambda_l |= 0x0600; 
  Int_t iLambda_s = fLambda_s & 0x01FF; iLambda_s |= 0x0200;

  Double_t iAl = iAlpha_l / 2048.0;
  Double_t iAs = 1.0-iAl;
  Double_t iLl = iLambda_l / 2048.0;
  Double_t iLs = iLambda_s / 2048.0;

  Double_t xi = 1-(iLl*iAs + iLs*iAl);
  rem1 = (Int_t)((iFactor/xi) * ((1-iLs)*iLl*iAl));
  rem2 = (Int_t)((iFactor/xi) * ((1-iLl)*iLs*iAs));
  correction = (rem1 + rem2) & 0x0FFF;
  fTailPed = iFactor - correction;

  for ( Int_t i=0; i<fTimeMax; i++ ) {
    result = ( GetValue(GAIN, i) - correction );
    if ( result<0 )
      result = 0;

    SetValue(TAIL, i, result);

    rem1 = (iLambda_l * (( rem1 + ((iAlpha_l * result) >> 11) ) & 0x0FFF)) >> 11;
    rem2 = (iLambda_s * (( rem2 + (result-((iAlpha_l * result) >> 11))) & 0x0FFF)) >> 11;

    correction = (rem1 + rem2) & 0x0FFF;
  }

}

