-------------------------------------------------------------------------------
-- Title      : ni_io_stim
-- Project    : Network Interface (NI) for the ALICE TRD TRAP2
-------------------------------------------------------------------------------
-- File       : ni_io_stim.vhd
-- Author     : Rolf Schneider  <rolf.schneider@kip.uni-heidelberg.de>
-- Company    : University Heidelberg - KIP
-- Last update: 2003-04-15
-- Platform   : Synopsys - v2003.03
-------------------------------------------------------------------------------
-- Description: Reads the input stimuli for local & global I/O signals
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author  Description
-- 2003/04/14  1.0      schneide        Created
-------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use std.textio.all;
use IEEE.std_logic_textio.all;

----------------------------------------------------------------------------------------------------
-- ENTITY
----------------------------------------------------------------------------------------------------
entity ni_io_stim is
  generic (
    filename : string := "ni_io_stim.txt";              -- stimulus filename
    start    : integer := 5;                            -- start time
    width    : integer := 32);                          -- data width
  port (
    we_0           : out STD_LOGIC;
    address_0      : out STD_LOGIC_VECTOR((width/2)-1 downto 0);
    din_0          : out STD_LOGIC_VECTOR(width-1 downto 0);
    we_1           : out STD_LOGIC;
    address_1      : out STD_LOGIC_VECTOR((width/2)-1 downto 0);
    din_1          : out STD_LOGIC_VECTOR(width-1 downto 0);
    we_2           : out STD_LOGIC;
    address_2      : out STD_LOGIC_VECTOR((width/2)-1 downto 0);
    din_2          : out STD_LOGIC_VECTOR(width-1 downto 0);
    we_3           : out STD_LOGIC;
    address_3      : out STD_LOGIC_VECTOR((width/2)-1 downto 0);
    din_3          : out STD_LOGIC_VECTOR(width-1 downto 0);
    we_g           : out STD_LOGIC;
    address_g      : out STD_LOGIC_VECTOR((width/2)-1 downto 0);
    din_g          : out STD_LOGIC_VECTOR(width-1 downto 0));
end ni_io_stim;

----------------------------------------------------------------------------------------------------
-- ARCHITECTURE
----------------------------------------------------------------------------------------------------
architecture v of ni_io_stim is

  -- CONSTANTS -------------------------------------------------------------------------------------
  file io_stim : text open read_mode is filename;
  constant period_time : time    := 8 ns;

  -- STRUCTURE -------------------------------------------------------------------------------------
begin

  read_io_stim: process
    variable lin : line;
    variable good : boolean;
    variable w0, w1, w2, w3, wg : std_logic;
    variable a0, a1, a2, a3, ag : std_logic_vector((width/2)-1 downto 0);
    variable d0, d1, d2, d3, dg : std_logic_vector(width-1 downto 0);
  begin
    we_0 <= '0'; address_0 <= (others => '0'); din_0 <= (others => '0');
    we_1 <= '0'; address_1 <= (others => '0'); din_1 <= (others => '0');
    we_2 <= '0'; address_2 <= (others => '0'); din_2 <= (others => '0');
    we_3 <= '0'; address_3 <= (others => '0'); din_3 <= (others => '0');
    we_g <= '0'; address_g <= (others => '0'); din_g <= (others => '0');
    wait for period_time * start;
    wait for 1 ns;
    while (endfile(io_stim) = false) loop
      readline(io_stim, lin);
      read(lin, w0, good);
      if w0 = '1' or w0 = '0' then
                        hread(lin, a0); hread(lin, d0);
        read(lin,  w1); hread(lin, a1); hread(lin, d1);
        read(lin,  w2); hread(lin, a2); hread(lin, d2);
        read(lin,  w3); hread(lin, a3); hread(lin, d3);
        read(lin,  wg); hread(lin, ag); hread(lin, dg);
        we_0 <= w0; address_0 <= a0; din_0 <= d0;
        we_1 <= w1; address_1 <= a1; din_1 <= d1;
        we_2 <= w2; address_2 <= a2; din_2 <= d2;
        we_3 <= w3; address_3 <= a3; din_3 <= d3;
        we_g <= wg; address_g <= ag; din_g <= dg;
        wait for period_time;
      end if;
    end loop;
    wait;
  end process;

end v;

----------------------------------------------------------------------------------------------------
-- CONFIGURATION
----------------------------------------------------------------------------------------------------
-- pragma translate_off
configuration ni_io_stim_CFG of ni_io_stim is
  for v
  end for; -- v
end ni_io_stim_CFG;
-- pragma translate_on

