----------------------------------------------------------------------------------------------------
-- SIMULATION OF A READOUT BOARD CONTAINING 17 CHIPS (NETWORK INTERFACE ONLY)
----------------------------------------------------------------------------------------------------
--
--   00    01 --> 02 <-- 03
--     `---------^  `------------
--                               |
--   04    05 --> 06 <-- 07      |
--     `---------^  `-----------,|
--                              ||
--                              16 -->
--   08    09 --> 10 <-- 11     ||
--     `---------^  `-----------|
--                               |
--   12    13 --> 14 <-- 15      |
--     `---------^  `------------
--
----------------------------------------------------------------------------------------------------


library IEEE, TRAP;
use IEEE.std_logic_1164.all;
use std.textio.all;
use IEEE.std_logic_textio.all;
use WORK.ni_global.all;
use WORK.ni_rb_global.all;

----------------------------------------------------------------------------------------------------
-- ENTITY
----------------------------------------------------------------------------------------------------
entity ni_rb_tb is
  generic (
    io_stim_file    : string  := "./STIMULI/io_stim_rbtr";  -- _??.txt will be added
    io_stim_start   : integer := 5;
    sniff_file      : string  := "./RESULTS/ni_sniff.txt";
    ol              : boolean := FALSE);
end ni_rb_tb;

----------------------------------------------------------------------------------------------------
-- ARCHITECTURE
----------------------------------------------------------------------------------------------------
architecture v of ni_rb_tb is

  -- CONSTANTS -------------------------------------------------------------------------------------
  constant period_time : time    := 8 ns;
  constant width       : integer := 32;
  
  type start_type is array(0 to chips-1) of integer;
  constant io_stim_start_offset : start_type := (
     0,  -- chip  0
    12,  -- chip  1
     0,  -- chip  2
    12,  -- chip  3
    13,  -- chip  4
     0,  -- chip  5
    13,  -- chip  6
     0,  -- chip  7
     1,  -- chip  8
     8,  -- chip  9
    13,  -- chip 10
     0,  -- chip 11
     7,  -- chip 12
     1,  -- chip 13
     9,  -- chip 14
     4,  -- chip 15
     5   -- chip 16
  );

  type fn_type is array(0 to chips-1) of string(28 downto 1);
  constant fn_array : fn_type := (io_stim_file & "_00" & ".txt",
                                  io_stim_file & "_01" & ".txt",
                                  io_stim_file & "_02" & ".txt",
                                  io_stim_file & "_03" & ".txt",
                                  io_stim_file & "_04" & ".txt",
                                  io_stim_file & "_05" & ".txt",
                                  io_stim_file & "_06" & ".txt",
                                  io_stim_file & "_07" & ".txt",
                                  io_stim_file & "_08" & ".txt",
                                  io_stim_file & "_09" & ".txt",
                                  io_stim_file & "_10" & ".txt",
                                  io_stim_file & "_11" & ".txt",
                                  io_stim_file & "_12" & ".txt",
                                  io_stim_file & "_13" & ".txt",
                                  io_stim_file & "_14" & ".txt",
                                  io_stim_file & "_15" & ".txt",
                                  io_stim_file & "_16" & ".txt"
  );

  -- NI SIGNALS ------------------------------------------------------------------------------------
  signal clk           : sig_array;
  signal clk_glob      : sig_array;
  signal clk_alws      : sig_array;
  signal reset_n       : sig_array;
  signal clear_n       : sig_array;
  signal mode_tr       : sig_array;
  signal mode_rr       : sig_array;
  signal finished_n    : sig_array;
  signal we_0          : sig_array;
  signal address_0     : addr_array;
  signal din_0         : data_array;
  signal dout_0        : data_array;
  signal intr_0        : intr_array;
  signal we_1          : sig_array;
  signal address_1     : addr_array;
  signal din_1         : data_array;
  signal dout_1        : data_array;
  signal intr_1        : intr_array;
  signal we_2          : sig_array;
  signal address_2     : addr_array;
  signal din_2         : data_array;
  signal dout_2        : data_array;
  signal intr_2        : intr_array;
  signal we_3          : sig_array;
  signal address_3     : addr_array;
  signal din_3         : data_array;
  signal dout_3        : data_array;
  signal intr_3        : intr_array;
  signal we_g          : sig_array;
  signal address_g     : addr_array;
  signal din_g         : data_array;
  signal dout_g        : data_array;
  signal intr_g        : sig_array;
  signal p_strobe_in_0 : sig_array;
  signal p_d_in_0      : port_array;
  signal p_ctrl_out_0  : sig_array;
  signal p_strobe_in_1 : sig_array;
  signal p_d_in_1      : port_array;
  signal p_ctrl_out_1  : sig_array;
  signal p_strobe_in_2 : sig_array;
  signal p_d_in_2      : port_array;
  signal p_ctrl_out_2  : sig_array;
  signal p_strobe_in_3 : sig_array;
  signal p_d_in_3      : port_array;
  signal p_ctrl_out_3  : sig_array;
  signal p_ctrl_in     : sig_array;
  signal p_strobe_out  : sig_array;
  signal p_d_out       : port_array;
  signal p_ctrl        : sig_array;

  -- SIGNALS ---------------------------------------------------------------------------------------
  signal clk_i     : STD_LOGIC;
  signal reset_n_i : STD_LOGIC;
  signal clear_n_i : STD_LOGIC;
  signal mode_tr_i : STD_LOGIC;
  signal mode_rr_i : STD_LOGIC;

  signal enable_strobe_n    : STD_LOGIC;
  signal sniff_p_strobe_out : STD_LOGIC;
  signal sniff_p_d_out      : STD_LOGIC_VECTOR((width/4)+1 downto 0);
  signal sniff_p_ctrl       : STD_LOGIC;

  -- COMPONENTS ------------------------------------------------------------------------------------
  component ni_top
    port (
      clk           : in  STD_LOGIC;
      clk_glob      : in  STD_LOGIC;
      clk_alws      : in  STD_LOGIC;
      reset_n       : in  STD_LOGIC;
      clear_n       : in  STD_LOGIC;
      mode_tr       : in  STD_LOGIC;
      mode_rr       : in  STD_LOGIC;
      finished_n    : out STD_LOGIC;
      we_0          : in  STD_LOGIC;
      address_0     : in  STD_LOGIC_VECTOR(io_address_bus_width-1 downto 0);
      din_0         : in  STD_LOGIC_VECTOR(width-1 downto 0);
      dout_0        : out STD_LOGIC_VECTOR(width-1 downto 0);
      intr_0        : out STD_LOGIC_VECTOR(2 downto 0);
      we_1          : in  STD_LOGIC;
      address_1     : in  STD_LOGIC_VECTOR(io_address_bus_width-1 downto 0);
      din_1         : in  STD_LOGIC_VECTOR(width-1 downto 0);
      dout_1        : out STD_LOGIC_VECTOR(width-1 downto 0);
      intr_1        : out STD_LOGIC_VECTOR(2 downto 0);
      we_2          : in  STD_LOGIC;
      address_2     : in  STD_LOGIC_VECTOR(io_address_bus_width-1 downto 0);
      din_2         : in  STD_LOGIC_VECTOR(width-1 downto 0);
      dout_2        : out STD_LOGIC_VECTOR(width-1 downto 0);
      intr_2        : out STD_LOGIC_VECTOR(2 downto 0);
      we_3          : in  STD_LOGIC;
      address_3     : in  STD_LOGIC_VECTOR(io_address_bus_width-1 downto 0);
      din_3         : in  STD_LOGIC_VECTOR(width-1 downto 0);
      dout_3        : out STD_LOGIC_VECTOR(width-1 downto 0);
      intr_3        : out STD_LOGIC_VECTOR(2 downto 0);
      we_g          : in  STD_LOGIC;
      address_g     : in  STD_LOGIC_VECTOR(io_address_bus_width-1 downto 0);
      din_g         : in  STD_LOGIC_VECTOR(width-1 downto 0);
      dout_g        : out STD_LOGIC_VECTOR(width-1 downto 0);
      intr_g        : out STD_LOGIC;
      p_strobe_in_0 : in  STD_LOGIC;
      p_d_in_0      : in  STD_LOGIC_VECTOR((width/4)+1 downto 0);
      p_ctrl_out_0  : out STD_LOGIC;
      p_strobe_in_1 : in  STD_LOGIC;
      p_d_in_1      : in  STD_LOGIC_VECTOR((width/4)+1 downto 0);
      p_ctrl_out_1  : out STD_LOGIC;
      p_strobe_in_2 : in  STD_LOGIC;
      p_d_in_2      : in  STD_LOGIC_VECTOR((width/4)+1 downto 0);
      p_ctrl_out_2  : out STD_LOGIC;
      p_strobe_in_3 : in  STD_LOGIC;
      p_d_in_3      : in  STD_LOGIC_VECTOR((width/4)+1 downto 0);
      p_ctrl_out_3  : out STD_LOGIC;
      p_ctrl_in     : in  STD_LOGIC;
      p_strobe_out  : out STD_LOGIC;
      p_d_out       : out STD_LOGIC_VECTOR((width/4)+1 downto 0);
      p_ctrl        : out STD_LOGIC);
  end component;

  component ni_sniffer
    generic (
      filename : string;
      sel_p    : integer;
      sel_s    : integer;
      ol       : boolean);
    port (
      strobe   : in std_logic;
      data     : in std_logic_vector(9 downto 0);
      ctrl_out : in std_logic;
      ctrl_in  : in std_logic);
  end component;

  component ni_io_stim is
    generic (
      filename : string;
      start    : integer;
      width    : integer);
    port (
      we_0      : out STD_LOGIC;
      address_0 : out STD_LOGIC_VECTOR((width/2)-1 downto 0);
      din_0     : out STD_LOGIC_VECTOR(width-1 downto 0);
      we_1      : out STD_LOGIC;
      address_1 : out STD_LOGIC_VECTOR((width/2)-1 downto 0);
      din_1     : out STD_LOGIC_VECTOR(width-1 downto 0);
      we_2      : out STD_LOGIC;
      address_2 : out STD_LOGIC_VECTOR((width/2)-1 downto 0);
      din_2     : out STD_LOGIC_VECTOR(width-1 downto 0);
      we_3      : out STD_LOGIC;
      address_3 : out STD_LOGIC_VECTOR((width/2)-1 downto 0);
      din_3     : out STD_LOGIC_VECTOR(width-1 downto 0);
      we_g      : out STD_LOGIC;
      address_g : out STD_LOGIC_VECTOR((width/2)-1 downto 0);
      din_g     : out STD_LOGIC_VECTOR(width-1 downto 0));
  end component ni_io_stim;

  -- STRUCTURE -------------------------------------------------------------------------------------
begin

  -- clk/reset stimuli -----------------------------------------------------------------------------
  reset_n_i <= '1',
               '0' after 2 ns,
               '1' after 15 ns;
  clear_n_i <= '1';
  mode_tr_i <= '0';
  mode_rr_i <= '0';

  U_clock: process
  begin
    clk_i <= '1';
    wait for period_time/2;
    clk_i <= '0';
    wait for period_time/2;
  end process;

  -- NI connections --------------------------------------------------------------------------------
  U_common_gen: for i in 0 to chips-1 generate
    clk(i) <= clk_i;
    clk_glob(i) <= clk_i;
    clk_alws(i) <= clk_i;
    reset_n(i)  <= reset_n_i;
    clear_n(i)  <= clear_n_i;
    mode_tr(i)  <= mode_tr_i;
    mode_rr(i)  <= mode_rr_i;

    U_ni_io_stim: ni_io_stim
      generic map (
        filename => fn_array(i),
        start    => io_stim_start + io_stim_start_offset(i),
        width    => width)
      port map (
        we_0      => we_0(i),
        address_0 => address_0(i),
        din_0     => din_0(i),
        we_1      => we_1(i),
        address_1 => address_1(i),
        din_1     => din_1(i),
        we_2      => we_2(i),
        address_2 => address_2(i),
        din_2     => din_2(i),
        we_3      => we_3(i),
        address_3 => address_3(i),
        din_3     => din_3(i),
        we_g      => we_g(i),
        address_g => address_g(i),
        din_g     => din_g(i));
  end generate;

  p_strobe_in_0( 0) <= '1';                                     p_d_in_0( 0) <= (others => '1');                    p_ctrl_in( 0) <= p_ctrl_out_1( 2);
  p_strobe_in_1( 0) <= '1';                                     p_d_in_1( 0) <= (others => '1');
  p_strobe_in_2( 0) <= '1';                                     p_d_in_2( 0) <= (others => '1');
  p_strobe_in_3( 0) <= '1';                                     p_d_in_3( 0) <= (others => '1');
  p_strobe_in_0( 1) <= '1';                                     p_d_in_0( 1) <= (others => '1');                    p_ctrl_in( 1) <= p_ctrl_out_0( 2);
  p_strobe_in_1( 1) <= '1';                                     p_d_in_1( 1) <= (others => '1');
  p_strobe_in_2( 1) <= '1';                                     p_d_in_2( 1) <= (others => '1');
  p_strobe_in_3( 1) <= '1';                                     p_d_in_3( 1) <= (others => '1');
  p_strobe_in_0( 2) <= transport p_strobe_out( 1) after 1 ns;   p_d_in_0( 2) <= transport p_d_out( 1) after 2 ns;   p_ctrl_in( 2) <= p_ctrl_out_2(16);
  p_strobe_in_1( 2) <= transport p_strobe_out( 0) after 1 ns;   p_d_in_1( 2) <= transport p_d_out( 0) after 2 ns;
  p_strobe_in_2( 2) <= '1';                                     p_d_in_2( 2) <= (others => '1');
  p_strobe_in_3( 2) <= transport p_strobe_out( 3) after 1 ns;   p_d_in_3( 2) <= transport p_d_out( 3) after 2 ns;
  p_strobe_in_0( 3) <= '1';                                     p_d_in_0( 3) <= (others => '1');                    p_ctrl_in( 3) <= p_ctrl_out_3( 2);
  p_strobe_in_1( 3) <= '1';                                     p_d_in_1( 3) <= (others => '1');
  p_strobe_in_2( 3) <= '1';                                     p_d_in_2( 3) <= (others => '1');
  p_strobe_in_3( 3) <= '1';                                     p_d_in_3( 3) <= (others => '1');

  p_strobe_in_0( 4) <= '1';                                     p_d_in_0( 4) <= (others => '1');                    p_ctrl_in( 4) <= p_ctrl_out_1( 6);
  p_strobe_in_1( 4) <= '1';                                     p_d_in_1( 4) <= (others => '1');
  p_strobe_in_2( 4) <= '1';                                     p_d_in_2( 4) <= (others => '1');
  p_strobe_in_3( 4) <= '1';                                     p_d_in_3( 4) <= (others => '1');
  p_strobe_in_0( 5) <= '1';                                     p_d_in_0( 5) <= (others => '1');                    p_ctrl_in( 5) <= p_ctrl_out_0( 6);
  p_strobe_in_1( 5) <= '1';                                     p_d_in_1( 5) <= (others => '1');
  p_strobe_in_2( 5) <= '1';                                     p_d_in_2( 5) <= (others => '1');
  p_strobe_in_3( 5) <= '1';                                     p_d_in_3( 5) <= (others => '1');
  p_strobe_in_0( 6) <= transport p_strobe_out( 5) after 1 ns;   p_d_in_0( 6) <= transport p_d_out( 5) after 2 ns;   p_ctrl_in( 6) <= p_ctrl_out_0(16);
  p_strobe_in_1( 6) <= transport p_strobe_out( 4) after 1 ns;   p_d_in_1( 6) <= transport p_d_out( 4) after 2 ns;
  p_strobe_in_2( 6) <= '1';                                     p_d_in_2( 6) <= (others => '1');
  p_strobe_in_3( 6) <= transport p_strobe_out( 7) after 1 ns;   p_d_in_3( 6) <= transport p_d_out( 7) after 2 ns;
  p_strobe_in_0( 7) <= '1';                                     p_d_in_0( 7) <= (others => '1');                    p_ctrl_in( 7) <= p_ctrl_out_3( 6);
  p_strobe_in_1( 7) <= '1';                                     p_d_in_1( 7) <= (others => '1');
  p_strobe_in_2( 7) <= '1';                                     p_d_in_2( 7) <= (others => '1');
  p_strobe_in_3( 7) <= '1';                                     p_d_in_3( 7) <= (others => '1');

  p_strobe_in_0( 8) <= '1';                                     p_d_in_0( 8) <= (others => '1');                    p_ctrl_in( 8) <= p_ctrl_out_1(10);
  p_strobe_in_1( 8) <= '1';                                     p_d_in_1( 8) <= (others => '1');
  p_strobe_in_2( 8) <= '1';                                     p_d_in_2( 8) <= (others => '1');
  p_strobe_in_3( 8) <= '1';                                     p_d_in_3( 8) <= (others => '1');
  p_strobe_in_0( 9) <= '1';                                     p_d_in_0( 9) <= (others => '1');                    p_ctrl_in( 9) <= p_ctrl_out_0(10);
  p_strobe_in_1( 9) <= '1';                                     p_d_in_1( 9) <= (others => '1');
  p_strobe_in_2( 9) <= '1';                                     p_d_in_2( 9) <= (others => '1');
  p_strobe_in_3( 9) <= '1';                                     p_d_in_3( 9) <= (others => '1');
  p_strobe_in_0(10) <= transport p_strobe_out( 9) after 1 ns;   p_d_in_0(10) <= transport p_d_out( 9) after 2 ns;   p_ctrl_in(10) <= p_ctrl_out_1(16);
  p_strobe_in_1(10) <= transport p_strobe_out( 8) after 1 ns;   p_d_in_1(10) <= transport p_d_out( 8) after 2 ns;
  p_strobe_in_2(10) <= '1';                                     p_d_in_2(10) <= (others => '1');
  p_strobe_in_3(10) <= transport p_strobe_out(11) after 1 ns;   p_d_in_3(10) <= transport p_d_out(11) after 2 ns;
  p_strobe_in_0(11) <= '1';                                     p_d_in_0(11) <= (others => '1');                    p_ctrl_in(11) <= p_ctrl_out_3(10);
  p_strobe_in_1(11) <= '1';                                     p_d_in_1(11) <= (others => '1');
  p_strobe_in_2(11) <= '1';                                     p_d_in_2(11) <= (others => '1');
  p_strobe_in_3(11) <= '1';                                     p_d_in_3(11) <= (others => '1');

  p_strobe_in_0(12) <= '1';                                     p_d_in_0(12) <= (others => '1');                    p_ctrl_in(12) <= p_ctrl_out_1(14);
  p_strobe_in_1(12) <= '1';                                     p_d_in_1(12) <= (others => '1');
  p_strobe_in_2(12) <= '1';                                     p_d_in_2(12) <= (others => '1');
  p_strobe_in_3(12) <= '1';                                     p_d_in_3(12) <= (others => '1');
  p_strobe_in_0(13) <= '1';                                     p_d_in_0(13) <= (others => '1');                    p_ctrl_in(13) <= p_ctrl_out_0(14);
  p_strobe_in_1(13) <= '1';                                     p_d_in_1(13) <= (others => '1');
  p_strobe_in_2(13) <= '1';                                     p_d_in_2(13) <= (others => '1');
  p_strobe_in_3(13) <= '1';                                     p_d_in_3(13) <= (others => '1');
  p_strobe_in_0(14) <= transport p_strobe_out(13) after 1 ns;   p_d_in_0(14) <= transport p_d_out(13) after 2 ns;   p_ctrl_in(14) <= p_ctrl_out_3(16);
  p_strobe_in_1(14) <= transport p_strobe_out(12) after 1 ns;   p_d_in_1(14) <= transport p_d_out(12) after 2 ns;
  p_strobe_in_2(14) <= '1';                                     p_d_in_2(14) <= (others => '1');
  p_strobe_in_3(14) <= transport p_strobe_out(15) after 1 ns;   p_d_in_3(14) <= transport p_d_out(15) after 2 ns;
  p_strobe_in_0(15) <= '1';                                     p_d_in_0(15) <= (others => '1');                    p_ctrl_in(15) <= p_ctrl_out_3(14);
  p_strobe_in_1(15) <= '1';                                     p_d_in_1(15) <= (others => '1');
  p_strobe_in_2(15) <= '1';                                     p_d_in_2(15) <= (others => '1');
  p_strobe_in_3(15) <= '1';                                     p_d_in_3(15) <= (others => '1');

  p_strobe_in_0(16) <= transport p_strobe_out( 6) after 1 ns;   p_d_in_0(16) <= transport p_d_out( 6) after 2 ns;   p_ctrl_in(16) <= '1';
  p_strobe_in_1(16) <= transport p_strobe_out(10) after 1 ns;   p_d_in_1(16) <= transport p_d_out(10) after 2 ns;
  p_strobe_in_2(16) <= transport p_strobe_out( 2) after 1 ns;   p_d_in_2(16) <= transport p_d_out( 2) after 2 ns;
  p_strobe_in_3(16) <= transport p_strobe_out(14) after 1 ns;   p_d_in_3(16) <= transport p_d_out(14) after 2 ns;

  -- NI instances ----------------------------------------------------------------------------------
  U_ni_top_gen: for i in 0 to chips-1 generate
    U_ni_top: ni_top
      port map (
        clk           => clk(i),
        clk_glob      => clk_glob(i),
        clk_alws      => clk_alws(i),
        reset_n       => reset_n(i),
        clear_n       => clear_n(i),
        mode_tr       => mode_tr(i),
        mode_rr       => mode_rr(i),
        finished_n    => finished_n(i),
        we_0          => we_0(i),
        address_0     => address_0(i),
        din_0         => din_0(i),
        dout_0        => dout_0(i),
        intr_0        => intr_0(i),
        we_1          => we_1(i),
        address_1     => address_1(i),
        din_1         => din_1(i),
        dout_1        => dout_1(i),
        intr_1        => intr_1(i),
        we_2          => we_2(i),
        address_2     => address_2(i),
        din_2         => din_2(i),
        dout_2        => dout_2(i),
        intr_2        => intr_2(i),
        we_3          => we_3(i),
        address_3     => address_3(i),
        din_3         => din_3(i),
        dout_3        => dout_3(i),
        intr_3        => intr_3(i),
        we_g          => we_g(i),
        address_g     => address_g(i),
        din_g         => din_g(i),
        dout_g        => dout_g(i),
        intr_g        => intr_g(i),
        p_strobe_in_0 => p_strobe_in_0(i),
        p_d_in_0      => p_d_in_0(i),
        p_ctrl_out_0  => p_ctrl_out_0(i),
        p_strobe_in_1 => p_strobe_in_1(i),
        p_d_in_1      => p_d_in_1(i),
        p_ctrl_out_1  => p_ctrl_out_1(i),
        p_strobe_in_2 => p_strobe_in_2(i),
        p_d_in_2      => p_d_in_2(i),
        p_ctrl_out_2  => p_ctrl_out_2(i),
        p_strobe_in_3 => p_strobe_in_3(i),
        p_d_in_3      => p_d_in_3(i),
        p_ctrl_out_3  => p_ctrl_out_3(i),
        p_ctrl_in     => p_ctrl_in(i),
        p_strobe_out  => p_strobe_out(i),
        p_d_out       => p_d_out(i),
        p_ctrl        => p_ctrl(i));
  end generate;

  -- output port protocol --------------------------------------------------------------------------
  enable_strobe_n    <= '1', '0' after 100 ns;
  sniff_p_strobe_out <= enable_strobe_n or p_strobe_out(16);
  sniff_p_d_out      <= transport p_d_out(16) after 1 ns;
  sniff_p_ctrl       <= transport p_ctrl(16) after 1 ns;

  U_ni_sniffer_16: ni_sniffer
    generic map (
      filename => sniff_file,
      sel_p    => 8,
      sel_s    => 9,
      ol       => ol)
    port map (
      strobe   => sniff_p_strobe_out,
      data     => sniff_p_d_out,
      ctrl_out => sniff_p_ctrl,
      ctrl_in  => p_ctrl_in(16));

end v;

----------------------------------------------------------------------------------------------------
-- CONFIGURATION
----------------------------------------------------------------------------------------------------
-- pragma translate_off
configuration ni_rb_tb_CFG of ni_rb_tb is
  for v
  end for; -- v
end ni_rb_tb_CFG;
-- pragma translate_on

