-------------------------------------------------------------------------------
-- Title      : ni_sniffer
-- Project    : Network Interface (NI) for the ALICE TRD TRAP2
-------------------------------------------------------------------------------
-- File       : ni_sniffer.vhd
-- Author     : Rolf Schneider  <rolf.schneider@kip.uni-heidelberg.de>
-- Company    : University Heidelberg - KIP
-- Last update: 2003-04-16
-- Platform   : Synopsys - v2003.03
-------------------------------------------------------------------------------
-- Description: Creates a output link protocol as text file. Use the perl
--              grep_words.pl script to extract the transmitted 32 Bit words.
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author  Description
-- 2003/04/08  1.0      schneide        Created
-------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use std.textio.all;
use IEEE.std_logic_textio.all;

----------------------------------------------------------------------------------------------------
-- ENTITY
----------------------------------------------------------------------------------------------------
entity ni_sniffer is
  generic (
    filename : string := "ni_sniff.txt";                -- output filename
    sel_p    : integer := 8;                            -- position of partiy bit (pass 1)
    sel_s    : integer := 9;                            -- position of spare bit (pass 2)
    ol       : boolean := FALSE);                       -- optical link output mode (clk/ctrl)
  port (
    strobe   : in std_logic;                            -- strobe signal (clk in ol mode)
    data     : in std_logic_vector(9 downto 0);         -- data incl. partiy & strobe
    ctrl_out : in std_logic;                            -- ctrl output to OASE
    ctrl_in  : in std_logic);                           -- ctrl input of output port
end ni_sniffer;

----------------------------------------------------------------------------------------------------
-- ARCHITECTURE
----------------------------------------------------------------------------------------------------
architecture v of ni_sniffer is

  -- CONSTANTS -------------------------------------------------------------------------------------
  file io_scan : text open write_mode is filename;

  -- FUNCTIONS -------------------------------------------------------------------------------------
  function exclude(data: std_logic_vector(9 downto 0)) return std_logic_vector is
    variable q1pass : std_logic_vector(8 downto 0);
    variable q2pass : std_logic_vector(7 downto 0);
  begin
    q1pass := data(9 downto sel_s+1) & data(sel_s-1 downto 0);
    q2pass := q1pass(8 downto sel_p+1) & q1pass(sel_p-1 downto 0);
    return q2pass;
  end exclude;

signal strobe_last_value : std_logic;

  -- STRUCTURE -------------------------------------------------------------------------------------
begin
  strobe_last_value <= transport strobe after 1 ns;
  
  U_scan_full_gen: if ol=FALSE generate
    U_scan_full: process
      variable lin : line;
      variable d   : std_logic_vector(31 downto 0);
      variable di  : integer := 0;
      procedure check_event(ev: boolean) is
      begin
        if ev then write(lin, string'("* ")); else write(lin, string'("  ")); end if;
      end check_event;
    begin
      wait until strobe'event or data'event or ctrl_out'event or ctrl_in'event;
      if strobe='1' and strobe'event and strobe_last_value='Z' then di:=0; end if;
      if falling_edge(strobe) or rising_edge(strobe) then
        d((8*di)+7 downto 8*di) := exclude(data);
        di := di + 1;
        if di = 4 then di := 0; end if;
      end if;
      if rising_edge(strobe) and di = 0 then
        write(lin, string'("0x")); hwrite(lin, d); write(lin, string'("  "));
      else
        write(lin, string'("            "));
      end if;
      write(lin, strobe);   check_event(strobe'event);
      write(lin, data);     check_event(data'event);
      write(lin, ctrl_out); check_event(ctrl_out'event);
      write(lin, ctrl_in);  check_event(ctrl_in'event);
      write(lin, string'("(")); write(lin, now); write(lin, string'(")"));
      writeline(io_scan, lin);
    end process;
  end generate;

  U_scan_ol_gen: if ol=TRUE generate
    U_scan_ol: process
      variable lin   : line;
      variable d     : std_logic_vector(31 downto 0);
      variable di    : integer := 0;
      variable wrote : boolean := FALSE;
      procedure check_event(ev: boolean) is
      begin
        if ev then write(lin, string'("* ")); else write(lin, string'("  ")); end if;
      end check_event;
    begin
      wait until strobe'event or data'event or ctrl_out'event or ctrl_in'event;
      if ctrl_out='0' and (falling_edge(strobe) or rising_edge(strobe)) then
        d((8*di)+7 downto 8*di) := exclude(data);
        di := di + 1;
        if di = 4 then
          write(lin, string'("0x")); hwrite(lin, d); write(lin, string'("  "));
          wrote := TRUE;
          di := 0;
        end if;
      end if;
      if wrote or data'event or ctrl_out'event or ctrl_in'event then
        if wrote=FALSE then write(lin, string'("            ")); end if;
        write(lin, data);     check_event(data'event);
        write(lin, ctrl_out); check_event(ctrl_out'event);
        write(lin, ctrl_in);  check_event(ctrl_in'event);
        write(lin, string'("(")); write(lin, now); write(lin, string'(")"));
        writeline(io_scan, lin);
        wrote := FALSE;
      end if;
    end process;
  end generate;

end v;

----------------------------------------------------------------------------------------------------
-- CONFIGURATION
----------------------------------------------------------------------------------------------------
-- pragma translate_off
configuration ni_sniffer_CFG of ni_sniffer is
  for v
  end for; -- v
end ni_sniffer_CFG;
-- pragma translate_on

