
################################################################################
# define all synchronous clocks into appropriate groups
################################################################################

# Clock Name: CLK120_FANOUT
# this clk root is distributed accross the chip to four LVDS outputs, which are
# used to forward te clock to other MCMs. It is also fed into a mux allowing to
# use this clock or an external clock as reference into the PLL480.
# Note: this clock has to come first or it will not be properly generated because
# it is a subtree from CLK120
ClkGroup
+ top_CLK_mux/CLKFanOutRoot/Z


# Clock Name: PTRG_FANOUT
# the pretrigger signal is distributed via LVDS and fanned out like the clock 
# signal by the tracklet mergers. The appropriate fan-out outputs are distributed
# accross the chip requiring buffering. Therefore this fan-out is defined as
# clock for simplicity. 
# Note: The phase between Pretrigger and CLK120 is not guaranteed.
ClkGroup
+ PTRG_IN/DI


# Clock Name: RESET (??? sinks)
# we generate a clock tree for RESET to avoid huge fan ins 
# (synopsys run with max fanout off)
ClkGroup
+ Itrap2top/ck/rst/rst_n_out_reg/Q


# the reset to the SCSN
ClkGroup
+ Itrap2top/ck/rst/pup_rst_n_out_reg/Q


#
# below are definitions of clock splines (no trees)
# they are defined for the clock tree tool not for skew but for transition time.
# ctgen inserts appropriate buffers to drive the in part long distances. Further
# all clock splines are shielded for noise protection.
#


# Clock Name: CLK_ADC_IN (1 sink)
# The input to the ADC clock (120 Mhz). This is a spline only.
ClkGroup
+ CLK_ADC_IN/DI


# Clock Name: CLK_ADC_OUT (2 sinks)
# This clock is generated inside the ADC block and used there for all ADCs, 
# where it is multiplied by two from the external 120 MHz reference input 
# (CLK_ADC_IN). 
ClkGroup
+ IADC/adc_i/CLK_OUT




# Clock Name: PLL_DIG_PAD (2 sinks)
# alternate digital reference clock input (for running the digital part
# independent of the ADC clock. This clock input can be fed directly into
# the digital part (bypassing the PLL480) or act as reference for the PLL480.
ClkGroup
+ CLK_DIG_IN/DI


# Clock Name: PLL_DIG_IN (1 sink)
# this clock spline is the input to the PLL480 auxiliary PLL. It may be derived
# either from ADC_CLK_OUT or PLL_DIG_PAD. There is no requirement for skew but
# max transition time.
ClkGroup
+ top_CLK_mux/i3/Z


################################################################################
# define the generic route types to be used for the clock trees
################################################################################

# Clock Name: CLK120_FANOUT
# this clk root is distributed accross the chip to four LVDS outputs, which are
# used to forward te clock to other MCMs. It is also fed into a mux allowing to
# use this clock or an external clock as reference into the PLL480.
# Clock Name: CLK_ADC_OUT (2 sinks)
# This clock is generated inside the ADC block and used there for all ADCs, 
# where it is multiplied by two from the external 120 MHz reference input 
# (CLK_ADC_IN). 
AutoCTSRootPin    	top_CLK_mux/CLKFanOutRoot/Z
RouteType		CLKSpline
NoGating          	rising
MaxDelay          	5000ps
MinDelay          	0ps
MaxSkew           	300ps
MaxDepth          	100
SinkMaxTran       	300ps
BufMaxTran        	300ps
Buffer			BUFBD2 BUFBD4 BUFBD8 BUFBD12 BUFBD16
RouteClkNet             NO
PostOpt                 YES         # this is the default
DetailReport            YES
End


# Clock Name: PTRG_FANOUT
# the pretrigger signal is distributed via LVDS and fanned out like the clock 
# signal by the tracklet mergers. The appropriate fan-out outputs are distributed
# accross the chip requiring buffering. Therefore this fan-out is defined as
# clock for simplicity. 
# Note: The phase between Pretrigger and CLK120 is not guaranteed.
AutoCTSRootPin    	PTRG_IN/DI
# we do not need shielding here
RouteType		CLKTree
NoGating          	rising
MaxDelay          	5000ps
MinDelay          	1000ps
MaxSkew           	300ps
MaxDepth          	100
SinkMaxTran       	300ps
BufMaxTran        	300ps
Buffer			BUFBD2 BUFBD4 BUFBD8 BUFBD12 BUFBD16
RouteClkNet             NO
PostOpt                 YES         # this is the default
DetailReport            YES
End


# RESET TREE ROOT
AutoCTSRootPin    	Itrap2top/ck/rst/rst_n_out_reg/Q
RouteType		CLKTree
NoGating          	rising
MaxDelay          	5000ps
MinDelay          	0ps
MaxSkew           	2000ps
MaxDepth          	100
SinkMaxTran       	1000ps
BufMaxTran        	1000ps
Buffer			BUFBD2 BUFBD4 BUFBD8 BUFBD12 BUFBD16
RouteClkNet             NO
PostOpt                 YES         # this is the default
DetailReport            YES
End


# RESET TREE ROOT for SCSN
AutoCTSRootPin    	Itrap2top/ck/rst/pup_rst_n_out_reg/Q
RouteType		CLKTree
NoGating          	rising
MaxDelay          	5000ps
MinDelay          	0ps
MaxSkew           	2000ps
MaxDepth          	100
SinkMaxTran       	1000ps
BufMaxTran        	1000ps
Buffer			BUFBD2 BUFBD4 BUFBD8 BUFBD12 BUFBD16
RouteClkNet             NO
PostOpt                 YES         # this is the default
DetailReport            YES
End


#
# below are definitions of clock splines (no trees)
# they are defined for the clock tree tool not for skew but for transition time.
# ctgen inserts appropriate buffers to drive the in part long distances. Further
# all clock splines are shielded for noise protection.
#

# Clock Name: CLK_ADC_IN (1 sink)
# The input to the ADC clock (120 Mhz). This is a spline only.
AutoCTSRootPin    	CLK_ADC_IN/DI
RouteType		CLKSpline
NoGating          	rising
MaxDelay          	5000ps
MinDelay          	0ps
MaxSkew           	300ps
MaxDepth          	100
SinkMaxTran       	300ps
BufMaxTran        	300ps
Buffer			BUFBD2 BUFBD4 BUFBD8 BUFBD12 BUFBD16
RouteClkNet             NO
PostOpt                 YES         # this is the default
DetailReport            YES
End


# Clock Name: CLK_ADC_OUT (2 sinks)
# This clock is generated inside the ADC block and used there for all ADCs, 
# where it is multiplied by two from the external 120 MHz reference input 
# (CLK_ADC_IN). 
AutoCTSRootPin    	IADC/adc_i/CLK_OUT
RouteType		CLKSpline
NoGating          	rising
MaxDelay          	5000ps
MinDelay          	0ps
MaxSkew           	300ps
MaxDepth          	100
SinkMaxTran       	300ps
BufMaxTran        	300ps
Buffer			BUFBD2 BUFBD4 BUFBD8 BUFBD12 BUFBD16
RouteClkNet             NO
PostOpt                 YES         # this is the default
DetailReport            YES
End


# Clock Name: PLL_DIG_PAD (2 sinks)
# alternate digital reference clock input (for running the digital part
# independent of the ADC clock. This clock input can be fed directly into
# the digital part (bypassing the PLL480) or act as reference for the PLL480.
AutoCTSRootPin    	CLK_DIG_IN/DI
RouteType		CLKSpline
LeafPin
+ top_CLK_mux/i5/A rising
NoGating          	rising
MaxDelay          	5000ps
MinDelay          	0ps
MaxSkew           	300ps
MaxDepth          	100
SinkMaxTran       	500ps
BufMaxTran        	500ps
Buffer			BUFBD2 BUFBD4 BUFBD8 BUFBD12 BUFBD16
RouteClkNet             NO
PostOpt                 YES         # this is the default
DetailReport            YES
End


# Clock Name: PLL_DIG_IN (1 sink)
# this clock spline is the input to the PLL480 auxiliary PLL. It may be derived
# either from ADC_CLK_OUT or PLL_DIG_PAD. There is no requirement for skew but
# max transition time.
AutoCTSRootPin    	top_CLK_mux/i3/Z
RouteType		CLKSpline
NoGating          	rising
MaxDelay          	5000ps
MinDelay          	0ps
MaxSkew           	300ps
MaxDepth          	100
SinkMaxTran       	500ps
BufMaxTran        	500ps
Buffer			BUFBD2 BUFBD4 BUFBD8 BUFBD12 BUFBD16
RouteClkNet             NO
PostOpt                 YES         # this is the default
DetailReport            YES
End
