-------------------------------------------------------------------------------
-- Title      : ni_delay
-- Project    : Network Interface (NI) for the ALICE TRD TRAP3
-------------------------------------------------------------------------------
-- File       : ni_delay.vhd
-- Author     : Rolf Schneider  <rolf.schneider@kip.uni-heidelberg.de>
-- Company    : University Heidelberg - KIP
-- Last update: 2003-12-11
-- Platform   : Synopsys - v2003.03
-------------------------------------------------------------------------------
-- Description: produces a single delay unit with a mux at the output
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author  Description
-- 2003/02/27  1.0      schneide	Created
-- 2003/12/11  1.1      schneide	Replaced DLY4D16 by DLY2D16
--                                      Checked input caps & driver strengths
-------------------------------------------------------------------------------


library IEEE;
use IEEE.std_logic_1164.all;

----------------------------------------------------------------------------------------------------
-- ENTITY
----------------------------------------------------------------------------------------------------
entity ni_delay is
  generic (
    depth    : integer := 2);           -- number of delay buffers
                                        -- note: the input of each delay cell is inverted
                                        --       => the depth should be a multiple of two
  port (
    sel      : in  std_logic;           -- '0' forward, '1' delay
    data_in  : in  std_logic;           -- incoming data
    data_out : out std_logic);          -- outgoing delayed data
end ni_delay;

----------------------------------------------------------------------------------------------------
-- ARCHITECTURE
----------------------------------------------------------------------------------------------------
architecture v of ni_delay is
  
  -- SIGNALS ---------------------------------------------------------------------------------------
  signal data_line     : std_logic_vector(0 to depth);
  signal data_line_inv : std_logic_vector(0 to depth);

  -- COMPONENTS ------------------------------------------------------------------------------------
  component INVD1
    port (
      Z : out std_logic;
      A : in  std_logic);
  end component;

  component DLY2D16
    port (
      Z : out std_logic;
      A : in  std_logic);
  end component;

  component MUX2D1
    port (
      Z  : out std_logic;
      SL : in  std_logic;
      A1 : in  std_logic;
      A0 : in  std_logic);
  end component;

    -- STRUCTURE -------------------------------------------------------------------------------------
begin

  data_line(0) <= data_in;

  -- MUX to switch the input directly to the output
  U_select: MUX2D1
      port map (
        Z  => data_out,
        SL => sel,
        A1 => data_line(depth),
        A0 => data_in);

  -- generate $depth stages of delay+inverter
  -- $depth should be an even number
  U_stages: for i in 1 to depth generate
    U_inv: INVD1
      port map (
        Z => data_line_inv(i-1),
        A => data_line(i-1));
    U_delay: DLY2D16
      port map (
        Z => data_line(i),
        A => data_line_inv(i-1));
  end generate;

end v;

----------------------------------------------------------------------------------------------------
-- CONFIGURATION
----------------------------------------------------------------------------------------------------
-- pragma translate_off
configuration ni_delay_CFG of ni_delay is
  for v
  end for; -- v
end ni_delay_CFG;
-- pragma translate_on

