-------------------------------------------------------------------------------
-- Title      : ni_delay_bit
-- Project    : Network Interface (NI) for the ALICE TRD TRAP3
-------------------------------------------------------------------------------
-- File       : ni_delay_bit.vhd
-- Author     : Rolf Schneider  <rolf.schneider@kip.uni-heidelberg.de>
-- Company    : University Heidelberg - KIP
-- Last update: 2003-12-11
-- Platform   : Synopsys - v2003.03
-------------------------------------------------------------------------------
-- Description: produces a delay line of a given depth for a single bit
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author  Description
-- 2003/02/27  1.0      schneide	Created
-------------------------------------------------------------------------------


library IEEE;
use IEEE.std_logic_1164.all;

----------------------------------------------------------------------------------------------------
-- ENTITY
----------------------------------------------------------------------------------------------------
entity ni_delay_bit is
  generic (
    depth    : integer := 3);           -- number of ni_delay stages. The max. delay would be
                                        -- 2**$depth * (0.36+0.448+0.032+0.025)ns + ~0.07ns
                                        -- cells     * (  delays  +  inverters)   + output buffer
  port (
    delay    : in  std_logic_vector(depth-1 downto 0);  -- delay: delay*0.865ns
    data_in  : in  std_logic;           -- incoming data
    data_out : out std_logic);          -- delayed data
end ni_delay_bit;

----------------------------------------------------------------------------------------------------
-- ARCHITECTURE
----------------------------------------------------------------------------------------------------
architecture v of ni_delay_bit is

  -- SIGNALS ---------------------------------------------------------------------------------------
  signal data_line : std_logic_vector(depth downto 0);

  -- COMPONENTS ------------------------------------------------------------------------------------
  component ni_delay
    generic (
      depth : integer);
    port (
      sel      : in  std_logic;
      data_in  : in  std_logic;
      data_out : out std_logic);
  end component;

  component BUFBD2
    port (
      Z : out std_logic;
      A : in  std_logic);
  end component;

  -- STRUCTURE -------------------------------------------------------------------------------------
begin

  data_line(depth) <= data_in;

  -- add a balaced output buffer at the end of a delay line
  U_buffer: BUFBD2
    port map (
      Z => data_out,
      A => data_line(0));

  -- generate a programmable delay for a single bit line
  U_stages: for i in depth-1 downto 0 generate
    U_delay: ni_delay
      generic map (
        depth    => 2**(i+1))           -- start with at least two delay cells/inverters
      port map (
        sel      => delay(i),
        data_in  => data_line(i+1),
        data_out => data_line(i));
  end generate;

end v;

----------------------------------------------------------------------------------------------------
-- CONFIGURATION
----------------------------------------------------------------------------------------------------
-- pragma translate_off
configuration ni_delay_bit_CFG of ni_delay_bit is
  for v
  end for; -- v
end ni_delay_bit_CFG;
-- pragma translate_on
