-------------------------------------------------------------------------------
-- Title      : ni_exclude_out
-- Project    : Network Interface (NI) for the ALICE TRD TRAP3
-------------------------------------------------------------------------------
-- File       : ni_exclude_out.vhd
-- Author     : Rolf Schneider  <rolf.schneider@kip.uni-heidelberg.de>
-- Company    : University Heidelberg - KIP
-- Last update: 2003-12-11
-- Platform   : Synopsys - v2003.03
-------------------------------------------------------------------------------
-- Description: insert redundate bit to outgoing data
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author  Description
-- 2003/02/27  1.0      schneide	Created
-------------------------------------------------------------------------------


library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

----------------------------------------------------------------------------------------------------
-- ENTITY
----------------------------------------------------------------------------------------------------
entity ni_exclude_out is
  generic (
    width    : integer := 10;                           -- word width incl. spare & parity bit
    depth    : integer := 4);                           -- position selection word width
  port (
    din      : in  std_logic_vector(width-3 downto 0);  -- data in
    prty_bit : in  std_logic;                           -- parity bit in
    sel_p    : in  std_logic_vector(depth-1 downto 0);  -- the parity bit position: "1000"
    sel_s    : in  std_logic_vector(depth-1 downto 0);  -- the unused bit position: "1001"
    dout     : out std_logic_vector(width-1 downto 0)   -- data out
    );
end ni_exclude_out;

----------------------------------------------------------------------------------------------------
-- ARCHITECTURE
----------------------------------------------------------------------------------------------------
architecture v of ni_exclude_out is

  -- SIGNALS ---------------------------------------------------------------------------------------
  signal q1pass : std_logic_vector(width-2 downto 0);
  signal q2pass : std_logic_vector(width-1 downto 0);
  signal spare  : std_logic;

  -- STRUCTURE -------------------------------------------------------------------------------------
begin

  spare <= 'X';

  -- insert parity bit into q1pass at position sel_p
  U_q1pass: with conv_integer(sel_p) select
        Q1pass <= din(7 downto 0) & prty_bit                   when 0,
                  din(7 downto 1) & prty_bit & din(         0) when 1,
                  din(7 downto 2) & prty_bit & din(1 downto 0) when 2,
                  din(7 downto 3) & prty_bit & din(2 downto 0) when 3,
                  din(7 downto 4) & prty_bit & din(3 downto 0) when 4,
                  din(7 downto 5) & prty_bit & din(4 downto 0) when 5,
                  din(7 downto 6) & prty_bit & din(5 downto 0) when 6,
                  din(7         ) & prty_bit & din(6 downto 0) when 7,
                                    prty_bit & din(7 downto 0) when others;

  -- insert spare bit into q2pass at position sel_s
  U_q2pass: with conv_integer(sel_s) select
        q2pass <= q1pass(8 downto 0) & spare                      when 0,
                  q1pass(8 downto 1) & spare & q1pass(         0) when 1,
                  q1pass(8 downto 2) & spare & q1pass(1 downto 0) when 2,
                  q1pass(8 downto 3) & spare & q1pass(2 downto 0) when 3,
                  q1pass(8 downto 4) & spare & q1pass(3 downto 0) when 4,
                  q1pass(8 downto 5) & spare & q1pass(4 downto 0) when 5,
                  q1pass(8 downto 6) & spare & q1pass(5 downto 0) when 6,
                  q1pass(8 downto 7) & spare & q1pass(6 downto 0) when 7,
                  q1pass(8         ) & spare & q1pass(7 downto 0) when 8,
                                       spare & q1pass(8 downto 0) when others;

  dout  <= q2pass;

end;

----------------------------------------------------------------------------------------------------
-- CONFIGURATION
----------------------------------------------------------------------------------------------------
-- pragma translate_off
configuration ni_exclude_out_CFG of ni_exclude_out is
  for v
  end for; -- v
end ni_exclude_out_CFG;
-- pragma translate_on
