-------------------------------------------------------------------------------
-- Title      : ni_fifo_mb
-- Project    : Network Interface (NI) for the ALICE TRD TRAP3
-------------------------------------------------------------------------------
-- File       : ni_fifo_mb.vhd
-- Author     : Rolf Schneider  <rolf.schneider@kip.uni-heidelberg.de>
-- Company    : University Heidelberg - KIP
-- Last update: 2003-12-11
-- Platform   : Synopsys - v2003.03
-------------------------------------------------------------------------------
-- Description: fifo controller with asynchronous pop for UMC memory builder's
--              synchronous read/write dual port register file.
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author  Description
-- 2003/03/10  1.0      schneide	Created
-- 2003/12/11  1.1      schneide	Output entries as status signal
-------------------------------------------------------------------------------


library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

----------------------------------------------------------------------------------------------------
-- ENTITY
----------------------------------------------------------------------------------------------------
entity ni_fifo_mb is
  generic (
    width : integer := 16;                              -- internal data bus width
    depth : integer := 7);                              -- fifo's memory address width
  port (
    clk          : in  std_logic;                       -- internal clock
    reset_n      : in  std_logic;                       -- ansynchronous reset
    clear_n      : in  std_logic;                       -- synchronous reset
    mode         : in  std_logic_vector(2 downto 0);    -- readout mode needed for end signature
    push_req_n   : in  std_logic;                       -- push into fifo
    pop_req_n    : in  std_logic;                       -- pop from fifo
    data_in      : in  std_logic_vector(width-1 downto 0); -- data input
    low_bound    : in  std_logic_vector(depth downto 0);   -- programmable lower bound
    high_bound   : in  std_logic_vector(depth downto 0);   -- programmable higher bound
    entries      : out std_logic_vector(depth downto 0);   -- entries in fifo
    low          : out std_logic;                       -- fifo <=low_bound
    high         : out std_logic;                       -- fifo >=high_bound
    empty        : out std_logic;                       -- fifo is empty signal
    almost_empty : out std_logic;                       -- one single word in fifo signal
    half_full    : out std_logic;                       -- fifo is half filled signal
    almost_full  : out std_logic;                       -- one single word left signal
    full         : out std_logic;                       -- fifo is full signal
    error        : out std_logic;                       -- async error, hold for one cycle
    word_low     : out std_logic;                       -- signals if low (high) word is read
    tr_es_word   : in  std_logic_vector(width-1 downto 0); -- end signature word (trigger readout)
    tr_es        : out std_logic;                       -- end sig detected (trigger readout)
    rr_es_word   : in  std_logic_vector(width-1 downto 0); -- end signature for raw data readout
    rr_es        : out std_logic;                       -- end sig detected (raw data readout)
    data_out     : out std_logic_vector(width-1 downto 0)); -- data ouput
end ni_fifo_mb;

----------------------------------------------------------------------------------------------------
-- ARCHITECTURE
----------------------------------------------------------------------------------------------------
architecture v of ni_fifo_mb is
  
  -- CONSTANTS -------------------------------------------------------------------------------------
  constant ctl_min : std_logic_vector(depth downto 0) := conv_std_logic_vector(0, depth+1);
  constant ctl_max : std_logic_vector(depth downto 0) := conv_std_logic_vector(2**depth, depth+1);

  -- SIGNALS ---------------------------------------------------------------------------------------
  signal data_in_i     : std_logic_vector(width-1 downto 0);
  signal pop_req_n_i   : std_logic;
  signal word_low_i    : std_logic;
  signal error_i       : std_logic;
  signal data_out_i    : std_logic_vector(width-1 downto 0);

  signal mem_REN       : std_logic;
  signal mem_RADR      : std_logic_vector(depth-1 downto 0);
  signal mem_WEN       : std_logic;
  signal mem_WADR      : std_logic_vector(depth-1 downto 0);
  signal mem_DI        : std_logic_vector(width-1 downto 0);
  signal mem_DOUT      : std_logic_vector(width-1 downto 0);

  signal ctl_push_n    : std_logic;
  signal ctl_pop_n     : std_logic;
  signal ctl_pop_n_reg : std_logic;
  signal ctl_entries   : std_logic_vector(depth downto 0);

  signal mem_din_reg   : std_logic_vector(width-1 downto 0);
  signal mem_dout_reg  : std_logic_vector(width-1 downto 0);

  -- COMPONENTS ------------------------------------------------------------------------------------
  component R128X16M1  -- VST memory builder component with fixed sizes
    port (
      RCK  : in  std_logic;
      REN  : in  std_logic;
      RADR : in  std_logic_vector(6 downto 0);
      WCK  : in  std_logic;
      WEN  : in  std_logic;
      WADR : in  std_logic_vector(6 downto 0);
      DI   : in  std_logic_vector(15 downto 0);
      DOUT : out std_logic_vector(15 downto 0));
  end component;

  -- STRUCTURE -------------------------------------------------------------------------------------
begin

  -- if fifo is full push as last low word the end signature into the fifo
  U_full_end: process(mode, ctl_entries, word_low_i, tr_es_word, rr_es_word, data_in)
  begin
    data_in_i <= data_in;
    case mode is
      -- idle
      when "000" => null;
      -- trigger readout
      when "001" => if (ctl_entries = conv_std_logic_vector((2**depth)-2, width+1) and word_low_i = '1') or
                      (ctl_entries = conv_std_logic_vector((2**depth)-1, width+1) and word_low_i = '0') then
                     data_in_i <= tr_es_word;
                   end if;
      -- raw data readout
      when "010" => if (ctl_entries = conv_std_logic_vector((2**depth)-2, width+1) and word_low_i = '1') or
                      (ctl_entries = conv_std_logic_vector((2**depth)-1, width+1) and word_low_i = '0') then
                     data_in_i <= rr_es_word;
                   end if;
      -- network mode
      when others => null;
    end case;
  end process;

  -- select output out of input, registered input, mem output, registered mem output
  -- produce additional pop signal
  data_out <= data_out_i;
  U_mem_sel: process(push_req_n, pop_req_n, ctl_pop_n_reg, ctl_entries, data_in_i, mem_din_reg, mem_DOUT, mem_dout_reg)
  begin
    ctl_push_n <= push_req_n;
    ctl_pop_n  <= pop_req_n;
    data_out_i <= mem_dout_reg;

    if (ctl_entries = conv_std_logic_vector(0, depth+1)) then
      data_out_i <= data_in_i;
    elsif (ctl_entries = conv_std_logic_vector(1, depth+1)) then
      data_out_i <= mem_din_reg;
      if (push_req_n = '0' and pop_req_n = '1') then ctl_pop_n <= '0'; end if;
    elsif (ctl_entries = conv_std_logic_vector(2, depth+1)) then
      if ctl_pop_n_reg = '0' then data_out_i <= mem_DOUT; end if;
    else
      if ctl_pop_n_reg = '0' then data_out_i <= mem_DOUT; end if;
    end if;
  end process;


  -- fifo address counters for memory
  U_rw_addr: process(clk, reset_n, clear_n, ctl_pop_n, ctl_push_n, ctl_entries, mem_RADR, mem_WADR)
  begin
    if reset_n = '0' then
      mem_RADR    <= (others => '0');
      mem_WADR    <= (others => '0');
    elsif rising_edge(clk) then
      mem_RADR <= mem_RADR;
      mem_WADR <= mem_WADR;
      if (ctl_push_n = '0' and ctl_entries /= ctl_max) then mem_WADR <= mem_WADR + 1; end if;
      if (ctl_pop_n = '0'  and ctl_entries /= ctl_min) then mem_RADR <= mem_RADR + 1; end if;
      if (ctl_push_n = '0' and ctl_pop_n = '0') then mem_RADR <= mem_RADR + 1;
                                                     mem_WADR <= mem_WADR + 1; end if;
      if clear_n = '0' then mem_RADR <= (others => '0');
                            mem_WADR <= (others => '0');
      end if;
    end if;
  end process;

  -- count entries of fifo separate from addresses (additional pop)
  U_entries: process(clk, reset_n, clear_n, push_req_n, pop_req_n, ctl_entries)
  begin
    if reset_n = '0' then
      ctl_entries <= (others => '0');
    elsif rising_edge(clk) then
      ctl_entries <= ctl_entries;
      if (push_req_n = '0' and ctl_entries /= ctl_max) then ctl_entries <= ctl_entries + 1; end if;
      if (pop_req_n = '0'  and ctl_entries /= ctl_min) then ctl_entries <= ctl_entries - 1; end if;
      if (push_req_n = '0' and pop_req_n = '0') then ctl_entries <= ctl_entries; end if;
      if clear_n = '0' then ctl_entries <= (others => '0'); end if;
    end if;
  end process;

  -- synchronous read/write memory
  mem_REN <= '0'; -- ctl_pop_n;
  mem_WEN <= ctl_push_n or error_i;
  mem_DI  <= data_in_i;
  U_mem: R128X16M1
    port map (
      RCK  => clk,
      REN  => mem_REN,
      RADR => mem_RADR,
      WCK  => clk,
      WEN  => mem_WEN,
      WADR => mem_WADR,
      DI   => mem_DI,
      DOUT => mem_DOUT);

  -- register for data input with push as write enable
  U_mem_din_reg: process(clk, reset_n, clear_n, mem_WEN, mem_DI, mem_din_reg)
  begin
    if reset_n = '0' then
      mem_din_reg <= (others => '0');
    elsif rising_edge(clk) then
      if mem_WEN = '0' then mem_din_reg <= mem_DI;
                       else mem_din_reg <= mem_din_reg;
      end if;
      if clear_n = '0' then mem_din_reg <= (others => '0'); end if;
    end if;
  end process;

  -- delay pop request for memory output register
  U_ctl_pop_n_reg: process(clk, reset_n, clear_n, ctl_pop_n)
  begin
    if reset_n = '0' then
      ctl_pop_n_reg <= '1';
    elsif rising_edge(clk) then
      ctl_pop_n_reg <= ctl_pop_n;
    end if;
  end process;

  -- register for memory data output
  U_mem_dout_reg: process(clk, reset_n, clear_n, ctl_pop_n_reg, mem_DOUT, mem_dout_reg)
  begin
    if reset_n = '0' then
      mem_dout_reg <= (others => '0');
    elsif rising_edge(clk) then
      if ctl_pop_n_reg = '0' then mem_dout_reg <= mem_DOUT;
                             else mem_dout_reg <= mem_dout_reg;
      end if;
      if clear_n = '0' then mem_dout_reg <= (others => '0'); end if;
    end if;
  end process;

  -- status signals
  entries      <= ctl_entries;
  low          <= '1' when ctl_entries <= low_bound  else '0';
  high         <= '1' when ctl_entries >= high_bound else '0';
  empty        <= '1' when ctl_entries  = conv_std_logic_vector(0, depth+1)            else '0';
  almost_empty <= '1' when ctl_entries <= conv_std_logic_vector(1, depth+1)            else '0';
  half_full    <= '1' when ctl_entries >= conv_std_logic_vector((2**depth)/2, depth+1) else '0';
  almost_full  <= '1' when ctl_entries >= conv_std_logic_vector((2**depth)-1, depth+1) else '0';
  full         <= '1' when ctl_entries  = conv_std_logic_vector(2**depth, depth+1)     else '0';
  error <= error_i;
  error_i <= '1' when ((ctl_entries = ctl_min and pop_req_n = '0' and push_req_n = '1') or
                       (ctl_entries = ctl_max and push_req_n = '0' and pop_req_n = '1')) else '0';

  word_low <= word_low_i;
  U_word_low: process(clk, reset_n, clear_n, pop_req_n, error_i, word_low_i)
  begin
    if reset_n = '0' then
      word_low_i <= '1';
    elsif rising_edge(clk) then
      if (pop_req_n = '0' and error_i = '0') then
        word_low_i <= not word_low_i;
      end if;
      if clear_n = '0' then word_low_i <= '1'; end if;
    end if;
  end process;

  tr_es <= '1' when (data_out_i = tr_es_word and word_low_i = '1' and error_i = '0') else '0';
  rr_es <= '1' when (data_out_i = rr_es_word and word_low_i = '1' and error_i = '0') else '0';

end v;

----------------------------------------------------------------------------------------------------
-- CONFIGURATION
----------------------------------------------------------------------------------------------------
-- pragma translate_off
configuration ni_fifo_mb_CFG of ni_fifo_mb is
  for v
  end for; -- v
end ni_fifo_mb_CFG;
-- pragma translate_on
