-------------------------------------------------------------------------------
-- Title      : ni_fsm
-- Project    : Network Interface (NI) for the ALICE TRD TRAP3
-------------------------------------------------------------------------------
-- File       : ni_fsm.vhd
-- Author     : Rolf Schneider  <rolf.schneider@kip.uni-heidelberg.de>
-- Company    : University Heidelberg - KIP
-- Last update: 2003-12-12
-- Platform   : Synopsys - v2003.03
-------------------------------------------------------------------------------
-- Description: main FSM for the NI
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author  Description
-- 2003/03/19  1.0      schneide	Created
-- 2003/12/11  1.1      schneide	Output fsmstate as status,
--                                      Clean get_next_??_state functions
-- 2003/12/11  1.2      schneide	Added fifo_clr_disable
-------------------------------------------------------------------------------


library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

----------------------------------------------------------------------------------------------------
-- ENTITY
----------------------------------------------------------------------------------------------------
entity ni_fsm is

  port (
    clk              : in  std_logic;                     -- internal clock
    reset_n          : in  std_logic;                     -- asynchronous reset
    clear_n          : in  std_logic;                     -- synchronous reset
    finished_n       : out std_logic;                     -- signals MSM end of readout
    fsm_id           : out std_logic_vector(4 downto 0);  -- current fsm state
    -- global io ctrl
    mode             : in  std_logic_vector(2 downto 0);  -- readout mode from ctrl register
    mode_fsm         : out std_logic_vector(2 downto 0);  -- current readout mode of the FSM
    order_root       : in  std_logic;                     -- root (of raw data readout)
    order_tr         : in  std_logic_vector(17 downto 0); -- order to process int/ext data tr streams
    order_rr         : in  std_logic_vector(17 downto 0); -- order to process int/ext data rr streams
    word_cut         : in  std_logic_vector(31 downto 0); -- cut readout stream
    word_cnt         : in  std_logic_vector(31 downto 0); -- number of incoming words
    fifo_clr_disable : in  std_logic;                     -- disable the fifo clear signal
    -- local io
    lio_bus_din_we_n : out std_logic_vector(7 downto 0);  -- write low/high word to data bus register
    lio_bus_dout_low : out std_logic_vector(3 downto 0);  -- select low/high word from data in register
    lio_data_valid   : out std_logic_vector(3 downto 0);  -- status bit: local io bus data is valid
    lio_ctrl         : in  std_logic_vector(3 downto 0);  -- control signal from configuration
    lio_ctrl_en_n    : in  std_logic_vector(3 downto 0);  -- control enable from configuration
    lio_dout_r_n     : in  std_logic_vector(3 downto 0);  -- CPU read from data register
    lio_din_w_n      : in  std_logic_vector(3 downto 0);  -- CPU wrote to data register
    lio_tr_es        : in  std_logic_vector(3 downto 0);  -- low word of data in reg is trigger end sig
    lio_rr_es        : in  std_logic_vector(3 downto 0);  -- low word of data in reg is raw data end sig
    -- port in
    pi_clear_n       : out std_logic;                     -- sync reset for input ports
    pi_ctrl_in       : out std_logic_vector(3 downto 0);  -- ctrl output to input ports
    pi_we_n          : in  std_logic_vector(3 downto 0);  -- data valid from input ports
    pi_word_low      : in  std_logic_vector(3 downto 0);  -- word of input port is low word
    pi_tr_es         : in  std_logic_vector(3 downto 0);  -- low word of input port is trigger end sig
    pi_rr_es         : in  std_logic_vector(3 downto 0);  -- low word of input port is raw data end sig
    -- p2f mux
    sel_p2f          : out std_logic;                     -- connect in port to fifo (else local io)
    -- fifo
    f_clear_n        : out std_logic;                     -- sync reset for fifo fsm
    f_push_req_n     : out std_logic_vector(3 downto 0);  -- push word into fifos
    f_pop_req_n      : out std_logic_vector(3 downto 0);  -- pop wird from fifos
    f_high           : in  std_logic_vector(3 downto 0);  -- fifo is >= programmable high bound
    f_empty          : in  std_logic_vector(3 downto 0);  -- fifo is empty signals
    f_almost_empty   : in  std_logic_vector(3 downto 0);  -- fifo is almost empty signals
    f_almost_full    : in  std_logic_vector(3 downto 0);  -- fifo is almost full signals
    f_full           : in  std_logic_vector(3 downto 0);  -- fifo is full signals
    f_error          : in  std_logic_vector(3 downto 0);  -- error from fifo FSMs
    f_word_low       : in  std_logic_vector(3 downto 0);  -- low word is read from fifo
    f_tr_es          : in  std_logic_vector(3 downto 0);  -- low word of fifo is trigger end sig
    f_rr_es          : in  std_logic_vector(3 downto 0);  -- low word of fifo is raw data end sig
    -- f2p mux
    sel_f2p          : out std_logic_vector(1 downto 0);  -- connect fifo to out port (else local io)
    -- num mux
    sel_num          : out std_logic_vector(1 downto 0);  -- selects number of fifo/local io to out port
    sel_e2p          : out std_logic_vector(2 downto 0);  -- selects between data, es and tp
    -- port out
    po_clear_n       : out std_logic;                     -- sync reset for output port
    po_ctrl_out      : in  std_logic;                     -- control input of output port
    po_we_n          : out std_logic);                    -- data valid to output port
end ni_fsm;

----------------------------------------------------------------------------------------------------
-- ARCHITECTURE
----------------------------------------------------------------------------------------------------
architecture v of ni_fsm is

  constant p0  : std_logic_vector(2 downto 0) := "000";
  constant p1  : std_logic_vector(2 downto 0) := "001";
  constant p2  : std_logic_vector(2 downto 0) := "010";
  constant p3  : std_logic_vector(2 downto 0) := "011";
  constant int : std_logic_vector(2 downto 0) := "100";
  constant lst : std_logic_vector(2 downto 0) := "111";

  constant lio  : std_logic_vector(1 downto 0) := "00";
  constant fifo : std_logic_vector(1 downto 0) := "01";
  constant pi   : std_logic_vector(1 downto 0) := "10";

  constant clear_id      : integer :=  0;
  constant idle_id       : integer :=  1;
  constant idle_tr_id    : integer :=  2;
  constant idle_rr_id    : integer :=  3;
  constant idle_net_id   : integer :=  4;
  constant send_tri0l_id : integer :=  5;
  constant send_tri0h_id : integer :=  6;
  constant send_tri1l_id : integer :=  7;
  constant send_tri1h_id : integer :=  8;
  constant send_tri2l_id : integer :=  9;
  constant send_tri2h_id : integer := 10;
  constant send_tri3l_id : integer := 11;
  constant send_tri3h_id : integer := 12;
  constant send_trp0_id  : integer := 13;
  constant send_trp1_id  : integer := 14;
  constant send_trp2_id  : integer := 15;
  constant send_trp3_id  : integer := 16;
  constant send_rrf0_id  : integer := 17;
  constant send_rrf1_id  : integer := 18;
  constant send_rrf2_id  : integer := 19;
  constant send_rrf3_id  : integer := 20;
  constant send_rrp0_id  : integer := 21;
  constant send_rrp1_id  : integer := 22;
  constant send_rrp2_id  : integer := 23;
  constant send_rrp3_id  : integer := 24;
  constant finished_id   : integer := 25;
  constant send_tpl_id   : integer := 26;
  constant send_tph_id   : integer := 27;

  -- SIGNALS ---------------------------------------------------------------------------------------
  subtype bus_we_type is std_logic_vector(1 downto 0);
  type    bus_we_array is array(3 downto 0) of bus_we_type;
  subtype order_type is std_logic_vector(2 downto 0);
  type    order_array is array(3 downto 0) of order_type;
  subtype wc_type is std_logic_vector(7 downto 0);
  type    wc_array is array(3 downto 0) of wc_type;
  signal lio_bus_din_we_n_i : bus_we_array;
  signal wcut               : wc_array;
  signal wcnt               : wc_array;
  signal cut_stream         : std_logic_vector(3 downto 0);
  signal order_tr_int       : std_logic_vector(2 downto 0);
  signal order_tr_start     : std_logic_vector(2 downto 0);
  signal order_tr_p         : order_array;
  signal order_rr_int       : std_logic_vector(2 downto 0);
  signal order_rr_start     : std_logic_vector(2 downto 0);
  signal order_rr_p         : order_array;
  signal cpu_wrote_n        : std_logic_vector(3 downto 0);
  signal cpu_wrote_reset_n  : std_logic_vector(3 downto 0);
  signal we_es_n            : std_logic_vector(3 downto 0);
  signal din_cnt            : bus_we_array;
  signal net_mode           : std_logic;
  signal push_din, dout_low : std_logic_vector(3 downto 0);
  signal net_pop_req_n      : std_logic_vector(3 downto 0);
  signal f_pop_req_n_i      : std_logic_vector(3 downto 0);
  signal clear_n_i          : std_logic;
  signal po_we_n_i          : std_logic;
  signal mode_we_n          : std_logic;
  signal mode_clear_n       : std_logic;
  signal mode_fsm_i         : std_logic_vector(2 downto 0);

  -- states for four FSMs controlling data exchange with CPUs
  type net_fsm_type is (
    RESET_STATE,
    CLEAR_STATE,
    WRITE_LOW_STATE,
    WRITE_HIGH_STATE,
    WAIT_READ_STATE);
  subtype net_fsm_array_type is net_fsm_type;
  type net_fsm_array is array (3 downto 0) of net_fsm_array_type;
  signal net_fsm_state, net_fsm_next_state: net_fsm_array;

  -- states for master FSM
  type fsm_type is (
    RESET_STATE,                        -- asynchronous reset
    CLEAR_STATE,                        -- state after reset or sync. clear
    IDLE_STATE,                         -- wait for readout mode
    IDLE_TR_STATE,                      -- ready for trigger readout
    IDLE_RR_STATE,                      -- ready for raw data readout
    IDLE_NET_STATE,                     -- ready for CPU network
    SEND_TRI0L_STATE,                   -- send tracklet data from cpu 0 (low word)
    SEND_TRI0H_STATE,                   -- send tracklet data from cpu 0 (high word)
    SEND_TRI1L_STATE,                   -- send tracklet data from cpu 1 (low word)
    SEND_TRI1H_STATE,                   -- send tracklet data from cpu 1 (high word)
    SEND_TRI2L_STATE,                   -- send tracklet data from cpu 2 (low word)
    SEND_TRI2H_STATE,                   -- send tracklet data from cpu 2 (high word)
    SEND_TRI3L_STATE,                   -- send tracklet data from cpu 3 (low word)
    SEND_TRI3H_STATE,                   -- send tracklet data from cpu 3 (high word)
    SEND_TRP0_STATE,                    -- send tracklet data from port0
    SEND_TRP1_STATE,                    -- send tracklet data from port1
    SEND_TRP2_STATE,                    -- send tracklet data from port2
    SEND_TRP3_STATE,                    -- send tracklet data from port3
    SEND_TRES_STATE0,                   -- send trigger readout end signature
    SEND_TRES_STATE1,                   -- send trigger readout end signature
    SEND_TRES_STATE2,                   -- send trigger readout end signature
    SEND_TRES_STATE3,                   -- send trigger readout end signature
    SEND_RRF0_STATE,                    -- send raw data from fifo 0 (cpu 0 data)
    SEND_RRF1_STATE,                    -- send raw data from fifo 1 (cpu 1 data)
    SEND_RRF2_STATE,                    -- send raw data from fifo 2 (cpu 2 data)
    SEND_RRF3_STATE,                    -- send raw data from fifo 3 (cpu 3 data)
    SEND_RRP0_STATE,                    -- send raw data from port 0
    SEND_RRP1_STATE,                    -- send raw data from port 1
    SEND_RRP2_STATE,                    -- send raw data from port 2
    SEND_RRP3_STATE,                    -- send raw data from port 3
    SEND_RRES_STATE0,                   -- send raw data end signature
    SEND_RRES_STATE1,                   -- send raw data end signature
    SEND_RRES_STATE2,                   -- send raw data end signature
    SEND_RRES_STATE3,                   -- send raw data end signature
    FINISHED_STATE,                     -- signal master state machine end of readout
    SEND_TPL_STATE,                     -- send low word of test pattern
    SEND_TPH_STATE                      -- send high word of test pattern
  );
  signal fsm_state, fsm_next_state: fsm_type;

  -- COMPONENTS ------------------------------------------------------------------------------------

  -- FUNCTIONS -------------------------------------------------------------------------------------

  -- return the state for trigger readout depending on current source number and readout order
  function get_next_tr_state (order_next: std_logic_vector(2 downto 0)) return fsm_type is
  begin
    case order_next is
      when p0  => return SEND_TRP0_STATE;
      when p1  => return SEND_TRP1_STATE;
      when p2  => return SEND_TRP2_STATE;
      when p3  => return SEND_TRP3_STATE;
      when int => return SEND_TRI0L_STATE;
      when lst => return SEND_TRES_STATE0;
      when others => return CLEAR_STATE;
    end case;
  end get_next_tr_state;

  -- return the state for raw data readout depending on current source number and readout order
  function get_next_rr_state (order_next : std_logic_vector(2 downto 0)) return fsm_type is
  begin
    case order_next is
      when p0  => return SEND_RRP0_STATE;
      when p1  => return SEND_RRP1_STATE;
      when p2  => return SEND_RRP2_STATE;
      when p3  => return SEND_RRP3_STATE;
      when int => return SEND_RRF0_STATE;
      when lst => return SEND_RRES_STATE0;
      when others => return CLEAR_STATE;
    end case;
  end get_next_rr_state;


  -- STRUCTURE -------------------------------------------------------------------------------------
begin

  we_es_n <= pi_we_n or not f_tr_es;

  order_tr_start <= order_tr(2 downto 0);
  order_tr_p(0)  <= order_tr(5 downto 3);
  order_tr_p(1)  <= order_tr(8 downto 6);
  order_tr_p(2)  <= order_tr(11 downto 9);
  order_tr_p(3)  <= order_tr(14 downto 12);
  order_tr_int   <= order_tr(17 downto 15);

  order_rr_start <= order_rr(2 downto 0);
  order_rr_p(0)  <= order_rr(5 downto 3);
  order_rr_p(1)  <= order_rr(8 downto 6);
  order_rr_p(2)  <= order_rr(11 downto 9);
  order_rr_p(3)  <= order_rr(14 downto 12);
  order_rr_int   <= order_rr(17 downto 15);

  f_pop_req_n <= f_pop_req_n_i and net_pop_req_n;
  lio_bus_din_we_n <= lio_bus_din_we_n_i(3) & lio_bus_din_we_n_i(2) & lio_bus_din_we_n_i(1) & lio_bus_din_we_n_i(0);

  U_wcut: for i in 3 downto 0 generate
    wcut(i) <= word_cut((i*8)+7 downto i*8);
    wcnt(i) <= word_cnt((i*8)+7 downto i*8);
    cut_stream(i) <= '1' when wcnt(i) >= wcut(i) and pi_word_low(i) = '1' else '0';
  end generate;

  po_we_n  <= po_we_n_i;
  mode_fsm <= mode_fsm_i;

  -- main FSM --------------------------------------------------------------------------------------

                                                                                        -- memory --
  U_fsm_mem: process(clk, reset_n, clear_n)
  begin
    if reset_n = '0' then
      fsm_state <= RESET_STATE;
    elsif rising_edge(clk) then
      fsm_state <= fsm_next_state;
      if clear_n = '0' then fsm_state <= CLEAR_STATE; end if;
    end if;
  end process;

                                                                                        -- states --
  U_fsm_state: process(clk, reset_n, fsm_state, pi_rr_es, order_rr_p, f_rr_es, po_ctrl_out, order_root, f_tr_es, cut_stream, order_tr_p, cpu_wrote_n, lio_tr_es, mode, order_rr_int, order_rr_start, order_tr_int, order_tr_start)
    variable src : integer;
  begin

    src := 0;

    case fsm_state is
      -- INIT --------------------------------------------------------------------------------------
      when RESET_STATE => fsm_next_state <= CLEAR_STATE;
      when CLEAR_STATE => fsm_next_state <= IDLE_STATE;
      when IDLE_STATE  => fsm_next_state <= IDLE_STATE;
                          case mode is
                            when "000"   => fsm_next_state <= IDLE_STATE;
                            when "001"   => fsm_next_state <= IDLE_TR_STATE;
                            when "010"   => fsm_next_state <= IDLE_RR_STATE;
                            when "011"   => fsm_next_state <= IDLE_NET_STATE;
                            when "100"   => fsm_next_state <= SEND_TPL_STATE;
                            when others  => fsm_next_state <= CLEAR_STATE;
                          end case;
      -- TRIGGER READOUT ---------------------------------------------------------------------------
      when IDLE_TR_STATE    => fsm_next_state <= get_next_tr_state(order_tr_start);
      when SEND_TRI0L_STATE => src := 0;
                               fsm_next_state <= SEND_TRI0L_STATE;
                               if cpu_wrote_n(src) = '0' then
                                 if lio_tr_es(src) = '1' then fsm_next_state <= SEND_TRI1L_STATE;
                                                         else fsm_next_state <= SEND_TRI0H_STATE;
                                 end if;
                               end if;
      when SEND_TRI0H_STATE => fsm_next_state <= SEND_TRI1L_STATE;
      when SEND_TRI1L_STATE => src := 1;
                               fsm_next_state <= SEND_TRI1L_STATE;
                               if cpu_wrote_n(src) = '0' then
                                 if lio_tr_es(src) = '1' then fsm_next_state <= SEND_TRI2L_STATE;
                                                         else fsm_next_state <= SEND_TRI1H_STATE;
                                 end if;
                               end if;
      when SEND_TRI1H_STATE => fsm_next_state <= SEND_TRI2L_STATE;
      when SEND_TRI2L_STATE => src := 2;
                               fsm_next_state <= SEND_TRI2L_STATE;
                               if cpu_wrote_n(src) = '0' then
                                 if lio_tr_es(src) = '1' then fsm_next_state <= SEND_TRI3L_STATE;
                                                         else fsm_next_state <= SEND_TRI2H_STATE;
                                 end if;
                               end if;
      when SEND_TRI2H_STATE => fsm_next_state <= SEND_TRI3L_STATE;
      when SEND_TRI3L_STATE => src := 3;
                               fsm_next_state <= SEND_TRI3L_STATE;
                               if cpu_wrote_n(src) = '0' then
                                 if lio_tr_es(src) = '1' then
                                   fsm_next_state <= get_next_tr_state(order_tr_int);
                                 else
                                   fsm_next_state <= SEND_TRI3H_STATE;
                                 end if;
                               end if;
      when SEND_TRI3H_STATE => fsm_next_state <= get_next_tr_state(order_tr_int);
      when SEND_TRP0_STATE  => src := 0;
                               fsm_next_state <= SEND_TRP0_STATE;
                               if f_tr_es(src) = '1' or cut_stream(src) = '1' then
                                 fsm_next_state <= get_next_tr_state(order_tr_p(src));
                               end if;
      when SEND_TRP1_STATE  => src := 1;
                               fsm_next_state <= SEND_TRP1_STATE;
                               if f_tr_es(src) = '1' or cut_stream(src) = '1' then
                                 fsm_next_state <= get_next_tr_state(order_tr_p(src));
                               end if;
      when SEND_TRP2_STATE  => src := 2;
                               fsm_next_state <= SEND_TRP2_STATE;
                               if f_tr_es(src) = '1' or cut_stream(src) = '1' then
                                 fsm_next_state <= get_next_tr_state(order_tr_p(src));
                               end if;
      when SEND_TRP3_STATE  => src := 3;
                               fsm_next_state <= SEND_TRP3_STATE;
                               if f_tr_es(src) = '1' or cut_stream(src) = '1' then
                                 fsm_next_state <= get_next_tr_state(order_tr_p(src));
                               end if;
      when SEND_TRES_STATE0 => fsm_next_state <= SEND_TRES_STATE1;
      when SEND_TRES_STATE1 => fsm_next_state <= SEND_TRES_STATE2;
      when SEND_TRES_STATE2 => fsm_next_state <= SEND_TRES_STATE3;
      when SEND_TRES_STATE3 => fsm_next_state <= FINISHED_STATE;
      -- RAW DATA READOUT --------------------------------------------------------------------------
      when IDLE_RR_STATE    => fsm_next_state <= get_next_rr_state(order_rr_start);
                               if (po_ctrl_out = '0' and order_root = '0') then
                                 fsm_next_state <= IDLE_RR_STATE;
                               end if;
      when SEND_RRF0_STATE  => src := 0;
                               fsm_next_state <= SEND_RRF0_STATE;
                               if f_rr_es(src) = '1' then fsm_next_state <= SEND_RRF1_STATE; end if;
      when SEND_RRF1_STATE  => src := 1;
                               fsm_next_state <= SEND_RRF1_STATE;
                               if f_rr_es(src) = '1' then fsm_next_state <= SEND_RRF2_STATE; end if;
      when SEND_RRF2_STATE  => src := 2;
                               fsm_next_state <= SEND_RRF2_STATE;
                               if f_rr_es(src) = '1' then fsm_next_state <= SEND_RRF3_STATE; end if;
      when SEND_RRF3_STATE  => src := 3;
                               fsm_next_state <= SEND_RRF3_STATE;
                               if f_rr_es(src) = '1' then
                                 fsm_next_state <= get_next_rr_state(order_rr_int);
                               end if;
      when SEND_RRP0_STATE  => src := 0;
                               fsm_next_state <= SEND_RRP0_STATE;
                               if pi_rr_es(src) = '1' then
                                 fsm_next_state <= get_next_rr_state(order_rr_p(src));
                               end if;
      when SEND_RRP1_STATE  => src := 1;
                               fsm_next_state <= SEND_RRP1_STATE;
                               if pi_rr_es(src) = '1' then
                                 fsm_next_state <= get_next_rr_state(order_rr_p(src));
                               end if;
      when SEND_RRP2_STATE  => src := 2;
                               fsm_next_state <= SEND_RRP2_STATE;
                               if pi_rr_es(src) = '1' then
                                 fsm_next_state <= get_next_rr_state(order_rr_p(src));
                               end if;
      when SEND_RRP3_STATE  => src := 3;
                               fsm_next_state <= SEND_RRP3_STATE;
                               if pi_rr_es(src) = '1' then
                                 fsm_next_state <= get_next_rr_state(order_rr_p(src));
                               end if;
      when SEND_RRES_STATE0 => fsm_next_state <= SEND_RRES_STATE1;
      when SEND_RRES_STATE1 => fsm_next_state <= SEND_RRES_STATE2;
      when SEND_RRES_STATE2 => fsm_next_state <= SEND_RRES_STATE3;
      when SEND_RRES_STATE3 => fsm_next_state <= FINISHED_STATE;
      -- FINISHED ----------------------------------------------------------------------------------
      when FINISHED_STATE   => fsm_next_state <= CLEAR_STATE;
      -- NETWORK MODE ------------------------------------------------------------------------------
      when IDLE_NET_STATE   => fsm_next_state <= IDLE_NET_STATE;
      -- TEST PATTERN MODE -------------------------------------------------------------------------
      when SEND_TPL_STATE   => fsm_next_state <= SEND_TPH_STATE;
      when SEND_TPH_STATE   => fsm_next_state <= SEND_TPL_STATE;
      -- OTHERWISE ---------------------------------------------------------------------------------
      when others           => fsm_next_state <= CLEAR_STATE;
    end case;
  end process;

                                                                                      -- output --
  U_fsm_out: process(clk, reset_n, fsm_state, pi_we_n, order_tr_start, dout_low, lio_ctrl_en_n, lio_ctrl, f_high, push_din, pi_rr_es, f_error, f_rr_es, f_tr_es, cpu_wrote_n, lio_tr_es, fifo_clr_disable)
    variable src : integer;
    variable num : std_logic_vector(1 downto 0);
  begin
    -- default values
    fsm_id           <= (others => '1');
    finished_n       <= '1';
    clear_n_i        <= '1';
    pi_clear_n       <= '1';
    f_clear_n        <= '1';
    po_clear_n       <= '1';
    src              := 0;
    lio_bus_dout_low <= (others => '1');
    pi_ctrl_in       <= (others => '0');
    sel_p2f          <= '1';
    f_push_req_n     <= (others => '1');
    f_pop_req_n_i    <= (others => '1');
    sel_f2p          <= fifo;
    sel_num          <= "00";
    sel_e2p          <= "000";
    po_we_n_i        <= '1';
    mode_we_n        <= '1';
    mode_clear_n     <= '1';

    net_mode          <= '0';
    cpu_wrote_reset_n <= (others => '1');

    case fsm_state is
      -- INIT ------------------------------------------------------------------------------------
      when RESET_STATE   =>    clear_n_i <= '0';
                               pi_clear_n <= '0';
                               f_clear_n <= '0';
                               po_clear_n <= '0';
                               cpu_wrote_reset_n <= (others => '0');
                               mode_clear_n <= '0';
      when CLEAR_STATE   =>    clear_n_i <= '0';
                               pi_clear_n <= '0';
                               f_clear_n <= '0' or fifo_clr_disable;
                               po_clear_n <= '0';
                               cpu_wrote_reset_n <= (others => '0');
                               mode_clear_n <= '0';
                               fsm_id <= conv_std_logic_vector(clear_id, 5);
      when IDLE_STATE    =>    clear_n_i <= '0';
                               pi_clear_n <= '0';
                               f_clear_n <= '0' or fifo_clr_disable;
                               po_clear_n <= '0';
                               cpu_wrote_reset_n <= (others => '0');
                               mode_we_n <= '0';
                               fsm_id <= conv_std_logic_vector(idle_id, 5);
      -- TRIGGER READOUT -------------------------------------------------------------------------
      when IDLE_TR_STATE   =>  fsm_id <= conv_std_logic_vector(idle_tr_id, 5);
      when SEND_TRI0L_STATE => src := 0;  -- source is local io
                               sel_num <= conv_std_logic_vector(src, 2);
                               sel_f2p <= lio;
                               if (cpu_wrote_n(src) = '0' and lio_tr_es(src) = '0') then
                                 po_we_n_i <=  '0';
                               end if;
                               f_push_req_n <= pi_we_n;
                               fsm_id <= conv_std_logic_vector(send_tri0l_id, 5);
      when SEND_TRI0H_STATE => src := 0;  -- source is local io
                               sel_num <= conv_std_logic_vector(src, 2);
                               sel_f2p <= lio;
                               lio_bus_dout_low(src) <= '0';
                               po_we_n_i <=  '0';
                               f_push_req_n <= pi_we_n;
                               fsm_id <= conv_std_logic_vector(send_tri0h_id, 5);
      when SEND_TRI1L_STATE => src := 1;  -- source is local io
                               sel_num <= conv_std_logic_vector(src, 2);
                               sel_f2p <= lio;
                               if (cpu_wrote_n(src) = '0' and lio_tr_es(src) = '0') then
                                 po_we_n_i <=  '0';
                               end if;
                               f_push_req_n <= pi_we_n;
                               fsm_id <= conv_std_logic_vector(send_tri1l_id, 5);
      when SEND_TRI1H_STATE => src := 1;  -- source is local io
                               sel_num <= conv_std_logic_vector(src, 2);
                               sel_f2p <= lio;
                               lio_bus_dout_low(src) <= '0';
                               po_we_n_i <=  '0';
                               f_push_req_n <= pi_we_n;
                               fsm_id <= conv_std_logic_vector(send_tri1h_id, 5);
      when SEND_TRI2L_STATE => src := 2;  -- source is local io
                               sel_num <= conv_std_logic_vector(src, 2);
                               sel_f2p <= lio;
                               if (cpu_wrote_n(src) = '0' and lio_tr_es(src) = '0') then
                                 po_we_n_i <=  '0';
                               end if;
                               f_push_req_n <= pi_we_n;
                               fsm_id <= conv_std_logic_vector(send_tri2l_id, 5);
      when SEND_TRI2H_STATE => src := 2;  -- source is local io
                               sel_num <= conv_std_logic_vector(src, 2);
                               sel_f2p <= lio;
                               lio_bus_dout_low(src) <= '0';
                               po_we_n_i <=  '0';
                               f_push_req_n <= pi_we_n;
                               fsm_id <= conv_std_logic_vector(send_tri2h_id, 5);
      when SEND_TRI3L_STATE => src := 3;  -- source is local io
                               sel_num <= conv_std_logic_vector(src, 2);
                               sel_f2p <= lio;
                               if (cpu_wrote_n(src) = '0' and lio_tr_es(src) = '0') then
                                 po_we_n_i <=  '0';
                               end if;
                               f_push_req_n <= pi_we_n;
                               fsm_id <= conv_std_logic_vector(send_tri3l_id, 5);
      when SEND_TRI3H_STATE => src := 3;  -- source is local io
                               sel_num <= conv_std_logic_vector(src, 2);
                               sel_f2p <= lio;
                               lio_bus_dout_low(src) <= '0';
                               po_we_n_i <=  '0';
                               f_push_req_n <= pi_we_n;
                               fsm_id <= conv_std_logic_vector(send_tri3h_id, 5);
      when SEND_TRP0_STATE =>  src := 0;  -- source is port
                               sel_num <= conv_std_logic_vector(src, 2);
                               f_push_req_n <= pi_we_n;
                               f_pop_req_n_i(src) <= '0';
                               if f_error(src) = '0' and f_tr_es(src) = '0' then
                                 po_we_n_i <= '0';
                               end if;
                               fsm_id <= conv_std_logic_vector(send_trp0_id, 5);
      when SEND_TRP1_STATE =>  src := 1;  -- source is port NEWCODE
                               sel_num <= conv_std_logic_vector(src, 2);
                               f_push_req_n <= pi_we_n;
                               f_pop_req_n_i(src) <= '0';
                               if f_error(src) = '0' and f_tr_es(src) = '0' then
                                 po_we_n_i <= '0';
                               end if;
                               fsm_id <= conv_std_logic_vector(send_trp1_id, 5);
      when SEND_TRP2_STATE =>  src := 2;  -- source is port
                               sel_num <= conv_std_logic_vector(src, 2);
                               f_push_req_n <= pi_we_n;
                               f_pop_req_n_i(src) <= '0';
                               if f_error(src) = '0' and f_tr_es(src) = '0' then
                                 po_we_n_i <= '0';
                               end if;
                               fsm_id <= conv_std_logic_vector(send_trp2_id, 5);
      when SEND_TRP3_STATE =>  src := 3;  -- source is port
                               sel_num <= conv_std_logic_vector(src, 2);
                               f_push_req_n <= pi_we_n;
                               f_pop_req_n_i(src) <= '0';
                               if f_error(src) = '0' and f_tr_es(src) = '0' then
                                 po_we_n_i <= '0';
                               end if;
                               fsm_id <= conv_std_logic_vector(send_trp3_id, 5);
      when SEND_TRES_STATE0 => sel_e2p <= "001";
                               po_we_n_i <= '0';
      when SEND_TRES_STATE1 => sel_e2p <= "001";
                               po_we_n_i <= '0';
      when SEND_TRES_STATE2 => sel_e2p <= "001";
                               po_we_n_i <= '0';
      when SEND_TRES_STATE3 => sel_e2p <= "001";
                               po_we_n_i <= '0';
      -- RAW DATA READOUT ------------------------------------------------------------------------
      when IDLE_RR_STATE   =>  sel_p2f <= '0';
                               f_push_req_n <= push_din;
                               lio_bus_dout_low <= dout_low;
                               fsm_id <= conv_std_logic_vector(idle_rr_id, 5);
      when SEND_RRF0_STATE =>  src := 0;  -- source is fifo
                               sel_p2f <= '0';
                               f_push_req_n <= push_din;
                               lio_bus_dout_low <= dout_low;
                               sel_num <= conv_std_logic_vector(src, 2);
                               f_pop_req_n_i(src) <= '0';
                               if (f_error(src) = '0' and f_rr_es(src) = '0') then po_we_n_i <= '0'; end if;
                               fsm_id <= conv_std_logic_vector(send_rrf0_id, 5);
      when SEND_RRF1_STATE =>  src := 1;  -- source is fifo
                               sel_p2f <= '0';
                               f_push_req_n <= push_din;
                               lio_bus_dout_low <= dout_low;
                               sel_num <= conv_std_logic_vector(src, 2);
                               f_pop_req_n_i(src) <= '0';
                               if (f_error(src) = '0' and f_rr_es(src) = '0') then po_we_n_i <= '0'; end if;
                               fsm_id <= conv_std_logic_vector(send_rrf1_id, 5);
      when SEND_RRF2_STATE =>  src := 2;  -- source is fifo
                               sel_p2f <= '0';
                               f_push_req_n <= push_din;
                               lio_bus_dout_low <= dout_low;
                               sel_num <= conv_std_logic_vector(src, 2);
                               f_pop_req_n_i(src) <= '0';
                               if (f_error(src) = '0' and f_rr_es(src) = '0') then po_we_n_i <= '0'; end if;
                               fsm_id <= conv_std_logic_vector(send_rrf2_id, 5);
      when SEND_RRF3_STATE =>  src := 3;  -- source is fifo
                               sel_p2f <= '0';
                               f_push_req_n <= push_din;
                               lio_bus_dout_low <= dout_low;
                               sel_num <= conv_std_logic_vector(src, 2);
                               f_pop_req_n_i(src) <= '0';
                               if (f_error(src) = '0' and f_rr_es(src) = '0') then po_we_n_i <= '0'; end if;
                               fsm_id <= conv_std_logic_vector(send_rrf3_id, 5);
      when SEND_RRP0_STATE =>  src := 0;  -- source is input port
                               sel_p2f <= '0';
                               f_push_req_n <= push_din;
                               lio_bus_dout_low <= dout_low;
                               sel_f2p <= pi;
                               sel_num <= conv_std_logic_vector(src, 2);
                               po_we_n_i <= pi_we_n(src);
                               if pi_rr_es(src) = '1' then po_we_n_i <= '1'; end if;
                               pi_ctrl_in <= "0001";
                               fsm_id <= conv_std_logic_vector(send_rrp0_id, 5);
      when SEND_RRP1_STATE =>  src := 1;  -- source is input port
                               sel_p2f <= '0';
                               f_push_req_n <= push_din;
                               lio_bus_dout_low <= dout_low;
                               sel_f2p <= pi;
                               sel_num <= conv_std_logic_vector(src, 2);
                               po_we_n_i <= pi_we_n(src);
                               if pi_rr_es(src) = '1' then po_we_n_i <= '1'; end if;
                               pi_ctrl_in <= "0010";
                               fsm_id <= conv_std_logic_vector(send_rrp1_id, 5);
      when SEND_RRP2_STATE =>  src := 2;  -- source is input port
                               sel_p2f <= '0';
                               f_push_req_n <= push_din;
                               lio_bus_dout_low <= dout_low;
                               sel_f2p <= pi;
                               sel_num <= conv_std_logic_vector(src, 2);
                               po_we_n_i <= pi_we_n(src);
                               if pi_rr_es(src) = '1' then po_we_n_i <= '1'; end if;
                               pi_ctrl_in <= "0100";
                               fsm_id <= conv_std_logic_vector(send_rrp2_id, 5);
      when SEND_RRP3_STATE =>  src := 3;  -- source is input port
                               sel_p2f <= '0';
                               f_push_req_n <= push_din;
                               lio_bus_dout_low <= dout_low;
                               sel_f2p <= pi;
                               sel_num <= conv_std_logic_vector(src, 2);
                               po_we_n_i <= pi_we_n(src);
                               if pi_rr_es(src) = '1' then po_we_n_i <= '1'; end if;
                               pi_ctrl_in <= "1000";
                               fsm_id <= conv_std_logic_vector(send_rrp3_id, 5);
      when SEND_RRES_STATE0 => sel_e2p <= "010";
                               po_we_n_i <= '0';
      when SEND_RRES_STATE1 => sel_e2p <= "010";
                               po_we_n_i <= '0';
      when SEND_RRES_STATE2 => sel_e2p <= "010";
                               po_we_n_i <= '0';
      when SEND_RRES_STATE3 => sel_e2p <= "010";
                               po_we_n_i <= '0';

      when FINISHED_STATE   => finished_n <= '0';
                               clear_n_i <= '0';
                               pi_clear_n <= '0';
                               f_clear_n <= '0' or fifo_clr_disable;
                               po_clear_n <= '0';
                               cpu_wrote_reset_n <= (others => '0');
                               mode_clear_n <= '0';
                               fsm_id <= conv_std_logic_vector(finished_id, 5);

      -- NETWORK MODE ----------------------------------------------------------------------------
      when IDLE_NET_STATE  =>  sel_p2f <= '1';
                               sel_f2p <= lio;
                               f_push_req_n <= pi_we_n;
                               sel_num <= order_tr_start(1 downto 0);
                               lio_bus_dout_low <= dout_low;
                               po_we_n_i <= push_din(conv_integer(order_tr_start(1 downto 0)));
                               net_mode <= '1';
                               if lio_ctrl_en_n(0) = '0' then pi_ctrl_in(0) <= lio_ctrl(0);
                                                         else pi_ctrl_in(0) <= f_high(0);
                               end if;
                               if lio_ctrl_en_n(1) = '0' then pi_ctrl_in(1) <= lio_ctrl(1);
                                                         else pi_ctrl_in(1) <= f_high(1);
                               end if;
                               if lio_ctrl_en_n(2) = '0' then pi_ctrl_in(2) <= lio_ctrl(2);
                                                         else pi_ctrl_in(2) <= f_high(2);
                               end if;
                               if lio_ctrl_en_n(3) = '0' then pi_ctrl_in(3) <= lio_ctrl(3);
                                                         else pi_ctrl_in(3) <= f_high(3);
                               end if;
                               fsm_id <= conv_std_logic_vector(idle_net_id, 5);

      -- TEST PATTERN MODE -------------------------------------------------------------------------
      when SEND_TPL_STATE  =>  sel_e2p <= "100";
                               po_we_n_i <= '0';
                               fsm_id <= conv_std_logic_vector(send_tpl_id, 5);
      when SEND_TPH_STATE  =>  sel_e2p <= "101";
                               po_we_n_i <= '0';
                               fsm_id <= conv_std_logic_vector(send_tph_id, 5);
      -- OTHERWISE ---------------------------------------------------------------------------------
      when others          =>  null;
    end case;
  end process;

  -- cpu wrote? ------------------------------------------------------------------------------------
  U_cpu_wrote : for i in 3 downto 0 generate
    U_cpu_wrote_reg : process(clk, cpu_wrote_reset_n, lio_din_w_n)
    begin
      if rising_edge(clk) then
        if(cpu_wrote_reset_n(i) = '0') then
          cpu_wrote_n(i) <= '1';
        elsif (lio_din_w_n(i) = '0') then
          cpu_wrote_n(i) <= '0';
        end if;
      end if;
    end process;
  end generate;

  -- din_w_delay -----------------------------------------------------------------------------------
  U_din_w_del_for : for i in 3 downto 0 generate
    U_din_w_del : process(clk, reset_n, clear_n_i, lio_din_w_n, din_cnt)
    begin
      if reset_n = '0' then
        din_cnt(i) <= "00";
      elsif rising_edge(clk) then
        if din_cnt(i) /= "00" then din_cnt(i) <= din_cnt(i) + 1; end if;
        if lio_din_w_n(i) = '0' and din_cnt(i) /= "01" then din_cnt(i) <= "01"; end if;
        if clear_n_i = '0' then din_cnt(i)<= "00"; end if;
      end if;
    end process;
    with din_cnt(i) select
      push_din(i) <= '1' when "00",
                     '0' when "01",
                     '0' when "10",
                     '1' when others;
    with din_cnt(i) select
      dout_low(i) <= '0' when "10",
                     '1' when others;
  end generate;

  -- net FSM ---------------------------------------------------------------------------------------

  U_net_fsm_for: for i in 3 downto 0 generate
                                                                                        -- memory --
    U_net_fsm_mem : process(clk, reset_n, clear_n_i)
    begin
      if reset_n = '0' then
        net_fsm_state(i) <= RESET_STATE;
      elsif rising_edge(clk) then
        net_fsm_state(i) <= net_fsm_next_state(i);
        if clear_n_i = '0' then net_fsm_state(i) <= CLEAR_STATE; end if;
      end if;
    end process;

                                                                                        -- states --
    U_net_fsm_state: process(clk, reset_n, net_fsm_state, net_mode, clear_n_i, f_error, lio_dout_r_n)
    begin
      case net_fsm_state(i) is
        when RESET_STATE      => net_fsm_next_state(i) <= CLEAR_STATE;
        when CLEAR_STATE      => if net_mode = '1' then net_fsm_next_state(i) <= WRITE_LOW_STATE;
                                                   else net_fsm_next_state(i) <= CLEAR_STATE;
                                 end if;
        when WRITE_LOW_STATE  => if f_error(i) = '1' then net_fsm_next_state(i) <= WRITE_LOW_STATE;
                                                     else net_fsm_next_state(i) <= WRITE_HIGH_STATE;
                                 end if;
        when WRITE_HIGH_STATE => if f_error(i) = '1' then net_fsm_next_state(i) <= WRITE_HIGH_STATE;
                                                     else net_fsm_next_state(i) <= WAIT_READ_STATE;
                                 end if;
        when WAIT_READ_STATE  => if lio_dout_r_n(i) = '0' then net_fsm_next_state(i) <= WRITE_LOW_STATE;
                                                          else net_fsm_next_state(i) <= WAIT_READ_STATE;
                                 end if;
        when others           => net_fsm_next_state(i) <= CLEAR_STATE;
      end case;
    end process;

                                                                                      -- output --
    U_net_fsm_out: process(net_fsm_state, f_error)
    begin
      -- default values
      net_pop_req_n(i)      <= '1';
      lio_bus_din_we_n_i(i) <= "11";
      lio_data_valid(i)     <= '0';

      case net_fsm_state(i) is
        when RESET_STATE      => null;
        when CLEAR_STATE      => null;
        when WRITE_LOW_STATE  => net_pop_req_n(i) <= '0';
                                 if f_error(i) = '0' then lio_bus_din_we_n_i(i) <= "10"; end if;
        when WRITE_HIGH_STATE => net_pop_req_n(i) <= '0';
                                 if f_error(i) = '0' then
                                   lio_bus_din_we_n_i(i) <= "01";
                                   lio_data_valid(i) <= '1';
                                 end if;
        when WAIT_READ_STATE  => lio_data_valid(i) <= '1';
        when others           => null;
      end case;
    end process;

  end generate;

  -- mode_fsm --------------------------------------------------------------------------------------
  U_mode: process(clk, reset_n, mode_clear_n, mode_we_n, mode)
  begin
    if reset_n='0' then
      mode_fsm_i <= (others => '0');
    elsif rising_edge(clk) then
      mode_fsm_i <= mode_fsm_i;
      if mode_we_n='0' then mode_fsm_i <= mode; end if;
      if mode_clear_n='0' then mode_fsm_i <= (others => '0'); end if;
    end if;
  end process;

end v;

----------------------------------------------------------------------------------------------------
-- CONFIGURATION
----------------------------------------------------------------------------------------------------
-- pragma translate_off
configuration ni_fsm_CFG of ni_fsm is
  for v
  end for; -- v
end ni_fsm_CFG;
-- pragma translate_on
