-------------------------------------------------------------------------------
-- Title      : ni_global
-- Project    : Network Interface (NI) for the ALICE TRD TRAP3
-------------------------------------------------------------------------------
-- File       : ni_global.vhd
-- Author     : Rolf Schneider  <rolf.schneider@kip.uni-heidelberg.de>
-- Company    : University Heidelberg - KIP
-- Last update: 2003-12-11
-- Platform   : Synopsys - v2003.03
-------------------------------------------------------------------------------
-- Description: global package for address spaces and control reset values
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author  Description
-- 2003/02/27  1.0      schneide	Created
-- 2003/12/11  1.1      schneide	Fsmstate added to global status
--                                      Fifo entries added to local status
-- 2003/12/11  1.2      schneide	Added ctrl bit for fifo_clear_disable
-------------------------------------------------------------------------------


library IEEE;
use IEEE.std_logic_1164.all;

----------------------------------------------------------------------------------------------------
-- PACKAGE
----------------------------------------------------------------------------------------------------
package ni_global is

  -- FUNCTION DECLARATIONS -------------------------------------------------------------------------
  function g_ctrl_w_sum return integer;
  function g_ctrl_offset(addr:integer) return integer;
  function g_stat_w_sum return integer;
  function g_stat_offset(addr:integer) return integer;
  function l_stat_w_sum return integer;
  function l_stat_offset(addr:integer) return integer;

  -- CONSTANT BUS WIDTHS ---------------------------------------------------------------------------
  constant io_address_bus_width         : integer := 16;  -- width of global/local address bus
  constant io_address_space_width       : integer := 8;   -- offset for data/ctrl/status
  constant io_address_offset            : integer := 6;   -- linear addressable space
  constant g_ctrl_regs                  : integer := 13;  -- number of global control registers
  constant g_stat_regs                  : integer := 3;   -- number of global status registers
  constant l_stat_regs                  : integer := 3;   -- number of local status registers
  constant io_data_bus_width            : integer := 32;  -- width of global/local data bus

  constant excl_depth                   : integer := 4;   -- number of bits for excl positions
  constant delay_depth                  : integer := 3;   -- number of delay stages of output port

  --------------------------------------------------------------------------------------------------
  -- IO address:
  --   FEDCBA9876543210
  --   ****************     network interface base address              (io_address_bus_width)
  --           ********     network interface linear address space      (io_address_space_width)
  --           **------     offset for control and status registers     (io_address_offset)
  --                  *     address space for control registers         (ctrl_address_space_width)
  --                  *     address space for control registers         (status_address_space_width)
  --------------------------------------------------------------------------------------------------

  -- SIGNAL TYPES ----------------------------------------------------------------------------------

  subtype data_type        is std_logic_vector(io_data_bus_width-1 downto 0);
  subtype io_address_space is std_logic_vector(io_address_space_width-1 downto 0);
  subtype io_address_cs    is std_logic_vector(io_address_bus_width-io_address_space_width-1 downto 0);

  type g_ctrl_array is array(g_ctrl_regs-1 downto 0) of data_type;
  type g_ctrl_w_array is array(g_ctrl_regs-1 downto 0) of integer range 0 to io_data_bus_width;
  type g_stat_array is array(g_stat_regs-1 downto 0) of data_type;
  type g_stat_w_array is array(g_stat_regs-1 downto 0) of integer range 0 to io_data_bus_width;
  type l_stat_array is array(l_stat_regs-1 downto 0) of data_type;
  type l_stat_w_array is array(l_stat_regs-1 downto 0) of integer range 0 to io_data_bus_width;

  -- for FSM
  subtype bus_we_h_type is std_logic_vector(1 downto 0);
  type    bus_we_h_array is array (3 downto 0) of bus_we_h_type;

  -- MAPPED CONTROL AND STATUS REGISTERS -----------------------------------------------------------
  
  constant io_addr_cs      : io_address_cs    := "00000000"; -- MSByte (local IO of NI is addressed)
  constant io_addr_g_cs    : io_address_cs    := "00001101"; -- MSByte (global IO of NI is addressed)
  
  constant io_addr_datain  : io_address_space := "00000000"; -- LSByte (data from CPU to NI)
  constant io_addr_dataout : io_address_space := "00000001"; --        (data from NI to CPU)
  constant ctrl_offset     : io_address_space := "01000000"; -- base address for control write
  constant status_offset   : io_address_space := "11000000"; -- base address for status read

  -- GLOBAL I/O CONTROL ----------------------------------------------------------------------------
  constant g_ctrl_w: g_ctrl_w_array := (
    32,  -- C
    11,  -- B
    11,  -- A
    11,  -- 9
    11,  -- 8
    16,  -- 7
    32,  -- 6
    32,  -- 5
    18,  -- 4
    18,  -- 3
    16,  -- 2
    30,  -- 1
     6   -- 0
  );

  constant g_ctrl_reset: g_ctrl_array:= (
    "11111111111111111111111111111111",  -- C: word cut
    "---------------------10001001100",  -- B: local control
    "---------------------10001001100",  -- A: local control
    "---------------------10001001100",  -- 9: local control
    "---------------------10001001100",  -- 8: local control
    "----------------0110000000100000",  -- 7: low/high bound
    "00000000000000001111111111111111",  -- 6: test pattern
    "00000000000000000000000000000000",  -- 5: end signature rr/tr
    "--------------111111111111111100",  -- 4: readout order rr
    "--------------111111111111111100",  -- 3: readout order tr
    "----------------1010001001000000",  -- 2: excl & ctrl/strobe dly
    "--100100100100100100100100100100",  -- 1: output delays
    "--------------------------001000"   -- 0: clear & mode
  );

  -- GLOBAL I/O STATUS -----------------------------------------------------------------------------
  constant g_stat_w: g_stat_w_array := (
     5,  -- 2
     4,  -- 1
     1   -- 0
  );

  -- LOCAL I/O STATUS ------------------------------------------------------------------------------
  constant l_stat_w: l_stat_w_array := (
     8,  -- 2
    24,  -- 1
    11   -- 0
  );

end ni_global;

----------------------------------------------------------------------------------------------------
-- PACKAGE BODY
----------------------------------------------------------------------------------------------------
package body ni_global is

  -- returns the total number of control bits of global I/O
  function g_ctrl_w_sum return integer is
    variable sum : integer;
  begin
    sum := 0;
    for i in g_ctrl_regs-1 downto 0 loop
      sum := sum + g_ctrl_w(i);
    end loop;
    return sum;
  end g_ctrl_w_sum;

  -- returns the lower bit position in the lobal ctrl array depending of the control address
  function g_ctrl_offset(addr:integer) return integer is
    variable sum : integer;
  begin
    sum := 0;
    if addr > 0 then
      for i in 0 to addr-1 loop
        sum := sum + g_ctrl_w(i);
      end loop;
    end if;
    return sum;
  end g_ctrl_offset;

  -- returns the total number of status bits of global I/O
  function g_stat_w_sum return integer is
    variable sum : integer;
  begin
    sum := 0;
    for i in g_stat_regs-1 downto 0 loop
      sum := sum + g_stat_w(i);
    end loop;
    return sum;
  end g_stat_w_sum;

  -- returns the lower bit position in the global status array depending of the status address
  function g_stat_offset(addr:integer) return integer is
    variable sum : integer;
  begin
    sum := 0;
    if addr > 0 then
      for i in 0 to addr-1 loop
        sum := sum + g_stat_w(i);
      end loop;
    end if;
    return sum;
  end g_stat_offset;

  -- returns the total number of status bits of local I/O
  function l_stat_w_sum return integer is
    variable sum : integer;
  begin
    sum := 0;
    for i in l_stat_regs-1 downto 0 loop
      sum := sum + l_stat_w(i);
    end loop;
    return sum;
  end l_stat_w_sum;

  -- returns the lower bit position in the local status array depending of the status address
  function l_stat_offset(addr:integer) return integer is
    variable sum : integer;
  begin
    sum := 0;
    if addr > 0 then
      for i in 0 to addr-1 loop
        sum := sum + l_stat_w(i);
      end loop;
    end if;
    return sum;
  end l_stat_offset;

end ni_global;
