-------------------------------------------------------------------------------
-- Title      : ni_io_interface
-- Project    : Network Interface (NI) for the ALICE TRD TRAP3
-------------------------------------------------------------------------------
-- File       : ni_io_interface.vhd
-- Author     : Rolf Schneider  <rolf.schneider@kip.uni-heidelberg.de>
-- Company    : University Heidelberg - KIP
-- Last update: 2003-12-11
-- Platform   : Synopsys - v2003.03
-------------------------------------------------------------------------------
-- Description: local I/O interfaces connected to local I/O bus
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author  Description
-- 2003/03/11  1.0      schneide	Created
-- 2003/03/19  1.1      schneide	all control register removed
-- 2003/04/04  1.2      schneide	registers to input added
-------------------------------------------------------------------------------


library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;
use WORK.ni_global.all;

----------------------------------------------------------------------------------------------------
-- ENTITY
----------------------------------------------------------------------------------------------------
entity ni_io_interface is
  generic (
    width        : integer := 32);                      -- data bus width of CPU bus
  port (
    clk          : in  std_logic;                       -- internal clock
    reset_n      : in  std_logic;                       -- asynchronous reset
    -- bus to/from CPU
    cpu_we       : in  std_logic;                       -- data valid signal from CPU
    cpu_address  : in  std_logic_vector(io_address_bus_width-1 downto 0);  -- local IO address
    cpu_din      : in  std_logic_vector(width-1 downto 0);                 -- local IO data from CPU
    cpu_dout     : out std_logic_vector(width-1 downto 0);                 -- local IO data to CPU
    -- internal bus
    bus_din_we_n : in  std_logic_vector(2-1 downto 0);  -- write select for high/low-word
    bus_dout_low : in  std_logic;                       -- read select for high/low-word
    bus_in       : in  std_logic_vector((width/2)-1 downto 0);             -- data bus from port
    bus_out      : out std_logic_vector((width/2)-1 downto 0);             -- data bus to port
    -- control and status ports
    status       : in  std_logic_vector(l_stat_w_sum-1 downto 0);          -- status bits from NI
    dout_r_n     : out std_logic;                       -- read from dout register
    din_w_n      : out std_logic;                       -- written to din register
    tr_es_word   : in  std_logic_vector((width/2)-1 downto 0);  -- end sig word (trigger readout)
    tr_es        : out std_logic;                       -- end sig detected (trigger readout)
    rr_es_word   : in  std_logic_vector((width/2)-1 downto 0);  -- end sig word (raw data readout)
    rr_es        : out std_logic);                      -- end sig detected (raw data readout)
end ni_io_interface;

----------------------------------------------------------------------------------------------------
-- ARCHITECTURE
----------------------------------------------------------------------------------------------------
architecture v of ni_io_interface is

  -- SIGNALS ---------------------------------------------------------------------------------------
  signal cpu_we_reg      : std_logic;
  signal cpu_address_reg : std_logic_vector(io_address_bus_width-1 downto 0);
  signal cpu_din_reg     : std_logic_vector(width-1 downto 0);
  
  signal cs_n            : std_logic;
  signal address         : std_logic_vector(io_address_space_width-1 downto 0);
  signal offset          : std_logic_vector(io_address_space_width-1 downto 0);
  signal reg_address     : std_logic_vector(io_address_offset-1 downto 0);
  signal din_reg_out     : std_logic_vector(width-1 downto 0);
  signal dout_reg_out    : std_logic_vector(width-1 downto 0);
  signal din_we_n        : std_logic;

  -- signals for status/control registers
  signal status_i       : l_stat_array;
  signal status_reg_out : l_stat_array;

  -- reset values
  signal io_null_reset_A: STD_LOGIC_VECTOR(width-1 downto 0);

  -- COMPONENTS ------------------------------------------------------------------------------------
  component ni_register
    generic (
      width : integer);
    port (
      clk     : in  STD_LOGIC;
      reset_n : in  STD_LOGIC;
      A       : in  STD_LOGIC_VECTOR(width-1 downto 0);
      O       : out STD_LOGIC_VECTOR(width-1 downto 0));
  end component;

  component ni_register_we
    generic (
      width : integer);
    port (
      clk     : in  STD_LOGIC;
      reset_n : in  STD_LOGIC;
      reset_A : in  STD_LOGIC_VECTOR(width-1 downto 0);
      we_n    : in  STD_LOGIC;
      A       : in  STD_LOGIC_VECTOR(width-1 downto 0);
      O       : out STD_LOGIC_VECTOR(width-1 downto 0));
  end component;

  -- STRUCTURE -------------------------------------------------------------------------------------
begin

  U_stat_ntt : for i in l_stat_regs-1 downto 0 generate
    status_i(i)(l_stat_w(i)-1 downto 0) <= status(l_stat_offset(i)+l_stat_w(i)-1 downto l_stat_offset(i));
  end generate;

  U_cpu_in: process(clk, reset_n)
  begin
    if reset_n = '0' then
      cpu_we_reg <= '0';
      cpu_address_reg <= (others => '0');
      cpu_din_reg <= (others => '0');
    elsif rising_edge(clk) then
      cpu_we_reg <= cpu_we;
      cpu_address_reg <= cpu_address;
      cpu_din_reg <= cpu_din;
    end if;
  end process;

  io_null_reset_A <= (others => '0');

  -- chip select and mapped addresses
  address <= cpu_address_reg(io_address_space_width-1 downto 0);
  offset(io_address_space_width-1 downto io_address_offset) <=
    cpu_address_reg(io_address_space_width-1 downto io_address_offset);
  offset(io_address_offset-1 downto 0) <=  (others => '0');
  cs_n <= '0' when cpu_address_reg(io_address_bus_width-1 downto io_address_space_width) = io_addr_cs
              else '1';
  reg_address <= address(io_address_offset-1 downto 0);

  -- data_in (data from CPU to BUS)
  din_we_n <= '0' when (cs_n = '0' and cpu_we_reg = '1' and address = io_addr_datain) else '1';
  U_din_reg : ni_register_we
    generic map (
      width   => width)
    port map (
      clk     => clk,
      reset_n => reset_n,
      reset_A => io_null_reset_A,
      we_n    => din_we_n,
      A       => cpu_din_reg,
      O       => din_reg_out);
  din_w_n <= din_we_n;
  bus_out <= din_reg_out((width/2)-1 downto 0) when bus_dout_low = '1'
             else din_reg_out(width-1 downto (width/2));
  rr_es   <= '1' when din_reg_out((width/2)-1 downto 0) = rr_es_word else '0';
  tr_es   <= '1' when din_reg_out((width/2)-1 downto 0) = tr_es_word else '0';

  -- data_out (data from BUS to CPU)
  U_dout_reg_low : ni_register_we
    generic map (
      width   => width/2)
    port map (
      clk     => clk,
      reset_n => reset_n,
      reset_A => io_null_reset_A((width/2)-1 downto 0),
      we_n    => bus_din_we_n(0),
      A       => bus_in,
      O       => dout_reg_out((width/2)-1 downto 0));
  U_dout_reg_high : ni_register_we
    generic map (
      width   => width/2)
    port map (
      clk     => clk,
      reset_n => reset_n,
      reset_A => io_null_reset_A((width/2)-1 downto 0),
      we_n    => bus_din_we_n(1),
      A       => bus_in,
      O       => dout_reg_out(width-1 downto (width/2)));

  -- status registers
  U_stat : for i in l_stat_regs-1 downto 0 generate
    U_status_reg : ni_register
      generic map (
        width   => l_stat_w(i))
      port map (
        clk     => clk,
        reset_n => reset_n,
        A       => status_i(i)(l_stat_w(i)-1 downto 0),
        O       => status_reg_out(i)(l_stat_w(i)-1 downto 0));
  end generate;

  -- output address select
  U_status_out: process(reg_address, address, cs_n, offset, status_reg_out, dout_reg_out)
    variable addr : integer;
  begin
    addr := conv_integer(reg_address);
    cpu_dout <= (others => '0');
    dout_r_n <= '1';
    if cs_n = '0' then
      if offset = status_offset then
        for i in l_stat_regs-1 downto 0 loop
          if addr = i then
            cpu_dout(l_stat_w(i)-1 downto 0) <= status_reg_out(i)(l_stat_w(i)-1 downto 0);
          end if;
        end loop;
      end if;
      if address = io_addr_dataout then
        cpu_dout <= dout_reg_out;
        dout_r_n <= '0';
      end if;
    end if;
  end process;

end v;

----------------------------------------------------------------------------------------------------
-- CONFIGURATION
----------------------------------------------------------------------------------------------------
-- pragma translate_off
configuration ni_io_interface_CFG of ni_io_interface is
  for v
  end for; -- v
end ni_io_interface_CFG;
-- pragma translate_on
