-------------------------------------------------------------------------------
-- Title      : ni_io_interface_g
-- Project    : Network Interface (NI) for the ALICE TRD TRAP3
-------------------------------------------------------------------------------
-- File       : ni_io_interface_g.vhd
-- Author     : Rolf Schneider  <rolf.schneider@kip.uni-heidelberg.de>
-- Company    : University Heidelberg - KIP
-- Last update: 2004-01-20
-- Platform   : Synopsys - v2003.03
-------------------------------------------------------------------------------
-- Description: global I/O interfaces connected to global I/O bus
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author  Description
-- 2003/03/19  1.0      schneide	Created
-- 2004/01/20  1.1      schneide	Added NI clk for status registers
-------------------------------------------------------------------------------


library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;
use WORK.ni_global.all;

----------------------------------------------------------------------------------------------------
-- ENTITY
----------------------------------------------------------------------------------------------------
entity ni_io_interface_g is
  generic (
    width       : integer := 32);                       -- data bus width of CPU bus
  port (
    clk         : in  std_logic;                        -- internal clock (NI clock)
    gclk        : in  std_logic;                        -- internal clock (global bus clock)
    reset_n     : in  std_logic;                        -- asynchronous reset
    -- Bus to/from CPU
    cpu_we      : in  std_logic;                        -- write enable sig from local IO bus
    cpu_address : in  std_logic_vector(io_address_bus_width-1 downto 0);  -- local IO address
    cpu_din     : in  std_logic_vector(width-1 downto 0);                 -- local IO data from CPU
    cpu_dout    : out std_logic_vector(width-1 downto 0);                 -- local IO data to CPU
    -- Control and status ports
    status      : in  std_logic_vector(g_stat_w_sum-1 downto 0);          -- status bits from NI
    ctrl        : out std_logic_vector(g_ctrl_w_sum-1 downto 0));         -- control bits to NI
end ni_io_interface_g;

----------------------------------------------------------------------------------------------------
-- ARCHITECTURE
----------------------------------------------------------------------------------------------------
architecture v of ni_io_interface_g is

  -- SIGNALS ---------------------------------------------------------------------------------------
  type ctrl_type is array(g_ctrl_regs-1 downto 0) of data_type;

  signal cs_n           : std_logic;
  signal address        : std_logic_vector(io_address_space_width-1 downto 0);
  signal offset         : std_logic_vector(io_address_space_width-1 downto 0);
  signal reg_address    : std_logic_vector(io_address_offset-1 downto 0);

  -- signals for control registers
  signal ctrl_i         : g_ctrl_array;
  signal ctrl_we_n      : std_logic_vector(g_ctrl_regs-1 downto 0);
  signal status_i       : g_stat_array;
  signal status_reg_out : g_stat_array;

  -- reset values
  signal ctrl_reset_A : ctrl_type;

  -- COMPONENTS ------------------------------------------------------------------------------------
  component ni_register
    generic (
      width : integer);
    port (
      clk     : in  STD_LOGIC;
      reset_n : in  STD_LOGIC;
      A       : in  STD_LOGIC_VECTOR(width-1 downto 0);
      O       : out STD_LOGIC_VECTOR(width-1 downto 0));
  end component;

  component ni_register_we
    generic (
      width : integer);
    port (
      clk     : in  STD_LOGIC;
      reset_n : in  STD_LOGIC;
      reset_A : in  STD_LOGIC_VECTOR(width-1 downto 0);
      we_n    : in  STD_LOGIC;
      A       : in  STD_LOGIC_VECTOR(width-1 downto 0);
      O       : out STD_LOGIC_VECTOR(width-1 downto 0));
  end component;

  -- STRUCTURE -------------------------------------------------------------------------------------
begin

  U_ctrl_ntt : for i in g_ctrl_regs-1 downto 0 generate
    ctrl(g_ctrl_offset(i)+g_ctrl_w(i)-1 downto g_ctrl_offset(i)) <= ctrl_i(i)(g_ctrl_w(i)-1 downto 0);
  end generate;

  U_stat_ntt : for i in g_stat_regs-1 downto 0 generate
    status_i(i)(g_stat_w(i)-1 downto 0) <= status(g_stat_offset(i)+g_stat_w(i)-1 downto g_stat_offset(i));
  end generate;

  -- chip select and mapped addresses
  address <= cpu_address(io_address_space_width-1 downto 0);
  offset(io_address_space_width-1 downto io_address_offset) <=
    cpu_address(io_address_space_width-1 downto io_address_offset);
  offset(io_address_offset-1 downto 0) <= (others => '0');
  cs_n <= '0' when cpu_address(io_address_bus_width-1 downto io_address_space_width) = io_addr_g_cs
              else '1';
  reg_address <= address(io_address_offset-1 downto 0);

  -- ctrl_we_n signals generation
  U_ctrl_we: process(reg_address, cs_n, cpu_we, offset)
  begin
    ctrl_we_n <= (others => '1');
    if(cs_n = '0' and cpu_we = '1' and offset = ctrl_offset) then
      ctrl_we_n(conv_integer(reg_address)) <= '0';
    end if;
  end process;

  -- ctrl registers
  U_ctrl : for i in g_ctrl_regs-1 downto 0 generate
    ctrl_reset_A(i) <= g_ctrl_reset(i);
    U_ctrl_reg : ni_register_we
      generic map (
        width   => g_ctrl_w(i))
      port map (
        clk     => gclk,
        reset_n => reset_n,
        reset_A => ctrl_reset_A(i)(g_ctrl_w(i)-1 downto 0),
        we_n    => ctrl_we_n(i),
        A       => cpu_din(g_ctrl_w(i)-1 downto 0),
        O       => ctrl_i(i)(g_ctrl_w(i)-1 downto 0));
  end generate;

  -- status registers
  U_stat : for i in g_stat_regs-1 downto 0 generate
    U_status_reg : ni_register
      generic map (
        width   => g_stat_w(i))
      port map (
        clk     => clk,
        reset_n => reset_n,
        A       => status_i(i)(g_stat_w(i)-1 downto 0),
        O       => status_reg_out(i)(g_stat_w(i)-1 downto 0));
  end generate;

  -- output address select, if not selected output zeros!!!
  U_status_out: process(reg_address, cs_n, offset, status_reg_out, ctrl_i)
    variable addr : integer;
  begin
    addr := conv_integer(reg_address);
    cpu_dout <= (others => '0');
    if cs_n = '0'  then
      if offset = status_offset then
        for i in g_stat_regs-1 downto 0 loop
          if addr = i then
            cpu_dout(g_stat_w(i)-1 downto 0) <= status_reg_out(i)(g_stat_w(i)-1 downto 0);
          end if;
        end loop;
      elsif offset = ctrl_offset then
        for i in g_ctrl_regs-1 downto 0 loop
          if addr = i then
            cpu_dout(g_ctrl_w(i)-1 downto 0) <= ctrl_i(i)(g_ctrl_w(i)-1 downto 0);
          end if;
        end loop;
      end if;
    end if;
  end process;

end v;

----------------------------------------------------------------------------------------------------
-- CONFIGURATION
----------------------------------------------------------------------------------------------------
-- pragma translate_off
configuration ni_io_interface_g_CFG of ni_io_interface_g is
  for v
  end for; -- v
end ni_io_interface_g_CFG;
-- pragma translate_on
