-------------------------------------------------------------------------------
-- Title      : ni_port_in
-- Project    : Network Interface (NI) for the ALICE TRD TRAP3
-------------------------------------------------------------------------------
-- File       : ni_port_in.vhd
-- Author     : Rolf Schneider  <rolf.schneider@kip.uni-heidelberg.de>
-- Company    : University Heidelberg - KIP
-- Last update: 2003-12-11
-- Platform   : Synopsys - v2003.03
-------------------------------------------------------------------------------
-- Description: input ports
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author  Description
-- 2003/03/06  1.0      schneide	Created
-------------------------------------------------------------------------------


library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

----------------------------------------------------------------------------------------------------
-- ENTITY
----------------------------------------------------------------------------------------------------

entity ni_port_in is
  generic (
    width        : integer := 16;                       -- internal bus width
    excl_depth   : integer := 4);                       -- number of bits for exclude positions
  port (
    clk          : in  std_logic;                       -- internal clock
    reset_n      : in  std_logic;                       -- asynchronous reset
    clear_n      : in  std_logic;                       -- synchronous reset
    sel_excl_s   : in  std_logic_vector(excl_depth-1 downto 0);  -- position of spare bit
    sel_excl_p   : in  std_logic_vector(excl_depth-1 downto 0);  -- position of parity bis
    ctrl_in      : in  std_logic;                       -- control signal (from FSM)
    we_n         : out std_logic;                       -- we directly from resync
    data_out     : out std_logic_vector(width-1 downto 0);  -- data output (into interface)
    word_low     : out std_logic;                       -- output word is low word
    prty_status  : out std_logic_vector(1 downto 0);    -- parity status bits
    cnt_en       : in  std_logic;                       -- error counter enable
    cnt_word     : out std_logic_vector(7 downto 0);    -- word count
    cnt_err_low  : out std_logic_vector(7 downto 0);    -- error count for low byte
    cnt_err_high : out std_logic_vector(7 downto 0);    -- error count for high byte
    p_ctrl_out   : out std_logic;                       -- control output to port
    p_strobe_in  : in  std_logic;                       -- strobe input signal
    tr_es_word   : in  std_logic_vector(width-1 downto 0); -- end signature word (trigger readout)
    tr_es        : out std_logic;                       -- end sig detected (trigger readout)
    rr_es_word   : in  std_logic_vector(width-1 downto 0); -- end signature for raw data readout
    rr_es        : out std_logic;                       -- end sig detected (raw data readout)
    p_d_in       : in  std_logic_vector((width/2)+1 downto 0));  -- data input
end ni_port_in;

----------------------------------------------------------------------------------------------------
-- ARCHITECTURE
----------------------------------------------------------------------------------------------------
architecture v of ni_port_in is

  -- CONSTANTS -------------------------------------------------------------------------------------
  constant ext_width : integer := (width/2)+2;

  -- SIGNALS ---------------------------------------------------------------------------------------
  signal we_n_i          : std_logic;
  signal data_out_i      : std_logic_vector(width-1 downto 0);
  signal data_resync_out : std_logic_vector((2*ext_width)-1 downto 0);
  signal prty_status_i   : std_logic_vector(1 downto 0);
  signal cnt_word_i      : std_logic_vector(7 downto 0);
  signal cnt_err_low_i   : std_logic_vector(7 downto 0);
  signal cnt_err_high_i  : std_logic_vector(7 downto 0);
  signal exclin_out_low  : std_logic_vector(ext_width-3 downto 0);
  signal exclin_out_high : std_logic_vector(ext_width-3 downto 0);
  signal prty_cmp_low    : std_logic;
  signal prty_cmp_high   : std_logic;
  signal prty_in_low     : std_logic;
  signal prty_in_high    : std_logic;
  signal word_low_i      : std_logic;

  -- COMPONENTS ------------------------------------------------------------------------------------

  component ni_exclude_in
    generic (
      width    : integer;
      depth    : integer);
    port (
      din      : in  std_logic_vector(width-1 downto 0);
      sel_s    : in  std_logic_vector(depth-1 downto 0);
      sel_p    : in  std_logic_vector(depth-1 downto 0);
      dout     : out std_logic_vector(width-3 downto 0);
      prty_bit : out std_logic);
  end component;

  component ni_resync
    generic (
      width : integer);
    port (
      clk      : in  std_logic;
      reset_n  : in  std_logic;
      clear_n  : in  std_logic;
      strobe   : in  std_logic;
      data_in  : in  std_logic_vector(width-1 downto 0);
      we_n     : out std_logic;
      data_out : out std_logic_vector((width*2)-1 downto 0));
  end component;

  -- STRUCTURE -------------------------------------------------------------------------------------
begin

  we_n       <= we_n_i;
  data_out_i <= exclin_out_high & exclin_out_low;
  data_out   <= data_out_i;
  word_low   <= word_low_i;

  -- resynchronization from incoming strobe signal to internal clock and data valid signal
  U_resync: ni_resync
    generic map (
        width => ext_width)
    port map (
        clk      => clk,
        reset_n  => reset_n,
        clear_n  => clear_n,
        strobe   => p_strobe_in,
        data_in  => p_d_in,
        we_n     => we_n_i,
        data_out => data_resync_out);

  -- get valid data byte & partiy bit for low and high byte
  U_dec_low: ni_exclude_in              -- extract data & partiy for low byte
    generic map (
      width    => ext_width,
      depth    => excl_depth)
    port map (
      din      => data_resync_out(ext_width-1 downto 0),
      sel_s    => sel_excl_s,
      sel_p    => sel_excl_p,
      dout     => exclin_out_low,
      prty_bit => prty_in_low);
  U_dec_high: ni_exclude_in             -- extract data & partiy for high byte
    generic map (
      width    => ext_width,
      depth    => excl_depth)
    port map (
      din      => data_resync_out((2*ext_width)-1 downto ext_width),
      sel_s    => sel_excl_s,
      sel_p    => sel_excl_p,
      dout     => exclin_out_high,
      prty_bit => prty_in_high);

  -- calculate parity for low and high byte from incoming data and compare with parity bit
  prty_status_i(1) <= prty_in_high xor prty_cmp_high;
  prty_status_i(0) <= prty_in_low xor prty_cmp_low;
  prty_status      <= prty_status_i;
  U_prty_low: process(exclin_out_low)   -- calc parity for low byte
    variable parity : std_logic;
  begin
    parity := '0';
    for i in ext_width-3 downto 0 loop
      parity := parity xor exclin_out_low(i);
    end loop;
    prty_cmp_low <= parity;
  end process;
  U_prty_high: process(exclin_out_high) -- calc parity for high byte
    variable parity : std_logic;
  begin
    parity := '0';
    for i in ext_width-3 downto 0 loop
      parity := parity xor exclin_out_high(i);
    end loop;
    prty_cmp_high <= parity;
  end process;

  -- register for ctrl output port
  U_ctrl_out_reg: process(clk, reset_n, ctrl_in)
  begin
    if reset_n = '0' then
      p_ctrl_out <= '0';
    elsif rising_edge(clk) then
      p_ctrl_out <= ctrl_in;
    end if;
  end process;

  -- count all incoming (16Bit) words
  cnt_word <= cnt_word_i;
  U_cnt_word: process(clk, reset_n, clear_n, cnt_word_i, cnt_en)
  begin
    if reset_n = '0' then cnt_word_i <= (others => '0');
    elsif rising_edge(clk) then
      cnt_word_i <= cnt_word_i;
      if (cnt_en = '1' and we_n_i = '0' and cnt_word_i /= "11111111") then
        cnt_word_i <= cnt_word_i + 1;
      end if;
      if clear_n = '0' then cnt_word_i <= (others => '0'); end if;
    end if;
  end process;

  -- count errors (wrong parity) for high/low byte if count enable is set
  cnt_err_low <= cnt_err_low_i;         -- error counter for low byte
  U_cnt_err_low: process(clk, reset_n, clear_n, prty_status_i, we_n_i, cnt_word_i, cnt_en, cnt_err_low_i)
  begin
    if reset_n = '0' then cnt_err_low_i <= (others => '0');
    elsif rising_edge(clk) then
      cnt_err_low_i <= cnt_err_low_i;
      if (cnt_en = '1' and prty_status_i(0) = '1' and we_n_i = '0' and cnt_word_i /= "11111111") then
        cnt_err_low_i <= cnt_err_low_i + 1;
      end if;
      if clear_n = '0' then cnt_err_low_i <= (others => '0'); end if;
    end if;
  end process;
  cnt_err_high <= cnt_err_high_i;       -- error counter for high byte
  U_cnt_err_high: process(clk, reset_n, clear_n, prty_status_i, we_n_i, cnt_word_i, cnt_en, cnt_err_high_i)
  begin
    if reset_n = '0' then cnt_err_high_i <= (others => '0');
    elsif rising_edge(clk) then
      cnt_err_high_i <= cnt_err_high_i;
      if (cnt_en = '1' and prty_status_i(1) = '1' and we_n_i = '0' and cnt_word_i /= "11111111") then
        cnt_err_high_i <= cnt_err_high_i + 1;
      end if;
      if clear_n = '0' then cnt_err_high_i <= (others => '0'); end if;
    end if;
  end process;

  -- 1 bit counter to detect if valid word is a high/low word
  U_lowhigh: process(clk, reset_n, clear_n, we_n_i, word_low_i)
  begin
    if reset_n = '0' then
      word_low_i <= '1';
    elsif rising_edge(clk) then
      if we_n_i = '0' then
        word_low_i <= not word_low_i;
      end if;
      if clear_n = '0' then word_low_i <= '1'; end if;
    end if;
  end process;

  -- generate end signature signals
  tr_es <= '1' when (data_out_i = tr_es_word and word_low_i = '1' and we_n_i = '0') else '0';
  rr_es <= '1' when (data_out_i = rr_es_word and word_low_i = '1' and we_n_i = '0') else '0';

end v;

----------------------------------------------------------------------------------------------------
-- CONFIGURATION
----------------------------------------------------------------------------------------------------
-- pragma translate_off
configuration ni_port_in_CFG of ni_port_in is
  for v
  end for; -- v
end ni_port_in_CFG;
-- pragma translate_on
