-------------------------------------------------------------------------------
-- Title      : ni_port_out
-- Project    : Network Interface (NI) for the ALICE TRD TRAP3
-------------------------------------------------------------------------------
-- File       : ni_port_out.vhd
-- Author     : Rolf Schneider  <rolf.schneider@kip.uni-heidelberg.de>
-- Company    : University Heidelberg - KIP
-- Last update: 2003-12-11
-- Platform   : Synopsys - v2003.03
-------------------------------------------------------------------------------
-- Description: output port
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author  Description
-- 2003/03/06  1.0      schneide	Created
-------------------------------------------------------------------------------


library IEEE;
use IEEE.std_logic_1164.all;


----------------------------------------------------------------------------------------------------
-- ENTITY
----------------------------------------------------------------------------------------------------

entity ni_port_out is
  generic (
    width        : integer := 16;                       -- internal bus width
    delay_depth  : integer := 3;                        -- number of delay stages for output port
    excl_depth   : integer := 4);                       -- number of bits for exclude positions
  port (
    clk          : in  std_logic;                       -- internal clock
    clk_alws     : in  std_logic;                       -- clock for optical link
    reset_n      : in  std_logic;                       -- asynchronous reset
    clear_n      : in  std_logic;                       -- sync. reset
    sel_excl_s   : in  std_logic_vector(excl_depth-1 downto 0);  -- position of spare bit
    sel_excl_p   : in  std_logic_vector(excl_depth-1 downto 0);  -- position of parity bit
    delay        : in  std_logic_vector((((width/2)+4)*delay_depth)-1 downto 0);  -- delays
    ol           : in  std_logic;                       -- module in front of optical link
    ctrl_out     : out std_logic;                       -- ctrl output signal
    we_n         : in  std_logic;                       -- write enable to write to the output port
    d_in         : in  std_logic_vector(width-1 downto 0);   -- data from interface into port
    p_ctrl_in    : in  std_logic;                       -- control input from port
    p_strobe_out : out std_logic;                       -- strobe output (clk output in front of OASE)
    p_ctrl_out   : out std_logic;                       -- ctrl output for OASE, eq to 'data_valid_n'
    p_d_out      : out std_logic_vector((width/2)+1 downto 0));  -- DDR data output
end ni_port_out;

----------------------------------------------------------------------------------------------------
-- ARCHITECTURE
----------------------------------------------------------------------------------------------------
architecture v of ni_port_out is

  -- CONSTANTS -------------------------------------------------------------------------------------
  constant ext_width : integer := (width/2)+2;

  -- SIGNALS ---------------------------------------------------------------------------------------
  signal d_in_ol         : std_logic_vector(width-1 downto 0);
  signal parity_low      : std_logic;
  signal parity_high     : std_logic;
  signal exclout_in_low  : std_logic_vector((width/2) downto 0);
  signal exclout_in_high : std_logic_vector((width/2) downto 0);
  signal data_sync_in    : std_logic_vector((ext_width*2)-1 downto 0);
  signal strobe_sync_out : std_logic;
  signal data_sync_out   : std_logic_vector(ext_width-1 downto 0);
  signal data_delay_in   : std_logic_vector(ext_width+1 downto 0);
  signal data_delay_out  : std_logic_vector(ext_width+1 downto 0);
  signal p_ctrl_out_reg  : std_logic;
  
  -- COMPONENTS ------------------------------------------------------------------------------------

  component ni_exclude_out
    generic (
      width    : integer;
      depth    : integer);
    port (
      din      : in  std_logic_vector(width-3 downto 0);
      prty_bit : in  std_logic;
      sel_p    : in  std_logic_vector(depth-1 downto 0);
      sel_s    : in  std_logic_vector(depth-1 downto 0);
      dout     : out std_logic_vector(width-1 downto 0));
  end component;

  component ni_sync
    generic (
      width : integer);
    port (
      clk      : in  std_logic;
      clk_alws : in  std_logic;
      reset_n  : in  std_logic;
      clear_n  : in  std_logic;
      ol       : in  std_logic;
      we_n     : in  std_logic;
      data_in  : in  std_logic_vector((width*2)-1 downto 0);
      strobe   : out std_logic;
      data_out : out std_logic_vector(width-1 downto 0));
  end component;

  component ni_delay_word
    generic (
      width : integer;
      depth : integer);
    port (
      delay    : in  std_logic_vector((width*depth)-1 downto 0);
      data_in  : in  std_logic_vector(width-1 downto 0);
      data_out : out std_logic_vector(width-1 downto 0));
  end component;

  component ni_register
    generic (
      width:    integer);
    port (
      clk:      in  std_logic;
      reset_n:  in  std_logic;
      A:        in  std_logic_vector(width-1 downto 0);
      O:        out std_logic_vector(width-1 downto 0));
  end component;

  -- STRUCTURE -------------------------------------------------------------------------------------
begin

  -- set data output to static '1's if control word is sent to optical link
  d_in_ol <= (others => '1') when we_n='1' else d_in;

  -- calculate parity for low and high byte of output word
  U_prty_low: process(d_in_ol)             -- low byte parity
    variable parity : std_logic;
  begin
    parity := '0';
    for i in (width/2)-1 downto 0 loop
      parity := parity xor d_in_ol(i);
    end loop;
    parity_low <= parity;
  end process;
  U_prty_high: process(d_in_ol)            -- high byte parity
    variable parity : std_logic;
  begin
    parity := '0';
    for i in width-1 downto (width/2) loop
      parity := parity xor d_in_ol(i);
    end loop;
    parity_high <= parity;
  end process;

  -- expand low and high byte with spare and parity bit
  U_excl_out_low: ni_exclude_out        -- insertion for low byte
    generic map (
      width    => ext_width,
      depth    => excl_depth)
    port map (
      din      => d_in_ol((width/2)-1 downto 0),
      prty_bit => parity_low,
      sel_p    => sel_excl_p,
      sel_s    => sel_excl_s,
      dout     => data_sync_in(ext_width-1 downto 0));
  U_excl_out_high: ni_exclude_out       -- insertion for high byte
    generic map (
      width    => ext_width,
      depth    => excl_depth)
    port map (
      din      => d_in_ol(width-1 downto (width/2)),
      prty_bit => parity_high,
      sel_p    => sel_excl_p,
      sel_s    => sel_excl_s,
      dout     => data_sync_in((2*ext_width)-1 downto ext_width));

  -- generate DDR output and strobe signal
  U_sync: ni_sync
    generic map (
        width    => ext_width)
    port map (
        clk      => clk,
        clk_alws => clk_alws,
        reset_n  => reset_n,
        clear_n  => clear_n,
        ol       => ol,
        we_n     => we_n,
        data_in  => data_sync_in,
        strobe   => strobe_sync_out,
        data_out => data_sync_out);

  -- output register for ctrl signal (OASE data valid)
  U_ctrl_out_reg: process(clk, reset_n, clear_n, we_n)
  begin
    if reset_n = '0' then
      p_ctrl_out_reg <= '1';
    elsif rising_edge(clk) then
      p_ctrl_out_reg <= we_n;
      if clear_n = '0' then p_ctrl_out_reg <= '1'; end if;
    end if;
  end process;

  -- delay each output bit incl. strobe signal
  data_delay_in <= p_ctrl_out_reg & strobe_sync_out & data_sync_out;
  U_delay: ni_delay_word
    generic map (
        width => ext_width+2,
        depth => delay_depth)
    port map (
        delay    => delay,
        data_in  => data_delay_in,
        data_out => data_delay_out);
  -- the following 'after' statements will be ignored during synthesis and is only
  -- necessary for simulation timing reasons
  p_d_out      <= data_delay_out(ext_width-1 downto 0) after 1 ns;
  p_strobe_out <= data_delay_out(ext_width);
  p_ctrl_out   <= data_delay_out(ext_width+1) after 1 ns;

  -- input register for ctrl signal
  U_ctrl_in_reg: process(clk, reset_n, p_ctrl_in)
  begin
    if reset_n = '0' then
      ctrl_out <= '0';
    elsif rising_edge(clk) then
      ctrl_out <= p_ctrl_in;
    end if;
  end process;

end v;

----------------------------------------------------------------------------------------------------
-- CONFIGURATION
----------------------------------------------------------------------------------------------------
-- pragma translate_off
configuration ni_port_out_CFG of ni_port_out is
  for v
  end for; -- v
end ni_port_out_CFG;
-- pragma translate_on
