-------------------------------------------------------------------------------
-- Title      : ni_sync
-- Project    : Network Interface (NI) for the ALICE TRD TRAP3
-------------------------------------------------------------------------------
-- File       : ni_sync.vhd
-- Author     : Rolf Schneider  <rolf.schneider@kip.uni-heidelberg.de>
-- Company    : University Heidelberg - KIP
-- Last update: 2003-12-11
-- Platform   : Synopsys - v2003.03
-------------------------------------------------------------------------------
-- Description: produces a DDR output with strobe signal from a sync. input
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author  Description
-- 2003/03/06  1.0      schneide	Created
-------------------------------------------------------------------------------


library IEEE;
use IEEE.std_logic_1164.all;

----------------------------------------------------------------------------------------------------
-- ENTITY
----------------------------------------------------------------------------------------------------
entity ni_sync is
  generic (
    width : integer := 10);                              -- width of the external interface
  port (
    clk      : in  std_logic;                            -- internal clock
    clk_alws : in  std_logic;                            -- clock for optical link
    reset_n  : in  std_logic;                            -- asynchronous reset
    clear_n  : in  std_logic;                            -- sync. reset
    ol       : in  std_logic;                            -- module in front of optical link
    we_n     : in  std_logic;                            -- write enable input
    data_in  : in  std_logic_vector((width*2)-1 downto 0);  -- incoming data
    strobe   : out std_logic;                            -- strobe output signal
    data_out : out std_logic_vector(width-1 downto 0));  -- data output, sync. to DDR strobe signal
end ni_sync;

----------------------------------------------------------------------------------------------------
-- ARCHITECTURE
----------------------------------------------------------------------------------------------------
architecture v of ni_sync is
  
  -- SIGNALS ---------------------------------------------------------------------------------------
  signal we_n_reg    : std_logic;
  signal data_in_reg : std_logic_vector((2*width)-1 downto 0);
  signal strobe_i    : std_logic;
  signal sel_data    : std_logic;
  
  -- COMPONENTS ------------------------------------------------------------------------------------
  component ni_register
    generic (
      width:    integer);
    port (
      clk:      in  STD_LOGIC;
      reset_n:  in  STD_LOGIC;
      A:        in  STD_LOGIC_VECTOR(width-1 downto 0);
      O:        out STD_LOGIC_VECTOR(width-1 downto 0));
  end component;

  component OR2D2
    port (
      Z  : out std_logic;
      A2 : in  std_logic;
      A1 : in  std_logic);
  end component;

  -- STRUCTURE -------------------------------------------------------------------------------------
begin

  strobe <= strobe_i;

  -- fetch data in output register
  U_data_in_reg : ni_register
    generic map (
      width => width*2)
    port map (
      clk     => clk,
      reset_n => reset_n,
      A       => data_in,
      O       => data_in_reg);

  -- fetch corresponding write enable signal in register
  U_we_n_reg: process(clk, reset_n, we_n)
  begin
    if reset_n = '0' then
      we_n_reg <= '1';                                -- async. reset of write enable
    elsif rising_edge(clk) then
      we_n_reg <= we_n;
      if clear_n = '0' then we_n_reg <= '1'; end if;  -- snyc. reset of write enable
    end if;
  end process;

  -- output strobe signal is gated clock signal or clock signal in optical link mode
  strobe_i <= clk_alws when ol = '1' else clk or we_n_reg;

  -- use different signal for data selection to set 'dont_touch' during synthesis on strobe signal
  U_byte_sel: OR2D2
    port map (
      Z  => sel_data,
      A2 => clk,
      A1 => we_n_reg);

  -- select high/low byte of output word
  data_out <=      data_in_reg(width-1 downto 0) when sel_data = '1'
              else data_in_reg((2*width)-1 downto width);

end v;

----------------------------------------------------------------------------------------------------
-- CONFIGURATION
----------------------------------------------------------------------------------------------------
-- pragma translate_off
configuration ni_sync_CFG of ni_sync is
  for v
  end for; -- v
end ni_sync_CFG;
-- pragma translate_on
