-------------------------------------------------------------------------------
-- Title      : ni_top
-- Project    : Network Interface (NI) for the ALICE TRD TRAP3
-------------------------------------------------------------------------------
-- File       : ni_top.vhd
-- Author     : Rolf Schneider  <rolf.schneider@kip.uni-heidelberg.de>
-- Company    : University Heidelberg - KIP
-- Last update: 2004-01-20
-- Platform   : Synopsys - v2003.03
-------------------------------------------------------------------------------
-- Description: toplevel unit of the NI. Includes (as components four local
--              io_interfaces, one global io_interface, FSM, four input ports,
--              one output port and four fifos. The data switches are part
--              of the top level design
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author  Description
-- 2003/03/19  1.0      schneide	Created
-- 2003/12/11  1.1      schneide	Changed name for entries type
--                                      Added entries & fsmstate to status
--                                      Removed unused signal in sensitivity
-- 2003/12/11  1.2      schneide	Added fifo_clear_disable (fcd)
-- 2004/01/20  1.3      schneide	Added clk to io_interface_g
-------------------------------------------------------------------------------


library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;
use WORK.ni_global.all;

----------------------------------------------------------------------------------------------------
-- ENTITY
----------------------------------------------------------------------------------------------------
entity ni_top is
  generic (
    width          :     INTEGER := 32);
  port (
    clk            : in  STD_LOGIC;     -- internal clock
    clk_glob       : in  STD_LOGIC;     -- global bus clock
    clk_alws       : in  STD_LOGIC;     -- clock for optical link
    reset_n        : in  STD_LOGIC;     -- asynchronous reset
    clear_n        : in  STD_LOGIC;     -- synchronous reset
    mode_tr        : in  STD_LOGIC;     -- trigger readout mode
    mode_rr        : in  STD_LOGIC;     -- raw data readout mode
    finished_n     : out STD_LOGIC;     -- tr/rr readout has finished
    -- local bus 0
    we_0           : in  STD_LOGIC;
    address_0      : in  STD_LOGIC_VECTOR(io_address_bus_width-1 downto 0);
    din_0          : in  STD_LOGIC_VECTOR(width-1 downto 0);
    dout_0         : out STD_LOGIC_VECTOR(width-1 downto 0);
    intr_0         : out STD_LOGIC_VECTOR(2 downto 0);  -- interrupts to CPU0
    -- local bus 1
    we_1           : in  STD_LOGIC;
    address_1      : in  STD_LOGIC_VECTOR(io_address_bus_width-1 downto 0);
    din_1          : in  STD_LOGIC_VECTOR(width-1 downto 0);
    dout_1         : out STD_LOGIC_VECTOR(width-1 downto 0);
    intr_1         : out STD_LOGIC_VECTOR(2 downto 0);  -- interrupts to CPU1
    -- local bus 2
    we_2           : in  STD_LOGIC;
    address_2      : in  STD_LOGIC_VECTOR(io_address_bus_width-1 downto 0);
    din_2          : in  STD_LOGIC_VECTOR(width-1 downto 0);
    dout_2         : out STD_LOGIC_VECTOR(width-1 downto 0);
    intr_2         : out STD_LOGIC_VECTOR(2 downto 0);  -- interrupts to CPU2
    -- local bus 3
    we_3           : in  STD_LOGIC;
    address_3      : in  STD_LOGIC_VECTOR(io_address_bus_width-1 downto 0);
    din_3          : in  STD_LOGIC_VECTOR(width-1 downto 0);
    dout_3         : out STD_LOGIC_VECTOR(width-1 downto 0);
    intr_3         : out STD_LOGIC_VECTOR(2 downto 0);  -- interrupts to CPU3
    -- global
    we_g           : in  STD_LOGIC;
    address_g      : in  STD_LOGIC_VECTOR(io_address_bus_width-1 downto 0);
    din_g          : in  STD_LOGIC_VECTOR(width-1 downto 0);
    dout_g         : out STD_LOGIC_VECTOR(width-1 downto 0);
    intr_g         : out STD_LOGIC;                     -- global interrupt
    -- port 0
    p_strobe_in_0  : in  STD_LOGIC;
    p_d_in_0       : in  STD_LOGIC_VECTOR((width/4)+1 downto 0);
    p_ctrl_out_0   : out STD_LOGIC;
    -- port 1
    p_strobe_in_1  : in  STD_LOGIC;
    p_d_in_1       : in  STD_LOGIC_VECTOR((width/4)+1 downto 0);
    p_ctrl_out_1   : out STD_LOGIC;
    -- port 2
    p_strobe_in_2  : in  STD_LOGIC;
    p_d_in_2       : in  STD_LOGIC_VECTOR((width/4)+1 downto 0);
    p_ctrl_out_2   : out STD_LOGIC;
    -- port 3
    p_strobe_in_3  : in  STD_LOGIC;
    p_d_in_3       : in  STD_LOGIC_VECTOR((width/4)+1 downto 0);
    p_ctrl_out_3   : out STD_LOGIC;
    -- port 4 (output)
    p_ctrl_in      : in  STD_LOGIC;
    p_strobe_out   : out STD_LOGIC;     -- optical link mode: clock output
    p_d_out        : out STD_LOGIC_VECTOR((width/4)+1 downto 0);
    p_ctrl         : out STD_LOGIC      -- control/data signal for optical link output
    );
end ni_top;

----------------------------------------------------------------------------------------------------
-- ARCHITECTURE
----------------------------------------------------------------------------------------------------
architecture v of ni_top is

  -- CONSTANTS -------------------------------------------------------------------------------------
  constant iwidth          : integer := width/2;        -- internal bus width
  constant ext_width       : integer := (iwidth/2)+2;   -- external bus width
  constant delay_depth     : integer := 3;              -- number of delay stages

  -- SIGNALS ---------------------------------------------------------------------------------------
  signal clear_n_i      : std_logic;
  signal finished_n_i   : std_logic;
  signal rr_es_word     : std_logic_vector(iwidth-1 downto 0);
  signal tr_es_word     : std_logic_vector(iwidth-1 downto 0);
  signal imsm           : std_logic;
  signal mode           : std_logic_vector(2 downto 0);
  signal sel_p2f        : std_logic;
  signal sel_f2p        : std_logic_vector(1 downto 0);
  signal sel_num        : std_logic_vector(1 downto 0);
  signal sel_e2p        : std_logic_vector(2 downto 0);
  signal pi_clear_n     : std_logic;
  signal po_clear_n     : std_logic;
  signal f_clear_n      : std_logic;

  -- global io interface
  signal gio_we       : std_logic;
  signal gio_stat     : g_stat_array;
  signal gio_stat_i   : std_logic_vector(g_stat_w_sum-1 downto 0);
  signal gio_ctrl     : g_ctrl_array;
  signal gio_ctrl_i   : std_logic_vector(g_ctrl_w_sum-1 downto 0);

  -- local io interfaces
  subtype idata_type is std_logic_vector(iwidth-1 downto 0);
  type    idata_array is array (3 downto 0) of idata_type;
  subtype io_address_type is std_logic_vector(io_address_bus_width-1 downto 0);
  type    io_address_array is array (3 downto 0) of io_address_type;
  subtype status_type is std_logic_vector(l_stat_w_sum-1 downto 0);
  type    status_array_i is array (3 downto 0) of status_type;
  type    status_array is array (3 downto 0) of l_stat_array;
  type    data_array is array (3 downto 0) of data_type;
  signal  lio_we           : std_logic_vector(3 downto 0);
  signal  lio_address      : io_address_array;
  signal  lio_din          : data_array;
  signal  lio_dout         : data_array;
  signal  lio_bus_din_we_n : bus_we_h_array;
  signal  lio_bus_dout_low : std_logic_vector(3 downto 0);
  signal  lio_bus_in       : idata_array;
  signal  lio_bus_out      : idata_array;
  signal  lio_stat_i       : status_array_i;
  signal  lio_stat         : status_array;
  signal  lio_dout_r_n     : std_logic_vector(3 downto 0);
  signal  lio_din_w_n      : std_logic_vector(3 downto 0);
  signal  lio_tr_es        : std_logic_vector(3 downto 0);
  signal  lio_rr_es        : std_logic_vector(3 downto 0);

  -- input port signals
  subtype excl_type is std_logic_vector(3 downto 0);
  type    excl_array is array(3 downto 0) of excl_type;
  subtype ext_type is std_logic_vector(ext_width-1 downto 0);
  type    ext_array is array (3 downto 0) of ext_type;
  subtype cnt_type is std_logic_vector(7 downto 0);
  type    cnt_array is array (3 downto 0) of cnt_type;
  signal  pi_sel_excl_s   : excl_array;
  signal  pi_sel_excl_p   : excl_array;
  signal  pi_ctrl_in      : std_logic_vector(3 downto 0);
  signal  pi_we_n         : std_logic_vector(3 downto 0);
  signal  pi_data_out     : idata_array;
  signal  pi_word_low     : std_logic_vector(3 downto 0);
  signal  pi_prty_status  : bus_we_h_array;
  signal  pi_cnt_en       : std_logic_vector(3 downto 0);
  signal  pi_cnt_word     : cnt_array;
  signal  pi_cnt_err_low  : cnt_array;
  signal  pi_cnt_err_high : cnt_array;
  signal  pi_p_ctrl_out   : std_logic_vector(3 downto 0);
  signal  pi_p_strobe_in  : std_logic_vector(3 downto 0);
  signal  pi_tr_es        : std_logic_vector(3 downto 0);
  signal  pi_rr_es        : std_logic_vector(3 downto 0);
  signal  pi_p_d_in       : ext_array;

  -- fifo signals
  subtype fentries_type is std_logic_vector(7 downto 0);
  type    fentries_array is array (3 downto 0) of fentries_type;
  signal  f_push_req_n   : std_logic_vector(3 downto 0);
  signal  f_pop_req_n    : std_logic_vector(3 downto 0);
  signal  f_data_in      : idata_array;
  signal  f_low_bound    : fentries_array;
  signal  f_high_bound   : fentries_array;
  signal  f_low          : std_logic_vector(3 downto 0);
  signal  f_high         : std_logic_vector(3 downto 0);
  signal  f_entries      : fentries_array;
  signal  f_empty        : std_logic_vector(3 downto 0);
  signal  f_almost_empty : std_logic_vector(3 downto 0);
  signal  f_half_full    : std_logic_vector(3 downto 0);
  signal  f_almost_full  : std_logic_vector(3 downto 0);
  signal  f_full         : std_logic_vector(3 downto 0);
  signal  f_error        : std_logic_vector(3 downto 0);
  signal  f_word_low     : std_logic_vector(3 downto 0);
  signal  f_tr_es        : std_logic_vector(3 downto 0);
  signal  f_rr_es        : std_logic_vector(3 downto 0);
  signal  f_data_out     : idata_array;

  -- f2p_mux signals
  signal  f2p_out        : idata_array;
  signal  e2p_in         : std_logic_vector(iwidth-1 downto 0);

  -- e2p_mux signals
  signal  tp_low_word    : std_logic_vector(iwidth-1 downto 0);
  signal  tp_high_word   : std_logic_vector(iwidth-1 downto 0);

  -- output port signals
  signal  po_ctrl_in_i  : std_logic;
  signal  po_ol         : std_logic;
  signal  po_sel_excl_s : std_logic_vector(3 downto 0);
  signal  po_sel_excl_p : std_logic_vector(3 downto 0);
  signal  po_delay      : std_logic_vector(((ext_width+2)*delay_depth)-1 downto 0);
  signal  po_ctrl_out   : std_logic;
  signal  po_we_n       : std_logic;
  signal  po_d_in       : std_logic_vector(iwidth-1 downto 0);

  -- fsm signals
  signal  fsm_fsm_id           : std_logic_vector(4 downto 0);
  signal  fsm_mode             : std_logic_vector(2 downto 0);
  signal  fsm_mode_fsm         : std_logic_vector(2 downto 0);
  signal  fsm_order_root       : std_logic;
  signal  fsm_order_tr         : std_logic_vector(17 downto 0);
  signal  fsm_order_rr         : std_logic_vector(17 downto 0);
  signal  fsm_lio_bus_din_we_n : std_logic_vector(7 downto 0);
  signal  fsm_lio_data_valid   : std_logic_vector(3 downto 0);
  signal  fsm_lio_ctrl         : std_logic_vector(3 downto 0);
  signal  fsm_lio_ctrl_en_n    : std_logic_vector(3 downto 0);
  signal  fsm_word_cut         : std_logic_vector(31 downto 0);
  signal  fsm_word_cnt         : std_logic_vector(31 downto 0);
  signal  fsm_fcd              : std_logic;

  -- COMPONENTS ------------------------------------------------------------------------------------
  component ni_io_interface
    generic (
      width         : INTEGER);
    port (
      clk          : in  STD_LOGIC;
      reset_n      : in  STD_LOGIC;
      cpu_we       : in  STD_LOGIC;
      cpu_address  : in  STD_LOGIC_VECTOR(io_address_bus_width-1 downto 0);
      cpu_din      : in  STD_LOGIC_VECTOR(width-1 downto 0);
      cpu_dout     : out STD_LOGIC_VECTOR(width-1 downto 0);
      bus_din_we_n : in  STD_LOGIC_VECTOR(2-1 downto 0);
      bus_dout_low : in  STD_LOGIC;
      bus_in       : in  STD_LOGIC_VECTOR((width/2)-1 downto 0);
      bus_out      : out STD_LOGIC_VECTOR((width/2)-1 downto 0);
      status       : in  STD_LOGIC_VECTOR(l_stat_w_sum-1 downto 0);
      dout_r_n     : out STD_LOGIC;
      din_w_n      : out STD_LOGIC;
      tr_es_word   : in  STD_LOGIC_VECTOR(iwidth-1 downto 0);
      tr_es        : out STD_LOGIC;
      rr_es_word   : in  STD_LOGIC_VECTOR(iwidth-1 downto 0);
      rr_es        : out STD_LOGIC);
  end component;

  component ni_io_interface_g
    generic (
      width       : INTEGER);
    port (
      clk         : in  STD_LOGIC;
      gclk        : in  STD_LOGIC;
      reset_n     : in  STD_LOGIC;
      cpu_we      : in  STD_LOGIC;
      cpu_address : in  STD_LOGIC_VECTOR(io_address_bus_width-1 downto 0);
      cpu_din     : in  STD_LOGIC_VECTOR(width-1 downto 0);
      cpu_dout    : out STD_LOGIC_VECTOR(width-1 downto 0);
      status      : in  STD_LOGIC_VECTOR(g_stat_w_sum-1 downto 0);
      ctrl        : out STD_LOGIC_VECTOR(g_ctrl_w_sum-1 downto 0));
  end component;

  component ni_port_in
    generic (
      width        : integer;
      excl_depth   : integer);
    port (
      clk          : in  std_logic;
      reset_n      : in  std_logic;
      clear_n      : in  std_logic;
      sel_excl_s   : in  std_logic_vector(excl_depth-1 downto 0);
      sel_excl_p   : in  std_logic_vector(excl_depth-1 downto 0);
      ctrl_in      : in  std_logic;
      we_n         : out std_logic;
      data_out     : out std_logic_vector(width-1 downto 0);
      word_low     : out std_logic;
      prty_status  : out std_logic_vector(1 downto 0);
      cnt_en       : in  std_logic;
      cnt_word     : out std_logic_vector(7 downto 0);
      cnt_err_low  : out std_logic_vector(7 downto 0);
      cnt_err_high : out std_logic_vector(7 downto 0);
      p_ctrl_out   : out std_logic;
      p_strobe_in  : in  std_logic;
      tr_es_word   : in  std_logic_vector(width-1 downto 0);
      tr_es        : out std_logic;
      rr_es_word   : in  std_logic_vector(width-1 downto 0);
      rr_es        : out std_logic;
      p_d_in       : in  std_logic_vector((width/2)+1 downto 0));
  end component;

  component ni_port_out
    generic (
      width        : integer;
      delay_depth  : integer;
      excl_depth   : integer);
    port (
      clk          : in  std_logic;
      clk_alws     : in  std_logic;
      reset_n      : in  std_logic;
      clear_n      : in  std_logic;
      sel_excl_s   : in  std_logic_vector(excl_depth-1 downto 0);
      sel_excl_p   : in  std_logic_vector(excl_depth-1 downto 0);
      delay        : in  std_logic_vector((((width/2)+4)*delay_depth)-1 downto 0);
      ol           : in  std_logic;
      ctrl_out     : out std_logic;
      we_n         : in  std_logic;
      d_in         : in  std_logic_vector(width-1 downto 0);
      p_ctrl_in    : in  std_logic;
      p_strobe_out : out std_logic;
      p_ctrl_out   : out std_logic;
      p_d_out      : out std_logic_vector((width/2)+1 downto 0));
  end component;

  component ni_fifo_mb
    generic (
      width        : integer;
      depth        : integer);
    port (
      clk          : in  std_logic;
      reset_n      : in  std_logic;
      clear_n      : in  std_logic;
      mode         : in  std_logic_vector(2 downto 0);
      push_req_n   : in  std_logic;
      pop_req_n    : in  std_logic;
      data_in      : in  std_logic_vector(width-1 downto 0);
      low_bound    : in  std_logic_vector(depth downto 0);
      high_bound   : in  std_logic_vector(depth downto 0);
      entries      : out std_logic_vector(depth downto 0);
      low          : out std_logic;
      high         : out std_logic;
      empty        : out std_logic;
      almost_empty : out std_logic;
      half_full    : out std_logic;
      almost_full  : out std_logic;
      full         : out std_logic;
      error        : out std_logic;
      word_low     : out std_logic;
      tr_es_word   : in  std_logic_vector(width-1 downto 0);
      tr_es        : out std_logic;
      rr_es_word   : in  std_logic_vector(width-1 downto 0);
      rr_es        : out std_logic;
      data_out     : out std_logic_vector(width-1 downto 0));
  end component;

  component ni_fsm
    port (
      clk              : in  std_logic;
      reset_n          : in  std_logic;
      clear_n          : in  std_logic;
      finished_n       : out std_logic;
      fsm_id           : out std_logic_vector(4 downto 0);
      mode             : in  std_logic_vector(2 downto 0);
      mode_fsm         : out std_logic_vector(2 downto 0);
      order_root       : in  std_logic;
      order_tr         : in  std_logic_vector(17 downto 0);
      order_rr         : in  std_logic_vector(17 downto 0);
      word_cut         : in  std_logic_vector(31 downto 0);
      word_cnt         : in  std_logic_vector(31 downto 0);
      fifo_clr_disable : in  std_logic;
      lio_bus_din_we_n : out std_logic_vector(7 downto 0);
      lio_bus_dout_low : out std_logic_vector(3 downto 0);
      lio_data_valid   : out std_logic_vector(3 downto 0);
      lio_ctrl         : in  std_logic_vector(3 downto 0);
      lio_ctrl_en_n    : in  std_logic_vector(3 downto 0);
      lio_dout_r_n     : in  std_logic_vector(3 downto 0);
      lio_din_w_n      : in  std_logic_vector(3 downto 0);
      lio_tr_es        : in  std_logic_vector(3 downto 0);
      lio_rr_es        : in  std_logic_vector(3 downto 0);
      pi_clear_n       : out std_logic;
      pi_ctrl_in       : out std_logic_vector(3 downto 0);
      pi_we_n          : in  std_logic_vector(3 downto 0);
      pi_word_low      : in  std_logic_vector(3 downto 0);
      pi_tr_es         : in  std_logic_vector(3 downto 0);
      pi_rr_es         : in  std_logic_vector(3 downto 0);
      sel_p2f          : out std_logic;
      f_clear_n        : out std_logic;
      f_push_req_n     : out std_logic_vector(3 downto 0);
      f_pop_req_n      : out std_logic_vector(3 downto 0);
      f_high           : in  std_logic_vector(3 downto 0);
      f_empty          : in  std_logic_vector(3 downto 0);
      f_almost_empty   : in  std_logic_vector(3 downto 0);
      f_almost_full    : in  std_logic_vector(3 downto 0);
      f_full           : in  std_logic_vector(3 downto 0);
      f_error          : in  std_logic_vector(3 downto 0);
      f_word_low       : in  std_logic_vector(3 downto 0);
      f_tr_es          : in  std_logic_vector(3 downto 0);
      f_rr_es          : in  std_logic_vector(3 downto 0);
      sel_f2p          : out std_logic_vector(1 downto 0);
      sel_num          : out std_logic_vector(1 downto 0);
      sel_e2p          : out std_logic_vector(2 downto 0);
      po_clear_n       : out std_logic;
      po_ctrl_out      : in  std_logic;
      po_we_n          : out std_logic);
  end component;

  -- STRUCTURE -------------------------------------------------------------------------------------
begin

  --------------------------------------------------------------------------------------------------
  -- ASSIGN ALL GLOBAL I/O CONTROL SIGNALS --

  mode           <= gio_ctrl(0)( 2 downto  0);
  clear_n_i      <= clear_n and gio_ctrl(0)(3);
  imsm           <= gio_ctrl(0)(4);
  fsm_fcd        <= gio_ctrl(0)(5);
  po_delay       <= gio_ctrl(2)( 5 downto  0) & gio_ctrl(1)(29 downto 0);
  po_sel_excl_s  <= gio_ctrl(2)( 9 downto  6);
  po_sel_excl_p  <= gio_ctrl(2)(13 downto 10);
  fsm_order_root <= gio_ctrl(2)(14);
  po_ol          <= gio_ctrl(2)(15);
  fsm_order_tr   <= gio_ctrl(3)(17 downto  0);
  fsm_order_rr   <= gio_ctrl(4)(17 downto  0);
  tr_es_word     <= gio_ctrl(5)(iwidth-1 downto 0);
  rr_es_word     <= gio_ctrl(5)(width-1 downto iwidth);
  tp_low_word    <= gio_ctrl(6)(iwidth-1 downto 0);
  tp_high_word   <= gio_ctrl(6)(width-1 downto iwidth);
  U_io_g_ctrl_loop: for i in 3 downto 0 generate
    f_low_bound(i)       <= gio_ctrl(7)(7 downto 0);
    f_high_bound(i)      <= gio_ctrl(7)(15 downto 8);
    fsm_lio_ctrl(i)      <= gio_ctrl(8+i)(0);
    fsm_lio_ctrl_en_n(i) <= gio_ctrl(8+i)(1);
    pi_cnt_en(i)         <= gio_ctrl(8+i)(2);
    pi_sel_excl_s(i)     <= gio_ctrl(8+i)(6 downto 3);
    pi_sel_excl_p(i)     <= gio_ctrl(8+i)(10 downto 7);
  end generate;
  fsm_word_cut   <= gio_ctrl(12);

  -- ASSIGN ALL GLOBAL I/O STATUS SIGNALS --
  gio_stat(0)(0)          <= po_ctrl_in_i;
  gio_stat(1)(3 downto 0) <= f_error;
  gio_stat(2)(4 downto 0) <= fsm_fsm_id;

  -- ASSIGN ALL LOCAL I/O STATUS SIGNALS --
  U_io_l_stat_loop: for i in 3 downto 0 generate
    lio_stat(i)(0)(0)            <= f_empty(i);
    lio_stat(i)(0)(1)            <= f_almost_empty(i);
    lio_stat(i)(0)(2)            <= f_half_full(i);
    lio_stat(i)(0)(3)            <= f_almost_full(i);
    lio_stat(i)(0)(4)            <= f_full(i);
    lio_stat(i)(0)(5)            <= f_low(i);
    lio_stat(i)(0)(6)            <= f_high(i);
    lio_stat(i)(0)(7)            <= f_error(i);
    lio_stat(i)(0)( 9 downto  8) <= pi_prty_status(i);
    lio_stat(i)(0)(10)           <= fsm_lio_data_valid(i);
    lio_stat(i)(1)( 7 downto  0) <= pi_cnt_word(i);
    lio_stat(i)(1)(15 downto  8) <= pi_cnt_err_low(i);
    lio_stat(i)(1)(23 downto 16) <= pi_cnt_err_high(i);
    lio_stat(i)(2)( 7 downto  0) <= f_entries(i);
  end generate;

  --------------------------------------------------------------------------------------------------
  -- ASSIGN PORT TO INTERNAL SIGNALS AND VICE VERSA --

  U_g_ctrl_ntt : for i in g_ctrl_regs-1 downto 0 generate       -- global control vector to array
    gio_ctrl(i)(g_ctrl_w(i)-1 downto 0) <=
      gio_ctrl_i(g_ctrl_offset(i)+g_ctrl_w(i)-1 downto g_ctrl_offset(i));
  end generate;

  U_g_stat_ntt : for i in g_stat_regs-1 downto 0 generate       -- global status array to vector
    gio_stat_i(g_stat_offset(i)+g_stat_w(i)-1 downto g_stat_offset(i)) <=
      gio_stat(i)(g_stat_w(i)-1 downto 0);
  end generate;

  U_l_stat_no : for j in 3 downto 0 generate
    U_l_stat_ntt : for i in l_stat_regs-1 downto 0 generate     -- local status array to vector
      lio_stat_i(j)(l_stat_offset(i)+l_stat_w(i)-1 downto l_stat_offset(i)) <=
        lio_stat(j)(i)(l_stat_w(i)-1 downto 0);
    end generate;
  end generate;

  po_ctrl_in_i <= not p_ctrl_in;                                -- output port

  gio_we <= we_g;                                               -- global I/O signals

                                                                -- local and global interrupts
  U_intr_sync: process(clk, reset_n)
  begin
    if reset_n='0' then
      intr_0 <= (others => '0');
      intr_1 <= (others => '0');
      intr_2 <= (others => '0');
      intr_3 <= (others => '0');
      intr_g <= '0';
    elsif rising_edge(clk) then
      intr_0 <= f_high(0) & f_low(0) & f_error(0);
      intr_1 <= f_high(1) & f_low(1) & f_error(1);
      intr_2 <= f_high(2) & f_low(2) & f_error(2);
      intr_3 <= f_high(3) & f_low(3) & f_error(3);
      intr_g <= po_ctrl_in_i;
    end if;
  end process;

  lio_we <= we_3 & we_2 & we_1 & we_0;                          -- local I/O address & data signals
  lio_address <= (address_3, address_2, address_1, address_0);
  lio_din     <= (din_3, din_2, din_1, din_0);
  dout_3 <= lio_dout(3);
  dout_2 <= lio_dout(2);
  dout_1 <= lio_dout(1);
  dout_0 <= lio_dout(0);
  lio_bus_in <= f_data_out;
  U_bus_din_we: for i in 3 downto 0 generate
    lio_bus_din_we_n(i) <= fsm_lio_bus_din_we_n((2*i)+1 downto 2*i);
  end generate;

  (p_ctrl_out_3, p_ctrl_out_2, p_ctrl_out_1, p_ctrl_out_0) <= not pi_p_ctrl_out;  -- input ports
  pi_p_strobe_in <= (p_strobe_in_3, p_strobe_in_2, p_strobe_in_1, p_strobe_in_0);
  pi_p_d_in      <= (p_d_in_3, p_d_in_2, p_d_in_1, p_d_in_0);

  fsm_mode <= mode(2) &                                         -- fsm signals
              ((mode_rr and not imsm) or mode(1)) &
              ((mode_tr and not imsm) or mode(0));
  fsm_word_cnt <= pi_cnt_word(3) & pi_cnt_word(2) & pi_cnt_word(1) & pi_cnt_word(0);
  U_finished: process(clk, reset_n)
  begin
    if reset_n = '0' then
      finished_n <= '1';
    elsif rising_edge(clk) then
      finished_n <= finished_n_i;
    end if;
  end process;

  --------------------------------------------------------------------------------------------------
  -- CONNECT COMPONENTS --

  U_io_glob: ni_io_interface_g                                  -- global I/O interface
    generic map (
      width       => width)
    port map (
      clk         => clk,
      gclk        => clk_glob,
      reset_n     => reset_n,
      cpu_we      => gio_we,
      cpu_address => address_g,
      cpu_din     => din_g,
      cpu_dout    => dout_g,
      status      => gio_stat_i,
      ctrl        => gio_ctrl_i);

  U_io_loc_loop: for i in 3 downto 0 generate                   -- local I/O interface
    U_io_loc: ni_io_interface
      generic map (
        width         => width)
      port map (
        clk          => clk,
        reset_n      => reset_n,
        cpu_we       => lio_we(i),
        cpu_address  => lio_address(i),
        cpu_din      => lio_din(i),
        cpu_dout     => lio_dout(i),
        bus_din_we_n => lio_bus_din_we_n(i),
        bus_dout_low => lio_bus_dout_low(i),
        bus_in       => lio_bus_in(i),
        bus_out      => lio_bus_out(i),
        status       => lio_stat_i(i),
        dout_r_n     => lio_dout_r_n(i),
        din_w_n      => lio_din_w_n(i),
        tr_es_word   => tr_es_word,
        tr_es        => lio_tr_es(i),
        rr_es_word   => rr_es_word,
        rr_es        => lio_rr_es(i));
  end generate;

  U_port_in_loop : for i in 3 downto 0 generate                 -- input ports
    U_port_in : ni_port_in
      generic map (
        width        => iwidth,
        excl_depth   => excl_depth)
      port map (
        clk          => clk,
        reset_n      => reset_n,
        clear_n      => pi_clear_n,
        sel_excl_s   => pi_sel_excl_s(i),
        sel_excl_p   => pi_sel_excl_p(i),
        ctrl_in      => pi_ctrl_in(i),
        we_n         => pi_we_n(i),
        data_out     => pi_data_out(i),
        word_low     => pi_word_low(i),
        prty_status  => pi_prty_status(i),
        cnt_en       => pi_cnt_en(i),
        cnt_word     => pi_cnt_word(i),
        cnt_err_low  => pi_cnt_err_low(i),
        cnt_err_high => pi_cnt_err_high(i),
        p_ctrl_out   => pi_p_ctrl_out(i),
        p_strobe_in  => pi_p_strobe_in(i),
        tr_es_word   => tr_es_word,
        tr_es        => pi_tr_es(i),
        rr_es_word   => rr_es_word,
        rr_es        => pi_rr_es(i),
        p_d_in       => pi_p_d_in(i));
  end generate;

  -- select fifo input between input ports and local io
  U_p2f: block                                                  -- port/lio to fifo select (p2f)
  begin
    f_data_in <= pi_data_out when sel_p2f = '1' else lio_bus_out;
  end block;

  U_fifo_mb_loop : for i in 3 downto 0 generate                 -- fifos
    U_fifo_mb : ni_fifo_mb
      generic map (
      width          => iwidth,
      depth          => 7)
      port map (
        clk          => clk,
        reset_n      => reset_n,
        clear_n      => f_clear_n,
        mode         => fsm_mode_fsm,
        push_req_n   => f_push_req_n(i),
        pop_req_n    => f_pop_req_n(i),
        data_in      => f_data_in(i),
        low_bound    => f_low_bound(i),
        high_bound   => f_high_bound(i),
        entries      => f_entries(i),
        low          => f_low(i),
        high         => f_high(i),
        empty        => f_empty(i),
        almost_empty => f_almost_empty(i),
        half_full    => f_half_full(i),
        almost_full  => f_almost_full(i),
        full         => f_full(i),
        error        => f_error(i),
        word_low     => f_word_low(i),
        tr_es_word   => tr_es_word,
        tr_es        => f_tr_es(i),
        rr_es_word   => rr_es_word,
        rr_es        => f_rr_es(i),
        data_out     => f_data_out(i));
  end generate;

  -- select output between fifo and local io busses
  U_f2p: block                                                  -- fifo/lio select (f2p)
  begin
    with sel_f2p select
      f2p_out <= lio_bus_out when "00",
                  f_data_out when "01",
                 pi_data_out when "10",
      (f2p_out'range => (others => '1')) when others;
  end block;

  -- select fifo/local I/O output depending on sel_num
  U_e2p: block                                                  -- no. fifo/lio select (e2p)
  begin
    e2p_in <= f2p_out(conv_integer(sel_num))(iwidth-1 downto 0);
  end block;

  -- select output between data (fifo/lio), trigger readout end sig
  -- and raw data readout end sig
  U_po_d: block                                                 -- e2p/const to port select (po_d)
  begin
    with sel_e2p select
      po_d_in <=     e2p_in when "000",
                 tr_es_word when "001",
                 rr_es_word when "010",
                tp_low_word when "100",
               tp_high_word when "101",
            (others => '1') when others;
  end block;

  U_port_out: ni_port_out                                       -- output port
    generic map (
      width        => iwidth,
      delay_depth  => delay_depth,
      excl_depth   => excl_depth)
    port map (
      clk          => clk,
      clk_alws     => clk_alws,
      reset_n      => reset_n,
      clear_n      => po_clear_n,
      sel_excl_s   => po_sel_excl_s,
      sel_excl_p   => po_sel_excl_p,
      delay        => po_delay,
      ol           => po_ol,
      ctrl_out     => po_ctrl_out,
      we_n         => po_we_n,
      d_in         => po_d_in,
      p_ctrl_in    => po_ctrl_in_i,
      p_strobe_out => p_strobe_out,
      p_ctrl_out   => p_ctrl,
      p_d_out      => p_d_out);

  U_fsm: ni_fsm                                                 -- master FSM
    port map (
      clk              => clk,
      reset_n          => reset_n,
      clear_n          => clear_n_i,
      finished_n       => finished_n_i,
      fsm_id           => fsm_fsm_id,
      mode             => fsm_mode,
      mode_fsm         => fsm_mode_fsm,
      order_root       => fsm_order_root,
      order_tr         => fsm_order_tr,
      order_rr         => fsm_order_rr,
      word_cut         => fsm_word_cut,
      word_cnt         => fsm_word_cnt,
      fifo_clr_disable => fsm_fcd,
      lio_bus_din_we_n => fsm_lio_bus_din_we_n,
      lio_bus_dout_low => lio_bus_dout_low,
      lio_data_valid   => fsm_lio_data_valid,
      lio_ctrl         => fsm_lio_ctrl,
      lio_ctrl_en_n    => fsm_lio_ctrl_en_n,
      lio_dout_r_n     => lio_dout_r_n,
      lio_din_w_n      => lio_din_w_n,
      lio_tr_es        => lio_tr_es,
      lio_rr_es        => lio_rr_es,
      pi_clear_n       => pi_clear_n,
      pi_ctrl_in       => pi_ctrl_in,
      pi_we_n          => pi_we_n,
      pi_word_low      => pi_word_low,
      pi_tr_es         => pi_tr_es,
      pi_rr_es         => pi_rr_es,
      sel_p2f          => sel_p2f,
      f_clear_n        => f_clear_n,
      f_push_req_n     => f_push_req_n,
      f_pop_req_n      => f_pop_req_n,
      f_high           => f_high,
      f_empty          => f_empty,
      f_almost_empty   => f_almost_empty,
      f_almost_full    => f_almost_full,
      f_full           => f_full,
      f_error          => f_error,
      f_word_low       => f_word_low,
      f_tr_es          => f_tr_es,
      f_rr_es          => f_rr_es,
      sel_f2p          => sel_f2p,
      sel_num          => sel_num,
      sel_e2p          => sel_e2p,
      po_clear_n       => po_clear_n,
      po_ctrl_out      => po_ctrl_out,
      po_we_n          => po_we_n);

end v;

----------------------------------------------------------------------------------------------------
-- CONFIGURATION
----------------------------------------------------------------------------------------------------
-- pragma translate_off
configuration ni_top_CFG of ni_top is
  for v
  end for; -- v
end ni_top_CFG;
-- pragma translate_on
